@@ -156,7 +156,7 @@ main()
 
       {
          hsize_t num_obj;
-         int i, obj_type;
+         int obj_type;
          char name[STR_LEN + 1];
          htri_t equal;
 
@@ -165,7 +165,7 @@ main()
          if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
          if ((osmonds_grpid = H5Gopen1(fileid, OSMONDS)) < 0) ERR;
          if (H5Gget_num_objs(osmonds_grpid, &num_obj) < 0) ERR;
-         for (i=0; i<num_obj; i++)
+         for (hsize_t i=0; i<num_obj; i++)
          {
             if (H5Gget_objname_by_idx(osmonds_grpid, i, name, STR_LEN+1) < 0) ERR;
             if ((obj_type = H5Gget_objtype_by_idx(osmonds_grpid, i)) < 0) ERR;
@@ -301,10 +301,10 @@ main()
       /* Initialize data. */
       for (i = 0; i < DIM6_LEN; i++)
       {
-	 obsdata[i].day = 15 * i + 1;
-	 obsdata[i].elev = 2 * i + 1;
+	 obsdata[i].day = (char)(15 * i + 1);
+	 obsdata[i].elev = (short)(2 * i + 1);
 	 obsdata[i].count = 2 * i + 1;
-	 obsdata[i].relhum = 2.0 * i + 1;
+	 obsdata[i].relhum = 2.0f * (float)i + 1;
 	 obsdata[i].time = 2.0 * i + 1;
       }
 
@@ -452,9 +452,9 @@ main()
          hr_data_out[i].starfleet_id = i;
          hr_data_out[i].svc_rec.i1 = 95;
          hr_data_out[i].svc_rec.i2 = 90;
-         if (sprintf(hr_data_out[i].name, "alien_%d", i) < 0) ERR;
-         hr_data_out[i].max_temp = 99.99;
-         hr_data_out[i].min_temp = -9.99;
+         if (snprintf(hr_data_out[i].name, sizeof(hr_data_out[i].name), "alien_%d", i) < 0) ERR;
+         hr_data_out[i].max_temp = 99.99f;
+         hr_data_out[i].min_temp = -9.99f;
          hr_data_out[i].percent_transporter_errosion = .1;
       }
 
@@ -557,9 +557,9 @@ main()
          hr_data_out[i].starfleet_id = i;
          hr_data_out[i].svc_rec.i1 = 95;
          hr_data_out[i].svc_rec.i2 = 90;
-         if (sprintf(hr_data_out[i].name, "alien_%d", i) < 0) ERR;
-         hr_data_out[i].max_temp = 99.99;
-         hr_data_out[i].min_temp = -9.99;
+         if (snprintf(hr_data_out[i].name, sizeof(hr_data_out[i].name), "alien_%d", i) < 0) ERR;
+         hr_data_out[i].max_temp = 99.99f;
+         hr_data_out[i].min_temp = -9.99f;
          hr_data_out[i].percent_transporter_errosion = .1;
       }
 
@@ -665,9 +665,9 @@ main()
          hr_data_out[i].starfleet_id = i;
          hr_data_out[i].svc_rec.i1 = 95;
          hr_data_out[i].svc_rec.i2 = 90;
-         if (sprintf(hr_data_out[i].name, "alien_%d", i) < 0) ERR;
-         hr_data_out[i].max_temp = 99.99;
-         hr_data_out[i].min_temp = -9.99;
+         if (snprintf(hr_data_out[i].name, sizeof(hr_data_out[i].name), "alien_%d", i) < 0) ERR;
+         hr_data_out[i].max_temp = 99.99f;
+         hr_data_out[i].min_temp = -9.99f;
          hr_data_out[i].percent_transporter_errosion = .1;
       }
 
@@ -756,8 +756,8 @@ main()
       /* Create some phony data. */
       for (i = 0; i < DIM1_LEN; i++)
       {
-         if (sprintf(hr_data_out[i].name, "alien_%d", i) < 0) ERR;
-         hr_data_out[i].max_temp = 99.99;
+         if (snprintf(hr_data_out[i].name, sizeof(hr_data_out[i].name), "alien_%d", i) < 0) ERR;
+         hr_data_out[i].max_temp = 99.99f;
       }
 
       /* Open file and get root group. */
@@ -841,8 +841,8 @@ main()
       /* Create some phony data. */
       for (i = 0; i < DIM2_LEN; i++)
       {
-         if (sprintf(hr_data_out[i].name, "alien_%d", i) < 0) ERR;
-         hr_data_out[i].max_temp = 99.99;
+         if (snprintf(hr_data_out[i].name, sizeof(hr_data_out[i].name), "alien_%d", i) < 0) ERR;
+         hr_data_out[i].max_temp = 99.99f;
       }
 
       /* Open file and get root group. */
@@ -921,10 +921,10 @@ main()
       {
 	 for (j = 0; j < STR_LEN + 1; j++)
 	    data_out[i].x[j] = 4;
-	 data_out[i].y = 99.99;
+	 data_out[i].y = 99.99f;
       }
 
-      /* Set latest_format in access propertly list and
+      /* Set latest_format in access property list and
        * H5P_CRT_ORDER_TRACKED in the creation property list. This
        * turns on HDF5 creation ordering. */
       if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
