@@ -41,7 +41,7 @@ main()
       char var_name[STR_LEN + 1];
       int v;
 
-      /* Create file, setting latest_format in access propertly list
+      /* Create file, setting latest_format in access property list
        * and H5P_CRT_ORDER_TRACKED in the creation property list. */
       if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
       if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -67,7 +67,7 @@ main()
       /* Create the variables. */
       for (v = 0; v < NUM_DATASETS; v++)
       {
-	 sprintf(var_name, "var_%d", v);
+	 snprintf(var_name, sizeof(var_name), "var_%d", v);
 /*	 printf("creating var %s\n", var_name);*/
 	 if ((datasetid[v] = H5Dcreate1(grpid, var_name, H5T_NATIVE_INT,
 				    spaceid, plistid)) < 0) ERR_RET;
