@@ -43,7 +43,7 @@ main()
       char name[MAX_SYMBOL_LEN + 1];
       ssize_t size;
 
-      /* Create file, setting latest_format in access propertly list
+      /* Create file, setting latest_format in access property list
        * and H5P_CRT_ORDER_TRACKED in the creation property list. */
       if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
       if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -85,7 +85,7 @@ main()
       if (H5Gget_num_objs(grpid, &num_obj) < 0) ERR;
       if (num_obj != NUM_ELEMENTS) ERR;
       printf("Original order:\n");
-      for (i = 0; i < num_obj; i++)
+      for (hsize_t i = 0; i < num_obj; i++)
       {
 #if H5_VERSION_GE(1,12,0)
 	 if (H5Oget_info_by_idx3(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC,
@@ -98,7 +98,7 @@ main()
 	 if ((size = H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
 					NULL, 0, H5P_DEFAULT)) < 0) ERR;
 	 H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
-				name, size+1, H5P_DEFAULT);
+                            name, (size_t)size+1, H5P_DEFAULT);
 	 if (strcmp(name, names[i])) ERR;
          printf("name %s\n", name);
       }
@@ -121,7 +121,7 @@ main()
       if (H5Gget_num_objs(grpid, &num_obj) < 0) ERR;
       if (num_obj != NUM_ELEMENTS) ERR;
       printf("New order:\n");
-      for (i = 0; i < num_obj; i++)
+      for (hsize_t i = 0; i < num_obj; i++)
       {
 #if H5_VERSION_GE(1,12,0)
          if (H5Oget_info_by_idx3(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC,
@@ -134,7 +134,7 @@ main()
          if ((size = H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
         				NULL, 0, H5P_DEFAULT)) < 0) ERR;
          H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
-        			name, size+1, H5P_DEFAULT);
+                            name, (size_t)size+1, H5P_DEFAULT);
          printf("name %s\n", name);
          /* if (strcmp(name, names2[i])) ERR; */
       }
