@@ -42,7 +42,7 @@ main()
         char name[MAX_SYMBOL_LEN + 1];
         ssize_t size;
 
-        /* Create file, setting latest_format in access propertly list
+        /* Create file, setting latest_format in access property list
          * and H5P_CRT_ORDER_TRACKED in the creation property list. */
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -63,7 +63,7 @@ main()
         /* Create the variables, one per element. */
         for (i = 0; i < NUM_ELEMENTS; i++)
         {
-            if ((did[i] = H5Dcreate(grpid, names[i], H5T_NATIVE_INT,
+            if ((did[i] = H5Dcreate1(grpid, names[i], H5T_NATIVE_INT,
                                     spaceid, H5P_DEFAULT)) < 0) ERR;
             if (H5Dclose(did[i]) < 0) ERR;
         }
@@ -79,11 +79,11 @@ main()
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, ELEMENTS_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, ELEMENTS_NAME)) < 0) ERR;
 
         if (H5Gget_num_objs(grpid, &num_obj) < 0) ERR;
         if (num_obj != NUM_ELEMENTS) ERR;
-        for (i = 0; i < num_obj; i++)
+        for (hsize_t i = 0; i < num_obj; i++)
         {
 #if H5_VERSION_GE(1,12,0)
             if (H5Oget_info_by_idx3(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC,
@@ -96,7 +96,7 @@ main()
             if ((size = H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
                                            NULL, 0, H5P_DEFAULT)) < 0) ERR;
             H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
-                               name, size+1, H5P_DEFAULT);
+                               name, (size_t)size+1, H5P_DEFAULT);
             if (strcmp(name, names[i])) ERR;
         }
         if (H5Pclose(fapl_id) < 0 ||
@@ -130,7 +130,7 @@ main()
         hsize_t dimscale_dims[1] = {DIM1_LEN};
         ssize_t size;
 
-        /* Create file, setting latest_format in access propertly list
+        /* Create file, setting latest_format in access property list
          * and H5P_CRT_ORDER_TRACKED in the creation property list. */
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -145,7 +145,7 @@ main()
         {
             /* Create our dimension scale. Use the built-in NAME attribute
              * on the dimscale. */
-            if ((dimscaleid = H5Dcreate(fileid, names[i], H5T_NATIVE_INT,
+            if ((dimscaleid = H5Dcreate1(fileid, names[i], H5T_NATIVE_INT,
                                         dimscale_spaceid, H5P_DEFAULT)) < 0) ERR;
             if (H5DSset_scale(dimscaleid, NAME_ATTRIBUTE) < 0) ERR;
 
@@ -162,11 +162,11 @@ main()
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
         if (H5Gget_num_objs(grpid, &num_obj) < 0) ERR;
         if (num_obj != NUM_DIMSCALES) ERR;
-        for (i = 0; i < num_obj; i++)
+        for (hsize_t i = 0; i < num_obj; i++)
         {
 #if H5_VERSION_GE(1,12,0)
             if (H5Oget_info_by_idx3(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC,
@@ -179,7 +179,7 @@ main()
             if ((size = H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
                                            NULL, 0, H5P_DEFAULT)) < 0) ERR;
             if (H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, i,
-                                   name, size+1, H5P_DEFAULT) < 0) ERR;
+                                   name, (size_t)size+1, H5P_DEFAULT) < 0) ERR;
             if (strcmp(name, names[i])) ERR;
         }
         if (H5Pclose(fapl_id) < 0 ||
@@ -199,7 +199,7 @@ main()
         hid_t did, fapl_id, fcpl_id, gcpl_id, attid;
         hsize_t num_obj;
         hid_t fileid, grpid, spaceid;
-        float val = 3.1495;
+        float val = 3.1495f;
 #if H5_VERSION_GE(1,12,0)
         H5O_info2_t obj_info;
 #else
@@ -208,7 +208,7 @@ main()
         char name[MAX_NAME_LEN + 1];
         ssize_t size;
 
-        /* Create file, setting latest_format in access propertly list
+        /* Create file, setting latest_format in access property list
          * and H5P_CRT_ORDER_TRACKED in the creation property list. */
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -227,7 +227,7 @@ main()
         if ((spaceid = H5Screate(H5S_SCALAR)) < 0) ERR;
 
         /* Create a scalar variable. */
-        if ((did = H5Dcreate(grpid, VAR_NAME, H5T_NATIVE_INT,
+        if ((did = H5Dcreate1(grpid, VAR_NAME, H5T_NATIVE_INT,
                              spaceid, H5P_DEFAULT)) < 0) ERR;
         if (H5Dclose(did) < 0) ERR;
 
@@ -238,11 +238,11 @@ main()
         if (H5Gunlink(grpid, VAR_NAME) < 0) ERR;
 
         /* Re-reate the scalar variable. */
-        if ((did = H5Dcreate(grpid, VAR_NAME, H5T_NATIVE_INT,
+        if ((did = H5Dcreate1(grpid, VAR_NAME, H5T_NATIVE_INT,
                              spaceid, H5P_DEFAULT)) < 0) ERR;
 
         /* Add an attribute. */
-        if ((attid = H5Acreate(did, "Some_Attribute", H5T_NATIVE_FLOAT, spaceid,
+        if ((attid = H5Acreate1(did, "Some_Attribute", H5T_NATIVE_FLOAT, spaceid,
                                H5P_DEFAULT)) < 0) ERR;
         if (H5Awrite(attid, H5T_NATIVE_FLOAT, &val) < 0) ERR;
 
@@ -260,7 +260,7 @@ main()
         if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
         if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, ELEMENTS_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, ELEMENTS_NAME)) < 0) ERR;
 
         if (H5Gget_num_objs(grpid, &num_obj) < 0) ERR;
         if (num_obj != 1) ERR;
@@ -276,7 +276,7 @@ main()
         if ((size = H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, 0,
                                        NULL, 0, H5P_DEFAULT)) < 0) ERR;
         H5Lget_name_by_idx(grpid, ".", H5_INDEX_CRT_ORDER, H5_ITER_INC, 0,
-                           name, size+1, H5P_DEFAULT);
+                           name, (size_t)size+1, H5P_DEFAULT);
         if (strcmp(name, VAR_NAME)) ERR;
         if (H5Pclose(fapl_id) < 0 ||
             H5Gclose(grpid) < 0 ||
@@ -310,7 +310,7 @@ main()
 	    for (y = 0; y < NY; y++)
 		data_out[x][y] = x * NY + y;
 
-	/* Create file, setting latest_format in access propertly list
+	/* Create file, setting latest_format in access property list
 	 * and H5P_CRT_ORDER_TRACKED in the creation property list. */
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -318,7 +318,7 @@ main()
 	if (H5Pset_link_creation_order(fcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0) ERR;
 	if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, fcpl_id, fapl_id)) < 0) ERR;
 
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	dimsize[0] = maxdimsize[0] = NX;
 	dimsize[1] = maxdimsize[1] = NY;
@@ -336,7 +336,7 @@ main()
 	if (H5Pset_deflate(plistid, DEFLATE_LEVEL) < 0) ERR;
 
 	/* Create the variable. */
-	if ((datasetid = H5Dcreate(grpid, SIMPLE_VAR_NAME, H5T_NATIVE_INT,
+	if ((datasetid = H5Dcreate1(grpid, SIMPLE_VAR_NAME, H5T_NATIVE_INT,
 				   spaceid, plistid)) < 0) ERR;
 
 	/* Write the data. */
@@ -354,7 +354,7 @@ main()
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
 	if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if ((datasetid = H5Dopen1(grpid, SIMPLE_VAR_NAME)) < 0) ERR;
 	if ((spaceid = H5Dget_space(datasetid)) < 0)
@@ -408,7 +408,7 @@ main()
 
 	/* Open file and read fill value of the variable. */
 	if ((fileid = H5Fopen(file_in, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 	if ((datasetid = H5Dopen1(grpid, VAR_NAME2)) < 0) ERR;
 	if ((propid = H5Dget_create_plist(datasetid)) < 0) ERR;
 	if (H5Pfill_value_defined(propid, &fill_status) < 0) ERR;
@@ -452,7 +452,7 @@ main()
 	char var_name[STR_LEN + 1];
 	int v;
 
-	/* Create file, setting latest_format in access propertly list
+	/* Create file, setting latest_format in access property list
 	 * and H5P_CRT_ORDER_TRACKED in the creation property list. */
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -461,7 +461,7 @@ main()
 	if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, fcpl_id, fapl_id)) < 0) ERR;
 
 	/* Open root group. */
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	/* Create 1 D data space with unlimited dimension. */
 	dimsize[0] = 0;
@@ -478,9 +478,9 @@ main()
 	/* Create the variables. */
 	for (v = 0; v < NUM_DATASETS; v++)
 	{
-	    sprintf(var_name, "var_%d", v);
+	    snprintf(var_name, sizeof(var_name), "var_%d", v);
 	    /*       printf("creating var %s\n", var_name);*/
-	    if ((datasetid[v] = H5Dcreate(grpid, var_name, H5T_NATIVE_INT,
+	    if ((datasetid[v] = H5Dcreate1(grpid, var_name, H5T_NATIVE_INT,
 					  spaceid, plistid)) < 0) ERR_RET;
 	}
 
@@ -499,7 +499,7 @@ main()
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
 	if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if (H5Pclose(fapl_id) < 0 ||
 	    H5Gclose(grpid) < 0 ||
@@ -517,7 +517,7 @@ main()
 	hid_t fileid, grpid, spaceid, plistid;
 	hsize_t chunksize = 1;
 
-	/* Create file, setting latest_format in access propertly list
+	/* Create file, setting latest_format in access property list
 	 * and H5P_CRT_ORDER_TRACKED in the creation property list. */
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -525,7 +525,7 @@ main()
 	if (H5Pset_link_creation_order(fcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0) ERR;
 	if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, fcpl_id, fapl_id)) < 0) ERR;
 
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if ((spaceid = H5Screate(H5S_SCALAR)) < 0) ERR;
 
@@ -544,12 +544,12 @@ main()
 
 	/* Create the variable. This will not work, because only
 	 * chunked datasets can use filters, and scalars can't be
-	 * chunked. The H5Dcreate() call will fail. */
-	if ((datasetid = H5Dcreate(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
+	 * chunked. The H5Dcreate1() call will fail. */
+	if ((datasetid = H5Dcreate1(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
 				   spaceid, plistid)) > 0) ERR;
 
 	/* Turn on error messages back on. */
-	H5Eset_auto2(H5E_DEFAULT, (H5E_auto2_t)&H5Eprint, stderr);
+	H5Eset_auto2(H5E_DEFAULT, (H5E_auto2_t)&H5Eprint1, stderr);
 
 	if (H5Pclose(fapl_id) < 0 ||
 	    H5Sclose(spaceid) < 0 ||
@@ -568,7 +568,7 @@ main()
 	int data_in, data_out = 42;
 	H5D_layout_t layout;
 
-	/* Create file, setting latest_format in access propertly list
+	/* Create file, setting latest_format in access property list
 	 * and H5P_CRT_ORDER_TRACKED in the creation property list. */
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -576,7 +576,7 @@ main()
 	if (H5Pset_link_creation_order(fcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0) ERR;
 	if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, fcpl_id, fapl_id)) < 0) ERR;
 
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if ((spaceid = H5Screate(H5S_SCALAR)) < 0) ERR;
 
@@ -586,7 +586,7 @@ main()
 	if (H5Pset_layout(plistid, H5D_COMPACT) < 0)ERR;
 
 	/* Create the variable. */
-	if ((datasetid = H5Dcreate(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
+	if ((datasetid = H5Dcreate1(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
 				   spaceid, plistid)) < 0) ERR;
 
 	/* Write the data. */
@@ -607,7 +607,7 @@ main()
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
 	if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, fapl_id)) < 0) ERR;
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if ((datasetid = H5Dopen1(grpid, SIMPLE_VAR_NAME1)) < 0) ERR;
 	if ((spaceid = H5Dget_space(datasetid)) < 0) ERR;
@@ -635,7 +635,7 @@ main()
 	hid_t fileid, grpid, spaceid, plistid;
 	hsize_t chunksize = 1;
 
-	/* Create file, setting latest_format in access propertly list
+	/* Create file, setting latest_format in access property list
 	 * and H5P_CRT_ORDER_TRACKED in the creation property list. */
 	if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
 	if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) ERR;
@@ -643,7 +643,7 @@ main()
 	if (H5Pset_link_creation_order(fcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0) ERR;
 	if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, fcpl_id, fapl_id)) < 0) ERR;
 
-	if ((grpid = H5Gopen(fileid, "/")) < 0) ERR;
+	if ((grpid = H5Gopen1(fileid, "/")) < 0) ERR;
 
 	if ((spaceid = H5Screate(H5S_SCALAR)) < 0) ERR;
 
@@ -659,12 +659,12 @@ main()
 
 	/* Create the variable. This will not work, because only
 	 * chunked datasets can use filters, and scalars can't be
-	 * chunked. The H5Dcreate() call will fail. */
-	if ((datasetid = H5Dcreate(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
+	 * chunked. The H5Dcreate1() call will fail. */
+	if ((datasetid = H5Dcreate1(grpid, SIMPLE_VAR_NAME1, H5T_NATIVE_INT,
 				   spaceid, plistid)) > 0) ERR;
 
 	/* Turn on error messages back on. */
-	H5Eset_auto2(H5E_DEFAULT, (H5E_auto2_t)&H5Eprint, stderr);
+	H5Eset_auto2(H5E_DEFAULT, (H5E_auto2_t)&H5Eprint1, stderr);
 
 	if (H5Pclose(fapl_id) < 0 ||
 	    H5Sclose(spaceid) < 0 ||
