@@ -13,16 +13,14 @@
 #include "netcdf.h"
 
 #include "config.h"
-#include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
 
-#include "ncdimscale.h"
 #include "nc_logging.h"
 #include "ncindex.h"
 #include "nc_provenance.h"
-
+#include "nchashmap.h"
 
 #include "netcdf_f.h"
 #include "netcdf_mem.h"
@@ -33,6 +31,7 @@
 
 /* Always needed */
 #include "nc.h"
+#include "ncglobal.h"
 
 /** The file ID is stored in the first two bytes of ncid. */
 #define FILE_ID_MASK (0xffff0000)
@@ -45,7 +44,7 @@
 
 /* typedef enum {GET, PUT} NC_PG_T; */
 /** These are the different objects that can be in our hash-lists. */
-typedef enum {NCNAT, NCVAR, NCDIM, NCATT, NCTYP, NCFLD, NCGRP} NC_SORT;
+typedef enum {NCNAT, NCVAR, NCDIM, NCATT, NCTYP, NCFLD, NCGRP, NCFIL} NC_SORT;
 
 /** The netCDF V2 error code. */
 #define NC_V2_ERR (-1)
@@ -56,9 +55,6 @@ typedef enum {NCNAT, NCVAR, NCDIM, NCATT, NCTYP, NCFLD, NCGRP} NC_SORT;
 /** One mega-byte. */
 #define MEGABYTE 1048576
 
-/** The HDF5 ID for the szip filter. */
-#define HDF5_FILTER_SZIP 4
-
 #define X_SCHAR_MIN     (-128)          /**< Minimum signed char value. */
 #define X_SCHAR_MAX     127             /**< Maximum signed char value. */
 #define X_UCHAR_MAX     255U            /**< Maximum unsigned char value. */
@@ -85,9 +81,6 @@ typedef enum {NCNAT, NCVAR, NCDIM, NCATT, NCTYP, NCFLD, NCGRP} NC_SORT;
 #define X_DOUBLE_MAX    1.7976931348623157e+308 /**< Maximum double value. */
 #define X_DOUBLE_MIN    (-X_DOUBLE_MAX)         /**< Minimum double value. */
 
-/** This is the number of netCDF atomic types. */
-#define NUM_ATOMIC_TYPES (NC_MAX_ATOMIC_TYPE + 1)
-
 /** Number of parameters needed for ZLIB filter. */
 #define CD_NELEMS_ZLIB 1
 
@@ -98,27 +91,16 @@ typedef enum {NCNAT, NCVAR, NCDIM, NCATT, NCTYP, NCFLD, NCGRP} NC_SORT;
 #define NC4_DATA_SET(nc,data) ((nc)->dispatchdata = (void *)(data))
 
 /* Reserved attribute flags: must be powers of 2. */
-/** Hidden dimscale-related, per-variable attributes; immutable and
- * unreadable thru API. */
-#define DIMSCALEFLAG 1
-
-/** Readonly global attributes; readable, but immutable thru the
- * API. */
-#define READONLYFLAG 2
-
-/** Subset of readonly flags; readable by name only thru the API. */
-#define NAMEONLYFLAG 4
-
-/** Subset of readonly flags; Value is actually in file. */
-#define MATERIALIZEDFLAG 8
-
-/* Generic reserved Attributes */
-#define NC_ATT_REFERENCE_LIST "REFERENCE_LIST" /**< HDF5 reference list attribute name. */
-#define NC_ATT_CLASS "CLASS"                   /**< HDF5 class atttribute name. */
-#define NC_ATT_DIMENSION_LIST "DIMENSION_LIST" /**< HDF5 dimension list attribute name. */
-#define NC_ATT_NAME "NAME"                     /**< HDF5 name atttribute name. */
-#define NC_ATT_COORDINATES COORDINATES         /**< Coordinates atttribute name. */
-#define NC_ATT_FORMAT "_Format"                /**< Format atttribute name. */
+    /** Hidden attributes; immutable and unreadable thru API. */
+#   define HIDDENATTRFLAG 1
+    /** Readonly attributes; readable, but immutable thru the API. */
+#   define READONLYFLAG 2
+    /** Subset of readonly flags; readable by name only thru the API. */
+#   define NAMEONLYFLAG 4
+    /** Mark reserved attributes that are constructed on the fly when accessed */
+#   define VIRTUALFLAG 8
+    /** Per-variable attribute, as opposed to global */
+#   define VARFLAG 16
 
 /** Boolean type, to make the code easier to read. */
 typedef enum {NC_FALSE = 0, NC_TRUE = 1} nc_bool_t;
@@ -126,7 +108,7 @@ typedef enum {NC_FALSE = 0, NC_TRUE = 1} nc_bool_t;
 /* Forward declarations. */
 struct NC_GRP_INFO;
 struct NC_TYPE_INFO;
-struct NC_FIlterobject;
+struct NCRCinfo;
 
 /**
  * This struct provides indexed Access to Meta-data objects. See the
@@ -142,8 +124,7 @@ typedef struct NC_OBJ
 {
     NC_SORT sort; /**< Type of object. */
     char* name;   /**< Name, assumed to be null terminated. */
-    size_t id;    /**< This objects ID. */
-    unsigned int hashkey; /**< The hash key, crc32(name). */
+    int id;    /**< This objects ID. */
 } NC_OBJ;
 
 /**
@@ -174,21 +155,19 @@ typedef struct NC_ATT_INFO
 {
     NC_OBJ hdr;             /**< The hdr contains the name and ID. */
     struct NC_OBJ *container;    /**< Pointer to containing group|var. */
-    int len;                /**< Length of attribute data. */
+    size_t len;             /**< Length of attribute data. */
     nc_bool_t dirty;        /**< True if attribute modified. */
     nc_bool_t created;      /**< True if attribute already created. */
     nc_type nc_typeid;      /**< NetCDF type of attribute's data. */
     void *format_att_info;  /**< Pointer to format-specific att info. */
     void *data;             /**< The attribute data. */
-    nc_vlen_t *vldata;      /**< VLEN data (only used for vlen types). */
-    char **stdata;          /**< String data (only for string type). */
 } NC_ATT_INFO_T;
 
 /** This is a struct to handle the var metadata. */
 typedef struct NC_VAR_INFO
 {
     NC_OBJ hdr;                  /**< The hdr contains the name and ID. */
-    char *hdf5_name;             /**< Used if name in HDF5 must be different from name. */
+    char *alt_name;              /**< Used if name in dispatcher must be different from hdr.name. */
     struct NC_GRP_INFO *container; /**< Pointer to containing group. */
     size_t ndims;                /**< Number of dims. */
     int *dimids;                 /**< Dim IDs. */
@@ -209,16 +188,13 @@ typedef struct NC_VAR_INFO
     void *fill_value;            /**< Pointer to fill value, or NULL. */
     size_t *chunksizes;          /**< For chunked storage, an array (size ndims) of chunksizes. */
     int storage;                 /**< Storage of this var, compact, contiguous, or chunked. */
+    int endianness;              /**< What endianness for the var? */
     int parallel_access;         /**< Type of parallel access for I/O on variable (collective or independent). */
-    nc_bool_t dimscale;          /**< True if var is a dimscale. */
-    nc_bool_t *dimscale_attached;  /**< Array of flags that are true if dimscale is attached for that dim index. */
-    nc_bool_t shuffle;           /**< True if var has shuffle filter applied. */
-    nc_bool_t fletcher32;        /**< True if var has fletcher32 filter applied. */
-    size_t chunk_cache_size;     /**< Size in bytes of the var chunk chache. */
-    size_t chunk_cache_nelems;   /**< Number of slots in var chunk cache. */
-    float chunk_cache_preemption; /**< Chunk cache preemtion policy. */
+    struct ChunkCache chunkcache; /* ChunkCache now defined in ncglobal.h */
+    int quantize_mode;           /**< Quantize mode. NC_NOQUANTIZE is 0, and means no quantization. */
+    int nsd;                     /**< Number of significant digits if quantization is used, 0 if not. */
     void *format_var_info;       /**< Pointer to any binary format info. */
-    NClist* filters;             /**< List of filters to be applied to var data.  */
+    void* filters;             /**< Record of the list of filters to be applied to var data; format dependent */
 } NC_VAR_INFO_T;
 
 /** This is a struct to handle the field metadata from a user-defined
@@ -251,7 +227,8 @@ typedef struct NC_TYPE_INFO
     size_t size;                 /**< Size of the type in memory, in bytes */
     nc_bool_t committed;         /**< True when datatype is committed in the file */
     nc_type nc_type_class;       /**< NC_VLEN, NC_COMPOUND, NC_OPAQUE, NC_ENUM, NC_INT, NC_FLOAT, or NC_STRING. */
-    void *format_type_info;      /**< HDF5-specific type info. */
+    void *format_type_info;      /**< dispatcher-specific type info. */
+    int varsized; 	         /**< <! 1 if this type is (recursively) variable sized; 0 if fixed size */
 
     /** Information for each type or class */
     union {
@@ -285,28 +262,27 @@ typedef struct NC_GRP_INFO
     NCindex* vars;               /**< NCindex<NC_VAR_INFO_T> * */
 } NC_GRP_INFO_T;
 
-/* These constants apply to the cmode parameter in the
- * HDF5_FILE_INFO_T defined below. */
-#define NC_CREAT 2      /**< in create phase, cleared by ncendef */
-#define NC_INDEF 8      /**< in define mode, cleared by ncendef */
-#define NC_NSYNC 0x10   /**< synchronise numrecs on change */
-#define NC_HSYNC 0x20   /**< synchronise whole header on change */
-#define NC_NDIRTY 0x40  /**< numrecs has changed */
-#define NC_HDIRTY 0x80  /**< header info has changed */
+/* These constants apply to the flags field in the
+ * NC_FILE_INFO_T defined below. */
+#define NC_INDEF  0x01  /**< in define mode, cleared by ncendef */
 
 /** This is the metadata we need to keep track of for each
- * netcdf-4/HDF5 file. */
-typedef struct  NC_FILE_INFO
+  * netcdf-4/ file; used by libhdf5, libnczarr, and libdap4 */
+
+typedef struct NC_FILE_INFO
 {
+    NC_OBJ hdr;
     NC *controller; /**< Pointer to containing NC. */
-#ifdef USE_PARALLEL4
+#ifdef USE_PARALLEL
     MPI_Comm comm;  /**< Copy of MPI Communicator used to open the file. */
     MPI_Info info;  /**< Copy of MPI Information Object used to open the file. */
 #endif
-    int flags;      /**< Flags used to open the file. */
-    int cmode;      /**< Create mode used to create the file. */
+    int cmode;      /**< Create/Open mode for the file. */
+    int flags;      /**< State transition flags . */
     nc_bool_t parallel;   /**< True if file is open for parallel access */
     nc_bool_t redef;      /**< True if redefining an existing file */
+    nc_bool_t no_attr_create_order; /**< True if the creation order tracking of attributes is disabled (netcdf-4 only) */
+    nc_bool_t no_dimscale_attach; /**< True if attaching dimscales to variables is disabled (netcdf-4 only) */
     int fill_mode;        /**< Fill mode for vars - Unused internally currently */
     nc_bool_t no_write;   /**< true if nc_open has mode NC_NOWRITE. */
     NC_GRP_INFO_T *root_grp; /**< Pointer to root group. */
@@ -344,162 +320,163 @@ typedef struct
     void *p;    /**< Pointer to VL data */
 } nc_hvl_t;
 
-/** The names of the atomic data types. */
-extern const char *nc4_atomic_name[NC_MAX_ATOMIC_TYPE + 1];
+/* These functions convert between different netcdf types. */
+extern int nc4_get_typelen_mem(NC_FILE_INFO_T *h5, nc_type xtype, size_t *len);
+extern int nc4_convert_type(const void *src, void *dest, const nc_type src_type,
+			    const nc_type dest_type, const size_t len, int *range_error,
+			    const void *fill_value, int strict_nc3, int quantize_mode,
+			    int nsd);
 
-/* These functions convert between netcdf and HDF5 types. */
-int nc4_get_typelen_mem(NC_FILE_INFO_T *h5, nc_type xtype, size_t *len);
-int nc4_convert_type(const void *src, void *dest, const nc_type src_type,
-                     const nc_type dest_type, const size_t len, int *range_error,
-                     const void *fill_value, int strict_nc3);
-
-/* These functions do HDF5 things. */
-int nc4_reopen_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
-int nc4_read_atts(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
+/* These functions do netcdf-4 things. */
+extern int nc4_reopen_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
+extern int nc4_read_atts(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
 
 /* Find items in the in-memory lists of metadata. */
-int nc4_find_nc_grp_h5(int ncid, NC **nc, NC_GRP_INFO_T **grp,
+extern int nc4_find_nc_grp_h5(int ncid, NC **nc, NC_GRP_INFO_T **grp,
                        NC_FILE_INFO_T **h5);
-int nc4_find_grp_h5(int ncid, NC_GRP_INFO_T **grp, NC_FILE_INFO_T **h5);
-int nc4_find_nc4_grp(int ncid, NC_GRP_INFO_T **grp);
-int nc4_find_dim(NC_GRP_INFO_T *grp, int dimid, NC_DIM_INFO_T **dim,
+extern int nc4_find_grp_h5(int ncid, NC_GRP_INFO_T **grp, NC_FILE_INFO_T **h5);
+extern int nc4_find_nc4_grp(int ncid, NC_GRP_INFO_T **grp);
+extern int nc4_find_dim(NC_GRP_INFO_T *grp, int dimid, NC_DIM_INFO_T **dim,
                  NC_GRP_INFO_T **dim_grp);
-int nc4_find_var(NC_GRP_INFO_T *grp, const char *name, NC_VAR_INFO_T **var);
-int nc4_find_dim_len(NC_GRP_INFO_T *grp, int dimid, size_t **len);
-int nc4_find_type(const NC_FILE_INFO_T *h5, int typeid1, NC_TYPE_INFO_T **type);
-NC_TYPE_INFO_T *nc4_rec_find_named_type(NC_GRP_INFO_T *start_grp, char *name);
-NC_TYPE_INFO_T *nc4_rec_find_equal_type(NC_GRP_INFO_T *start_grp, int ncid1,
+extern int nc4_find_var(NC_GRP_INFO_T *grp, const char *name, NC_VAR_INFO_T **var);
+extern int nc4_find_dim_len(NC_GRP_INFO_T *grp, int dimid, size_t **len);
+extern int nc4_find_type(const NC_FILE_INFO_T *h5, int typeid1, NC_TYPE_INFO_T **type);
+extern NC_TYPE_INFO_T *nc4_rec_find_named_type(NC_GRP_INFO_T *start_grp, char *name);
+extern NC_TYPE_INFO_T *nc4_rec_find_equal_type(NC_GRP_INFO_T *start_grp, int ncid1,
                                         NC_TYPE_INFO_T *type);
-int nc4_find_nc_att(int ncid, int varid, const char *name, int attnum,
+extern int nc4_find_nc_att(int ncid, int varid, const char *name, int attnum,
                     NC_ATT_INFO_T **att);
-int nc4_find_grp_h5_var(int ncid, int varid, NC_FILE_INFO_T **h5,
+extern int nc4_find_grp_h5_var(int ncid, int varid, NC_FILE_INFO_T **h5,
                         NC_GRP_INFO_T **grp, NC_VAR_INFO_T **var);
-int nc4_find_grp_att(NC_GRP_INFO_T *grp, int varid, const char *name,
+extern int nc4_find_grp_att(NC_GRP_INFO_T *grp, int varid, const char *name,
                      int attnum, NC_ATT_INFO_T **att);
-int nc4_get_typeclass(const NC_FILE_INFO_T *h5, nc_type xtype,
+extern int nc4_get_typeclass(const NC_FILE_INFO_T *h5, nc_type xtype,
                       int *type_class);
 
 /* Free various types */
-int nc4_type_free(NC_TYPE_INFO_T *type);
+extern int nc4_type_free(NC_TYPE_INFO_T *type);
 
 /* These list functions add and delete vars, atts. */
-int nc4_nc4f_list_add(NC *nc, const char *path, int mode);
-int nc4_nc4f_list_del(NC_FILE_INFO_T *h5);
-int nc4_file_list_add(int ncid, const char *path, int mode,
+extern int nc4_nc4f_list_add(NC *nc, const char *path, int mode);
+extern int nc4_nc4f_list_del(NC_FILE_INFO_T *h5);
+extern int nc4_file_list_add(int ncid, const char *path, int mode,
                       void **dispatchdata);
-int nc4_file_list_get(int ncid, char **path, int *mode,
+extern int nc4_file_list_get(int ncid, char **path, int *mode,
                       void **dispatchdata);
-int nc4_file_list_del(int ncid);
-int nc4_file_change_ncid(int ncid, unsigned short new_ncid_index);
-int nc4_var_list_add(NC_GRP_INFO_T* grp, const char* name, int ndims,
+extern int nc4_file_list_del(int ncid);
+extern int nc4_file_change_ncid(int ncid, unsigned short new_ncid_index);
+extern int nc4_var_list_add(NC_GRP_INFO_T* grp, const char* name, int ndims,
                      NC_VAR_INFO_T **var);
-int nc4_var_list_add2(NC_GRP_INFO_T* grp, const char* name,
+extern int nc4_var_list_add2(NC_GRP_INFO_T* grp, const char* name,
                       NC_VAR_INFO_T **var);
-int nc4_var_set_ndims(NC_VAR_INFO_T *var, int ndims);
-int nc4_var_list_del(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
-int nc4_dim_list_add(NC_GRP_INFO_T *grp, const char *name, size_t len,
+extern int nc4_var_set_ndims(NC_VAR_INFO_T *var, int ndims);
+extern int nc4_var_list_del(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
+extern int nc4_dim_list_add(NC_GRP_INFO_T *grp, const char *name, size_t len,
                      int assignedid, NC_DIM_INFO_T **dim);
-int nc4_dim_list_del(NC_GRP_INFO_T *grp, NC_DIM_INFO_T *dim);
-int nc4_type_new(size_t size, const char *name, int assignedid,
+extern int nc4_dim_list_del(NC_GRP_INFO_T *grp, NC_DIM_INFO_T *dim);
+extern int nc4_type_new(size_t size, const char *name, int assignedid,
                  NC_TYPE_INFO_T **type);
-int nc4_type_list_add(NC_GRP_INFO_T *grp, size_t size, const char *name,
+extern int nc4_type_list_add(NC_GRP_INFO_T *grp, size_t size, const char *name,
                       NC_TYPE_INFO_T **type);
-int nc4_type_list_del(NC_GRP_INFO_T *grp, NC_TYPE_INFO_T *type);
-int nc4_type_free(NC_TYPE_INFO_T *type);
-int nc4_field_list_add(NC_TYPE_INFO_T* parent, const char *name,
+extern int nc4_type_list_del(NC_GRP_INFO_T *grp, NC_TYPE_INFO_T *type);
+extern int nc4_type_free(NC_TYPE_INFO_T *type);
+extern int nc4_field_list_add(NC_TYPE_INFO_T* parent, const char *name,
                        size_t offset, nc_type xtype, int ndims,
                        const int *dim_sizesp);
-int nc4_att_list_add(NCindex *list, const char *name, NC_ATT_INFO_T **att);
-int nc4_att_list_del(NCindex *list, NC_ATT_INFO_T *att);
-int nc4_grp_list_add(NC_FILE_INFO_T *h5, NC_GRP_INFO_T *parent, char *name,
+extern int nc4_att_list_add(NCindex *list, const char *name, NC_ATT_INFO_T **att);
+extern int nc4_att_list_del(NCindex *list, NC_ATT_INFO_T *att);
+extern int nc4_grp_list_add(NC_FILE_INFO_T *h5, NC_GRP_INFO_T *parent, char *name,
                      NC_GRP_INFO_T **grp);
-int nc4_build_root_grp(NC_FILE_INFO_T *h5);
-int nc4_rec_grp_del(NC_GRP_INFO_T *grp);
-int nc4_enum_member_add(NC_TYPE_INFO_T *type, size_t size, const char *name,
+extern int nc4_build_root_grp(NC_FILE_INFO_T *h5);
+extern int nc4_rec_grp_del(NC_GRP_INFO_T *grp);
+extern int nc4_enum_member_add(NC_TYPE_INFO_T *type, size_t size, const char *name,
                         const void *value);
+extern int nc4_att_free(NC_ATT_INFO_T *att);
 
 /* Check and normalize names. */
-int NC_check_name(const char *name);
-int nc4_check_name(const char *name, char *norm_name);
-int nc4_normalize_name(const char *name, char *norm_name);
-int nc4_check_dup_name(NC_GRP_INFO_T *grp, char *norm_name);
+extern int NC_check_name(const char *name);
+extern int nc4_check_name(const char *name, char *norm_name);
+extern int nc4_normalize_name(const char *name, char *norm_name);
+extern int nc4_check_dup_name(NC_GRP_INFO_T *grp, char *norm_name);
 
-/* Find default fill value. */
-int nc4_get_default_fill_value(const NC_TYPE_INFO_T *type_info, void *fill_value);
+/* Get the fill value for a var. */
+extern int nc4_get_fill_value(NC_FILE_INFO_T *h5, NC_VAR_INFO_T *var, void **fillp);
+
+/* Find default fill value for atomic type. */
+extern int nc4_get_default_atomic_fill_value(nc_type, void *fill_value);
+
+/* Find default fill value for any type */
+extern int nc4_get_default_fill_value(NC_TYPE_INFO_T*, void *fill_value);
 
 /* Get an att given pointers to file, group, and perhaps ver info. */
-int nc4_get_att_ptrs(NC_FILE_INFO_T *h5, NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var,
+extern int nc4_get_att_ptrs(NC_FILE_INFO_T *h5, NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var,
                      const char *name, nc_type *xtype, nc_type mem_type,
                      size_t *lenp, int *attnum, void *data);
 
-/* Close the file. */
-int nc4_close_netcdf4_file(NC_FILE_INFO_T *h5, int abort, NC_memio *memio);
+/* Get variable/fixed size flag for type (ncid API level)*/
+extern int NC4_inq_type_fixed_size(int ncid, nc_type xtype, int* isfixedsizep);
+/* Manage the fixed/var sized'ness of a type */
+extern int NC4_recheck_varsize(NC_TYPE_INFO_T* parenttype, nc_type addedtype);
+extern int NC4_set_varsize(NC_TYPE_INFO_T* parenttype);
+extern int NC4_var_varsized(NC_VAR_INFO_T* var);
 
-/* HDF5 initialization/finalization */
-extern int nc4_hdf5_initialized;
-extern void nc4_hdf5_initialize(void);
-extern void nc4_hdf5_finalize(void);
+/* Close the file. */
+extern int nc4_close_netcdf4_file(NC_FILE_INFO_T *h5, int abort, NC_memio *memio);
+
+/* Compute default chunksizes */
+extern int nc4_find_default_chunksizes2(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
+extern int nc4_check_chunksizes(NC_GRP_INFO_T* grp, NC_VAR_INFO_T* var, const size_t* chunksizes);
 
 /* This is only included if --enable-logging is used for configure; it
    prints info about the metadata to stderr. */
 #ifdef LOGGING
-int log_metadata_nc(NC_FILE_INFO_T *h5);
+extern int log_metadata_nc(NC_FILE_INFO_T *h5);
 #endif
 
-/* Binary searcher for reserved attributes */
-extern const NC_reservedatt *NC_findreserved(const char *name);
+/**************************************************/
+/* Atomic types constants and functions */
+
+/** This is the number of netCDF atomic types (as opposed to max) . */
+#define NUM_ATOMIC_TYPES (NC_MAX_ATOMIC_TYPE + 1)
+
+/** @internal Names of atomic types. */
+extern const char* nc4_atomic_name[NUM_ATOMIC_TYPES];
+
+/* Misc functions */
+extern int NC4_inq_atomic_type(nc_type typeid1, char *name, size_t *size);
+extern int NC4_lookup_atomic_type(const char *name, nc_type* idp, size_t *sizep);
+extern int NC4_inq_atomic_typeid(int ncid, const char *name, nc_type *typeidp);
+extern int NC4_get_atomic_typeclass(nc_type xtype, int *type_class);
 
 /**************************************************/
-/* Internal filter related structures */
+/* Type alignment related functions */
 
-/* Internal filter actions */
-#define NCFILTER_DEF		1
-#define NCFILTER_REMOVE  	2
-#define NCFILTER_INQ	    	3
-#define NCFILTER_FILTERIDS      4
-#define NCFILTER_INFO		5
-#define NCFILTER_FREESPEC	6
-#define NCFILTER_CLIENT_REG	10
-#define NCFILTER_CLIENT_UNREG	11
-#define NCFILTER_CLIENT_INQ	12
+extern int nc_set_alignment(int threshold, int alignment);
+extern int nc_get_alignment(int* thresholdp, int* alignmentp);
 
-typedef enum NC_FILTER_SORT {
-	NC_FILTER_SORT_SPEC=((int)1),
-	NC_FILTER_SORT_IDS=((int)2),
-	NC_FILTER_SORT_CLIENT=((int)3),
-} NC_FILTER_SORT;
+/**************************************************/
+/* Binary searcher for reserved attributes */
+extern const NC_reservedatt* NC_findreserved(const char* name);
+/* reserved attribute initializer */
+extern void NC_initialize_reserved(void);
 
-/* Provide structs to pass args to filter_actions function for HDF5*/
+/* Generic reserved Attributes */
+#define NC_ATT_REFERENCE_LIST "REFERENCE_LIST"
+#define NC_ATT_CLASS "CLASS"
+#define NC_ATT_DIMENSION_LIST "DIMENSION_LIST"
+#define NC_ATT_NAME "NAME"
+#define NC_ATT_COORDINATES "_Netcdf4Coordinates" /*see hdf5internal.h:COORDINATES*/
+#define NC_ATT_FORMAT "_Format"
+#define NC_ATT_DIMID_NAME "_Netcdf4Dimid"
+#define NC_ATT_FILLVALUE "_FillValue"
+#define NC_ATT_NC3_STRICT_NAME "_nc3_strict"
+#define NC_XARRAY_DIMS "_ARRAY_DIMENSIONS"
+#define NC_ATT_CODECS "_Codecs"
 
-typedef struct NC_FILTER_SPEC_HDF5 {
-    int active;            /**< true iff HDF5 library was told to activate filter */
-    unsigned int filterid; /**< ID for arbitrary filter. */
-    size_t nparams;        /**< nparams for arbitrary filter. */
-    unsigned int* params;  /**< Params for arbitrary filter. */
-} NC_FILTER_SPEC_HDF5;
-
-typedef struct NC_FILTERIDS_HDF5 {
-    size_t nfilters;          /**< number of filters */
-    unsigned int* filterids;  /**< Filter ids. */
-} NC_FILTERIDS_HDF5;
-
-typedef struct NC_FILTER_CLIENT_HDF5 {
-    unsigned int id;
-    /* The filter info for hdf5 */
-    /* Avoid needing hdf.h by using void* */
-    void* info;
-} NC_FILTER_CLIENT_HDF5;
-
-typedef struct NC_FILTER_OBJ_HDF5 {
-    NC_Filterobject hdr; /* So we can cast it */
-    NC_FILTER_SORT sort; /* discriminate union */
-    union {
-        NC_FILTER_SPEC_HDF5 spec;
-        NC_FILTERIDS_HDF5 ids;
-        NC_FILTER_CLIENT_HDF5 client;
-    } u;
-} NC_FILTER_OBJ_HDF5;
-
-extern void NC4_freefilterspec(NC_FILTER_SPEC_HDF5* f);
+/* Must match values in libnczarr/zinternal.h */
+#define NC_NCZARR_SUPERBLOCK "_nczarr_superblock"
+#define NC_NCZARR_GROUP "_nczarr_group"
+#define NC_NCZARR_ARRAY "_nczarr_array"
+#define NC_NCZARR_ATTR "_nczarr_attr"
 
 #endif /* _NC4INTERNAL_ */
