@@ -4,7 +4,7 @@ Main header file for the C API.
 
 Copyright 2018, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
-2015, 2016, 2017, 2018
+2015, 2016, 2017, 2018, 2019
 University Corporation for Atmospheric Research/Unidata.
 
 See \ref copyright file for more info.
@@ -109,31 +109,59 @@ extern "C" {
  * different value than the above defaults, create an attribute with
  * the same type as the variable and this reserved name. The value you
  * give the attribute will be used as the fill value for that
- * variable. */
+ * variable. 
+ * Refactored to NC_FillValue in support of 
+ * https://github.com/Unidata/netcdf-c/issues/2858, and parameterized
+ * behind an unsafe macros option as part of 
+ * https://github.com/Unidata/netcdf-c/issues/3029
+ */
+#ifdef NETCDF_ENABLE_LEGACY_MACROS
 #define _FillValue      "_FillValue"
+#endif
+#define NC_FillValue      "_FillValue"
 #define NC_FILL         0       /**< Argument to nc_set_fill() to clear NC_NOFILL */
 #define NC_NOFILL       0x100   /**< Argument to nc_set_fill() to turn off filling of data. */
 
 /* Define the ioflags bits for nc_create and nc_open.
-   currently unused:
+   Currently unused in lower 16 bits:
         0x0002
-   and the whole upper 16 bits
+   All upper 16 bits are unused except
+        0x20000
+        0x40000
 */
 
+/* Lower 16 bits */
+
 #define NC_NOWRITE       0x0000 /**< Set read-only access for nc_open(). */
 #define NC_WRITE         0x0001 /**< Set read-write access for nc_open(). */
 
 #define NC_CLOBBER       0x0000 /**< Destroy existing file. Mode flag for nc_create(). */
 #define NC_NOCLOBBER     0x0004 /**< Don't destroy existing file. Mode flag for nc_create(). */
-
 #define NC_DISKLESS      0x0008  /**< Use diskless file. Mode flag for nc_open() or nc_create(). */
 #define NC_MMAP          0x0010  /**< \deprecated Use diskless file with mmap. Mode flag for nc_open() or nc_create()*/
 
 #define NC_64BIT_DATA    0x0020  /**< CDF-5 format: classic model but 64 bit dimensions and sizes */
 #define NC_CDF5          NC_64BIT_DATA  /**< Alias NC_CDF5 to NC_64BIT_DATA */
 
-#define NC_UDF0          0x0040  /**< User-defined format 0. */
-#define NC_UDF1          0x0080  /**< User-defined format 1. */
+/** @name User-Defined Format Mode Flags
+ * Mode flags for user-defined formats (UDF0-UDF9).
+ * Use with nc_def_user_format() to register custom format handlers.
+ * Can not be combined with other mode flags (e.g., NC_NETCDF4).
+ * See @ref user_defined_formats for details.
+ * @{ */
+#define NC_UDF0          0x0040  /**< User-defined format 0 (bit 6). */
+#define NC_UDF1          0x0080  /**< User-defined format 1 (bit 7). */
+/* UDF2-UDF9 use bits 16, 19-25 (skipping bits 17-18 which are used by
+ * NC_NOATTCREORD=0x20000 and NC_NODIMSCALE_ATTACH=0x40000) */
+#define NC_UDF2          0x10000  /**< User-defined format 2 (bit 16). */
+#define NC_UDF3          0x80000  /**< User-defined format 3 (bit 19). */
+#define NC_UDF4          0x100000  /**< User-defined format 4 (bit 20). */
+#define NC_UDF5          0x200000  /**< User-defined format 5 (bit 21). */
+#define NC_UDF6          0x400000  /**< User-defined format 6 (bit 22). */
+#define NC_UDF7          0x800000  /**< User-defined format 7 (bit 23). */
+#define NC_UDF8          0x1000000  /**< User-defined format 8 (bit 24). */
+#define NC_UDF9          0x2000000  /**< User-defined format 9 (bit 25). */
+/**@}*/
 
 #define NC_CLASSIC_MODEL 0x0100 /**< Enforce classic model on netCDF-4. Mode flag for nc_create(). */
 #define NC_64BIT_OFFSET  0x0200  /**< Use large (64-bit) file offsets. Mode flag for nc_create(). */
@@ -150,19 +178,25 @@ Use this in mode flags for both nc_create() and nc_open(). */
 
 #define NC_NETCDF4       0x1000  /**< Use netCDF-4/HDF5 format. Mode flag for nc_create(). */
 
-/** Turn on MPI I/O.
-Use this in mode flags for both nc_create() and nc_open(). */
+/** The following 3 flags are deprecated as of 4.6.2. Parallel I/O is now
+ * initiated by calling nc_create_par and nc_open_par, no longer by flags.
+ */
 #define NC_MPIIO         0x2000 /**< \deprecated */
-/** Turn on MPI POSIX I/O.
-Use this in mode flags for both nc_create() and nc_open(). */
-#define NC_MPIPOSIX      NC_MPIIO /**< \deprecated As of libhdf5 1.8.13. Now an alias */
+#define NC_MPIPOSIX      NC_MPIIO /**< \deprecated */
+#define NC_PNETCDF       (NC_MPIIO) /**< \deprecated */
 
 #define NC_PERSIST       0x4000  /**< Save diskless contents to disk. Mode flag for nc_open() or nc_create() */
 #define NC_INMEMORY      0x8000  /**< Read from memory. Mode flag for nc_open() or nc_create() */
 
-#define NC_PNETCDF       (NC_MPIIO) /**< \deprecated Use PnetCDF library; alias for NC_MPIIO. */
+/* Upper 16 bits */
+#define NC_NOATTCREORD  0x20000 /**< Disable the netcdf-4 (hdf5) attribute creation order tracking */
+#define NC_NODIMSCALE_ATTACH 0x40000 /**< Disable the netcdf-4 (hdf5) attaching of dimscales to variables (#2128) */
 
 #define NC_MAX_MAGIC_NUMBER_LEN 8 /**< Max len of user-defined format magic number. */
+/** Maximum number of user-defined format slots (UDF0-UDF9).
+ * @see nc_def_user_format(), nc_inq_user_format()
+ * @see @ref user_defined_formats */
+#define NC_MAX_UDF_FORMATS 10
 
 /** Format specifier for nc_set_default_format() and returned
  *  by nc_inq_format. This returns the format as provided by
@@ -172,8 +206,8 @@ Use this in mode flags for both nc_create() and nc_open(). */
  */
 /**@{*/
 #define NC_FORMAT_CLASSIC         (1)
-/* After adding CDF5 support, this flag
-   is somewhat confusing. So, it is renamed.
+/* After adding CDF5 support, the NC_FORMAT_64BIT
+   flag is somewhat confusing. So, it is renamed.
    Note that the name in the contributed code
    NC_FORMAT_64BIT was renamed to NC_FORMAT_CDF2
 */
@@ -186,6 +220,9 @@ Use this in mode flags for both nc_create() and nc_open(). */
 /* Alias */
 #define NC_FORMAT_CDF5    NC_FORMAT_64BIT_DATA
 
+/* Define a mask covering format flags only */
+#define NC_FORMAT_ALL (NC_64BIT_OFFSET|NC_64BIT_DATA|NC_CLASSIC_MODEL|NC_NETCDF4|NC_UDF0|NC_UDF1|NC_UDF2|NC_UDF3|NC_UDF4|NC_UDF5|NC_UDF6|NC_UDF7|NC_UDF8|NC_UDF9)
+
 /**@}*/
 
 /** Extended format specifier returned by  nc_inq_format_extended()
@@ -213,8 +250,27 @@ Use this in mode flags for both nc_create() and nc_open(). */
 #define NC_FORMATX_PNETCDF   (4)
 #define NC_FORMATX_DAP2      (5)
 #define NC_FORMATX_DAP4      (6)
-#define NC_FORMATX_UDF0      (8)
-#define NC_FORMATX_UDF1      (9)
+/** @name User-Defined Format Constants
+ * Format constants for user-defined formats (UDF0-UDF9).
+ * Used internally to identify dispatch tables.
+ * @see nc_def_user_format(), nc_inq_user_format()
+ * @see @ref user_defined_formats
+ * @{ */
+#define NC_FORMATX_UDF0      (8)  /**< User-defined format 0 */
+#define NC_FORMATX_UDF1      (9)  /**< User-defined format 1 */
+/**@}*/
+#define NC_FORMATX_NCZARR    (10) /**< Added in version 4.8.0 */
+/** @name User-Defined Format Constants (continued)
+ * @{ */
+#define NC_FORMATX_UDF2      (11) /**< User-defined format 2 */
+#define NC_FORMATX_UDF3      (12) /**< User-defined format 3 */
+#define NC_FORMATX_UDF4      (13) /**< User-defined format 4 */
+#define NC_FORMATX_UDF5      (14) /**< User-defined format 5 */
+#define NC_FORMATX_UDF6      (15) /**< User-defined format 6 */
+#define NC_FORMATX_UDF7      (16) /**< User-defined format 7 */
+#define NC_FORMATX_UDF8      (17) /**< User-defined format 8 */
+#define NC_FORMATX_UDF9      (18) /**< User-defined format 9 */
+/**@}*/
 #define NC_FORMATX_UNDEFINED (0)
 
   /* To avoid breaking compatibility (such as in the python library),
@@ -230,6 +286,7 @@ Use this in mode flags for both nc_create() and nc_open(). */
 #define NC_FORMAT_DAP2      NC_FORMATX_DAP2 /**< \deprecated As of 4.4.0, use NC_FORMATX_DAP2 */
 #define NC_FORMAT_DAP4      NC_FORMATX_DAP4 /**< \deprecated As of 4.4.0, use NC_FORMATX_DAP4 */
 #define NC_FORMAT_UNDEFINED NC_FORMATX_UNDEFINED /**< \deprecated As of 4.4.0, use NC_FORMATX_UNDEFINED */
+#define NC_FORMATX_ZARR     NC_FORMATX_NCZARR /**< \deprecated as of 4.8.0, use NC_FORMATX_NCZARR */
 
 /**@}*/
 
@@ -274,8 +331,11 @@ NOTE: The NC_MAX_DIMS, NC_MAX_ATTRS, and NC_MAX_VARS limits
 #define NC_MAX_VAR_DIMS 1024 /**< max per variable dimensions */
 /**@}*/
 
-/** This is the max size of an SD dataset name in HDF4 (from HDF4 documentation).*/
-#define NC_MAX_HDF4_NAME 64
+/** The max size of an SD dataset name in HDF4 (from HDF4
+ * documentation) is 64. But in in the wild we have encountered longer
+ * names. As long as the HDF4 name is not greater than NC_MAX_NAME,
+ * our code will be OK. */
+#define NC_MAX_HDF4_NAME NC_MAX_NAME
 
 /** In HDF5 files you can set the endianness of variables with
     nc_def_var_endian(). This define is used there. */
@@ -287,10 +347,14 @@ NOTE: The NC_MAX_DIMS, NC_MAX_ATTRS, and NC_MAX_VARS limits
 
 /** In HDF5 files you can set storage for each variable to be either
  * contiguous or chunked, with nc_def_var_chunking().  This define is
- * used there. */
+ * used there. Unknown storage is used for further extensions of HDF5
+ * storage models, which should be handled transparently by netcdf */
 /**@{*/
-#define NC_CHUNKED    0
-#define NC_CONTIGUOUS 1
+#define NC_CHUNKED         0
+#define NC_CONTIGUOUS      1
+#define NC_COMPACT         2
+#define NC_UNKNOWN_STORAGE 3
+#define NC_VIRTUAL         4
 /**@}*/
 
 /** In HDF5 files you can set check-summing for each variable.
@@ -314,6 +378,36 @@ there. */
 #define NC_MIN_DEFLATE_LEVEL 0 /**< Minimum deflate level. */
 #define NC_MAX_DEFLATE_LEVEL 9 /**< Maximum deflate level. */
 
+#define NC_SZIP_NN 32 /**< SZIP NN option mask. */
+#define NC_SZIP_EC 4  /**< SZIP EC option mask. */
+
+#define NC_NOQUANTIZE 0 /**< No quantization in use. */    
+#define NC_QUANTIZE_BITGROOM 1 /**< Use BitGroom quantization. */
+#define NC_QUANTIZE_GRANULARBR 2 /**< Use Granular BitRound quantization. */
+#define NC_QUANTIZE_BITROUND 3 /**< Use BitRound quantization. */
+
+/**@{*/
+/** When quantization is used for a variable, an attribute of the
+ * appropriate name is added. */
+#define NC_QUANTIZE_BITGROOM_ATT_NAME "_QuantizeBitGroomNumberOfSignificantDigits"
+#define NC_QUANTIZE_GRANULARBR_ATT_NAME "_QuantizeGranularBitRoundNumberOfSignificantDigits"
+#define NC_QUANTIZE_BITROUND_ATT_NAME "_QuantizeBitRoundNumberOfSignificantBits"
+/**@}*/
+
+/**@{*/
+/** For quantization, the allowed value of number of significant
+ * decimal and binary digits, respectively, for float. */
+#define NC_QUANTIZE_MAX_FLOAT_NSD (7)
+#define NC_QUANTIZE_MAX_FLOAT_NSB (23)
+/**@}*/
+
+/**@{*/
+/** For quantization, the allowed value of number of significant
+ * decimal and binary digits, respectively, for double. */
+#define NC_QUANTIZE_MAX_DOUBLE_NSD (15)
+#define NC_QUANTIZE_MAX_DOUBLE_NSB (52)
+/**@}*/
+
 /** The netcdf version 3 functions all return integer error status.
  * These are the possible values, in addition to certain values from
  * the system errno.h.
@@ -469,7 +563,26 @@ by the desired type. */
 #define NC_ERCFILE       (-133)    /**< RC file failure */
 #define NC_ENULLPAD      (-134)    /**< Header Bytes not Null-Byte padded */
 #define NC_EINMEMORY     (-135)    /**< In-memory file error */
-#define NC4_LAST_ERROR   (-136)    /**< @internal All netCDF errors > this. */
+#define NC_ENOFILTER     (-136)    /**< Filter not defined on variable. */
+#define NC_ENCZARR       (-137)    /**< Error at NCZarr layer. */
+#define NC_ES3           (-138)    /**< Generic S3 error */
+#define NC_EEMPTY        (-139)    /**< Attempt to read empty NCZarr map key */
+#define NC_EOBJECT       (-140)    /**< Some object exists when it should not */
+#define NC_ENOOBJECT     (-141)    /**< Some object not found */
+#define NC_EPLUGIN       (-142)    /**< Unclassified failure in accessing a dynamically loaded plugin> */
+#define NC_ENOTZARR      (-143)    /**< Malformed (NC)Zarr file */
+#define NC_EZARRMETA     (-144)    /**< Invalid (NC)Zarr file consolidated metadata */
+
+#define NC4_LAST_ERROR   (-144)    /**< @internal All netCDF errors > this. */
+
+/*
+ * Don't forget to update docs/all-error-codes.md if adding new error codes here!
+ *
+ */
+
+/* Errors for all remote access methods(e.g. DAP and CDMREMOTE)*/
+#define NC_EURL         (NC_EDAPURL)   /**< Malformed URL */
+#define NC_ECONSTRAINT  (NC_EDAPCONSTRAINT)   /**< Malformed Constraint*/
 
 /** @internal This is used in netCDF-4 files for dimensions without
  * coordinate vars. */
@@ -480,10 +593,6 @@ by the desired type. */
  * size_t. Doh! */
 #define NC_HAVE_NEW_CHUNKING_API 1
 
-/* Errors for all remote access methods(e.g. DAP and CDMREMOTE)*/
-#define NC_EURL         (NC_EDAPURL)   /**< Malformed URL */
-#define NC_ECONSTRAINT  (NC_EDAPCONSTRAINT)   /**< Malformed Constraint*/
-
 /*
  * The Interface
  */
@@ -513,14 +622,70 @@ nc_inq_libvers(void);
 EXTERNL const char *
 nc_strerror(int ncerr);
 
-/* Set up user-defined format. */
+/** Register a user-defined format.
+ *
+ * This function registers a custom format handler (dispatch table) for one of
+ * the 10 available UDF slots (UDF0-UDF9). After registration, files can be
+ * opened using the specified mode flag, or automatically via magic number detection.
+ *
+ * @param mode_flag One of NC_UDF0 through NC_UDF9, optionally combined with
+ *                  other mode flags (e.g., NC_UDF0 | NC_NETCDF4). Only one
+ *                  UDF flag should be specified.
+ * @param dispatch_table Pointer to the dispatch table containing function
+ *                       pointers for all netCDF API operations. The dispatch
+ *                       table's version field must match NC_DISPATCH_VERSION.
+ * @param magic_number Optional magic number string (max 8 bytes) for automatic
+ *                     format detection. Files starting with this string will
+ *                     automatically use this dispatch table. Pass NULL if not using
+ *                     magic number detection.
+ *
+ * @return NC_NOERR on success, error code on failure.
+ * @retval NC_EINVAL Invalid mode_flag or dispatch table version mismatch
+ * @retval NC_ENOTNC4 UDF support not enabled in this build
+ *
+ * @see nc_inq_user_format()
+ * @see @ref user_defined_formats
+ *
+ * @author Edward Hartnett
+ * @date 2/2/25
+ */
 typedef struct NC_Dispatch NC_Dispatch;
 EXTERNL int
 nc_def_user_format(int mode_flag, NC_Dispatch *dispatch_table, char *magic_number);
 
+/** Query a registered user-defined format.
+ *
+ * This function retrieves the dispatch table and magic number for a previously
+ * registered user-defined format.
+ *
+ * @param mode_flag One of NC_UDF0 through NC_UDF9. Only one UDF flag should
+ *                  be specified.
+ * @param dispatch_table Pointer to receive the dispatch table pointer. Pass NULL
+ *                       if not needed.
+ * @param magic_number Buffer to receive the magic number string (must be at least
+ *                     NC_MAX_MAGIC_NUMBER_LEN + 1 bytes). Pass NULL if not needed.
+ *
+ * @return NC_NOERR on success, error code on failure.
+ * @retval NC_EINVAL Invalid mode_flag or UDF slot not registered
+ * @retval NC_ENOTNC4 UDF support not enabled in this build
+ *
+ * @see nc_def_user_format()
+ * @see @ref user_defined_formats
+ *
+ * @author Edward Hartnett
+ * @date 2/2/25
+ */
 EXTERNL int
 nc_inq_user_format(int mode_flag, NC_Dispatch **dispatch_table, char *magic_number);
 
+/* Set the global alignment property */
+EXTERNL int
+nc_set_alignment(int threshold, int alignment);
+
+/* Get the global alignment property */
+EXTERNL int
+nc_get_alignment(int* thresholdp, int* alignmentp);
+
 EXTERNL int
 nc__create(const char *path, int cmode, size_t initialsz,
          size_t *chunksizehintp, int *ncidp);
@@ -622,7 +787,7 @@ nc_insert_array_compound(int ncid, nc_type xtype, const char *name,
 EXTERNL int
 nc_inq_type(int ncid, nc_type xtype, char *name, size_t *size);
 
-/* Get the id of a type from the name. */
+/* Get the id of a type from the name, which might be a fully qualified name */
 EXTERNL int
 nc_inq_typeid(int ncid, const char *name, nc_type *typeidp);
 
@@ -702,16 +867,6 @@ EXTERNL int
 nc_inq_vlen(int ncid, nc_type xtype, char *name, size_t *datum_sizep,
             nc_type *base_nc_typep);
 
-/* When you read VLEN type the library will actually allocate the
- * storage space for the data. This storage space must be freed, so
- * pass the pointer back to this function, when you're done with the
- * data, and it will free the vlen memory. */
-EXTERNL int
-nc_free_vlen(nc_vlen_t *vl);
-
-EXTERNL int
-nc_free_vlens(size_t len, nc_vlen_t vlens[]);
-
 /* Put or get one element in a vlen array. */
 EXTERNL int
 nc_put_vlen_element(int ncid, int typeid1, void *vlen_element,
@@ -721,13 +876,6 @@ EXTERNL int
 nc_get_vlen_element(int ncid, int typeid1, const void *vlen_element,
                     size_t *len, void *data);
 
-/* When you read the string type the library will allocate the storage
- * space for the data. This storage space must be freed, so pass the
- * pointer back to this function, when you're done with the data, and
- * it will free the string memory. */
-EXTERNL int
-nc_free_string(size_t len, char **data);
-
 /* Find out about a user defined type. */
 EXTERNL int
 nc_inq_user_type(int ncid, nc_type xtype, char *name, size_t *size,
@@ -770,6 +918,9 @@ nc_inq_enum_member(int ncid, nc_type xtype, int idx, char *name,
 
 
 /* Get enum name from enum value. Name size will be <= NC_MAX_NAME. */
+/* If value is zero and there is no matching ident, then return _UNDEFINED */
+#define NC_UNDEFINED_ENUM_IDENT "_UNDEFINED"
+
 EXTERNL int
 nc_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier);
 
@@ -836,6 +987,16 @@ nc_get_varm(int ncid, int varid,  const size_t *startp,
 
 /* Extra netcdf-4 stuff. */
 
+/* Set quantization settings for a variable. Quantizing data improves
+ * later compression. Must be called after nc_def_var and before
+ * nc_enddef. */
+EXTERNL int
+nc_def_var_quantize(int ncid, int varid, int quantize_mode, int nsd);
+
+/* Find out quantization settings of a var. */
+EXTERNL int
+nc_inq_var_quantize(int ncid, int varid, int *quantize_modep, int *nsdp);
+
 /* Set compression settings for a variable. Lower is faster, higher is
  * better. Must be called after nc_def_var and before nc_enddef. */
 EXTERNL int
@@ -847,6 +1008,10 @@ EXTERNL int
 nc_inq_var_deflate(int ncid, int varid, int *shufflep,
                    int *deflatep, int *deflate_levelp);
 
+/* Set szip compression for a variable. */
+EXTERNL int nc_def_var_szip(int ncid, int varid, int options_mask,
+                            int pixels_per_block);
+
 /* Find out szip settings of a var. */
 EXTERNL int
 nc_inq_var_szip(int ncid, int varid, int *options_maskp, int *pixels_per_blockp);
@@ -890,7 +1055,7 @@ nc_inq_var_endian(int ncid, int varid, int *endianp);
 EXTERNL int
 nc_def_var_filter(int ncid, int varid, unsigned int id, size_t nparams, const unsigned int* parms);
 
-/* Learn about the filter on a variable */
+/* Learn about the first filter on a variable */
 EXTERNL int
 nc_inq_var_filter(int ncid, int varid, unsigned int* idp, size_t* nparams, unsigned int* params);
 
@@ -1702,6 +1867,78 @@ nc_put_var_string(int ncid, int varid, const char **op);
 EXTERNL int
 nc_get_var_string(int ncid, int varid, char **ip);
 
+/* Begin instance walking functions */
+
+/* When you read an array of string typed instances, the library will allocate
+ * the storage space for the strings in the array (but not the array itself).
+ * The strings must be freed eventually, so pass the pointer to the array plus
+ * the number of elements in the array to this function when you're done with
+ * the data, and it will free the all the string instances.
+ * The caller is still responsible for free'ing the array itself,
+ * if it was dynamically allocated.
+ */
+EXTERNL int
+nc_free_string(size_t nelems, char **data);
+
+/* When you read an array of VLEN typed instances, the library will allocate
+ * the storage space for the data in each VLEN in the array (but not the array itself).
+ * That VLEN data must be freed eventually, so pass the pointer to the array plus
+ * the number of elements in the array to this function when you're done with
+ * the data, and it will free the all the VLEN instances.
+ * The caller is still responsible for free'ing the array itself,
+ * if it was dynamically allocated.
+ *
+ * WARNING: this function only works if the basetype of the vlen type
+ * is fixed size. This means it is an atomic type except NC_STRING,
+ * or an NC_ENUM, or and NC_OPAQUE, or an NC_COMPOUND where all
+ * the fields of the compound type are themselves fixed size.
+ */
+EXTERNL int
+nc_free_vlens(size_t nelems, nc_vlen_t vlens[]);
+
+/* This function is a special case of "nc_free_vlens" where nelem == 1 */
+EXTERNL int
+nc_free_vlen(nc_vlen_t *vl);
+
+/**
+Reclaim an array of instances of an arbitrary type.
+This function is intended for use with e.g. nc_get_vara
+or the input to e.g. nc_put_vara.
+This function recursively walks the top-level instances to
+reclaim any nested data such as vlen or strings or such.
+
+WARNING: nc_reclaim_data does not reclaim the top-level
+memory because we do not know how it was allocated.  However
+nc_reclaim_data_all does attempt to reclaim top-level memory.
+
+WARNING: all data blocks below the top-level (e.g. string
+instances) will be reclaimed, so do not call if there is any
+static data in the instance.
+
+Should work for any netcdf format.
+*/
+
+EXTERNL int nc_reclaim_data(int ncid, nc_type xtypeid, void* memory, size_t nelems);
+EXTERNL int nc_reclaim_data_all(int ncid, nc_type xtypeid, void* memory, size_t nelems);
+
+/**
+Copy vector of arbitrary type instances.  This recursively walks
+the top-level instances to copy any nested data such as vlen or
+strings or such.
+
+Assumes it is passed a pointer to count instances of xtype.
+WARNING: nc_copy_data does not copy the top-level memory, but
+assumes a block of proper size was passed in.  However
+nc_copy_data_all does allocate top-level memory copy.
+
+Should work for any netcdf format.
+*/
+
+EXTERNL int nc_copy_data(int ncid, nc_type xtypeid, const void* memory, size_t count, void* copy);
+EXTERNL int nc_copy_data_all(int ncid, nc_type xtypeid, const void* memory, size_t count, void** copyp);
+
+/* end recursive instance walking functions */
+
 /* Begin Deprecated, same as functions with "_ubyte" replaced by "_uchar" */
 EXTERNL int
 nc_put_att_ubyte(int ncid, int varid, const char *name, nc_type xtype,
@@ -1739,8 +1976,10 @@ nc_get_varm_ubyte(int ncid, int varid, const size_t *startp,
                   const ptrdiff_t * imapp, unsigned char *ip);
 EXTERNL int
 nc_put_var_ubyte(int ncid, int varid, const unsigned char *op);
+
 EXTERNL int
 nc_get_var_ubyte(int ncid, int varid, unsigned char *ip);
+
 /* End Deprecated */
 
 /* Set the log level. 0 shows only errors, 1 only major messages,
@@ -1758,19 +1997,17 @@ nc_show_metadata(int ncid);
 
 /* End {put,get}_var */
 
-/* #ifdef _CRAYMPP */
+/* Delete a file. */
+EXTERNL int
+nc_delete(const char *path);
+
 /*
- * Public interfaces to better support
- * CRAY multi-processor systems like T3E.
- * A tip of the hat to NERSC.
- */
-/*
- * It turns out we need to declare and define
- * these public interfaces on all platforms
- * or things get ugly working out the
- * FORTRAN interface. On !_CRAYMPP platforms,
- * these functions work as advertised, but you
- * can only use "processor element" 0.
+ * The following functions were written to accommodate the old Cray
+ * systems. Modern HPC systems do not use these functions any more,
+ * but use the nc_open_par()/nc_create_par() functions instead. These
+ * functions are retained for backward compatibibility. These
+ * functions work as advertised, but you can only use "processor
+ * element" 0.
  */
 
 EXTERNL int
@@ -1781,9 +2018,6 @@ EXTERNL int
 nc__open_mp(const char *path, int mode, int basepe,
         size_t *chunksizehintp, int *ncidp);
 
-EXTERNL int
-nc_delete(const char *path);
-
 EXTERNL int
 nc_delete_mp(const char *path, int basepe);
 
@@ -1793,8 +2027,6 @@ nc_set_base_pe(int ncid, int pe);
 EXTERNL int
 nc_inq_base_pe(int ncid, int *pe);
 
-/* #endif _CRAYMPP */
-
 /* This v2 function is used in the nc_test program. */
 EXTERNL int
 nctypelen(nc_type datatype);
@@ -1973,6 +2205,14 @@ EXTERNL int nc_initialize(void);
 */
 EXTERNL int nc_finalize(void);
 
+/* Programmatic access to the internal .rc table */
+
+/* Get the value corresponding to key | return NULL; caller frees  result */
+EXTERNL char* nc_rc_get(const char* key);
+
+/* Set/overwrite the value corresponding to key */
+EXTERNL int nc_rc_set(const char* key, const char* value);
+
 #if defined(__cplusplus)
 }
 #endif
