@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribuution conditions.
  *********************************************************************/
 
@@ -9,11 +9,10 @@
 #include "dapdump.h"
 #include "ncd2dispatch.h"
 #include "ocx.h"
+#include <stddef.h>
 
 #define NEWVARM
 
-static DCEnode* save = NULL;
-
 /* Define a tracker for memory to support*/
 /* the concatenation*/
 struct NCMEMORY {
@@ -24,11 +23,11 @@ struct NCMEMORY {
 /* Forward:*/
 static NCerror moveto(NCDAPCOMMON*, Getvara*, CDFnode* dataroot, void* memory);
 static NCerror movetor(NCDAPCOMMON*, OCdatanode currentcontent,
-		   NClist* path, int depth,
+		   NClist* path, size_t depth,
 		   Getvara*, size_t dimindex,
 		   struct NCMEMORY*, NClist* segments);
 static NCerror movetofield(NCDAPCOMMON*, OCdatanode,
-			   NClist*, int depth,
+			   NClist*, size_t depth,
 		   	   Getvara*, size_t dimindex,
 		   	   struct NCMEMORY*, NClist* segments);
 
@@ -97,7 +96,7 @@ nc3d_getvarx(int ncid, int varid,
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    int i;
+    size_t i;
     NC* drno;
     NC* substrate;
     NCDAPCOMMON* dapcomm;
@@ -173,7 +172,7 @@ fprintf(stderr,"\n");
 
     /* Fill in missing arguments */
     if(startp == NULL)
-	startp = nc_sizevector0;
+	startp = NC_coord_zero;
 
     if(countp == NULL) {
         /* Accumulate the dimension sizes */
@@ -185,7 +184,7 @@ fprintf(stderr,"\n");
     }
 
     if(stridep == NULL)
-	stridep = nc_ptrdiffvector1;
+	stridep = NC_stride_one;
 
     /* Validate the dimension sizes */
     for(i=0;i<ncrank;i++) {
@@ -196,7 +195,7 @@ fprintf(stderr,"\n");
 	goto fail;
       }
       if(startp[i] >= dim->dim.declsize
-	 || startp[i]+(stridep[i]*(countp[i]-1)) >= dim->dim.declsize) {
+	 || startp[i]+((size_t)stridep[i]*(countp[i]-1)) >= dim->dim.declsize) {
 	ncstat = NC_EINVALCOORDS;
 	goto fail;
       }
@@ -412,7 +411,6 @@ fprintf(stderr,"cache.datadds=%s\n",dumptree(cachenode->datadds));
 
     /* Switch to datadds tree space*/
     varainfo->target = xtarget;
-save = (DCEnode*)varaprojection;
     ncstat = moveto(dapcomm,varainfo,varainfo->cache->datadds,data);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 
@@ -429,7 +427,7 @@ fail:
 static NCerror
 removepseudodims(DCEprojection* proj)
 {
-    int i;
+    size_t i;
 #ifdef DEBUG1
 fprintf(stderr,"removesequencedims.before: %s\n",dumpprojection(proj));
 #endif
@@ -481,7 +479,7 @@ static NCerror
 movetor(NCDAPCOMMON* nccomm,
 	OCdatanode currentcontent,
 	NClist* path,
-        int depth, /* depth is position in segment list*/
+        size_t depth, /* depth is position in segment list*/
 	Getvara* xgetvar,
         size_t dimindex, /* dimindex is position in xgetvar->slices*/
 	struct NCMEMORY* memory,
@@ -493,7 +491,6 @@ movetor(NCDAPCOMMON* nccomm,
     CDFnode* xnode = (CDFnode*)nclistget(path,depth);
     OCdatanode reccontent = NULL;
     OCdatanode dimcontent = NULL;
-    OCdatanode fieldcontent = NULL;
     Dapodometer* odom = NULL;
     int hasstringdim = 0;
     DCEsegment* segment;
@@ -505,11 +502,12 @@ movetor(NCDAPCOMMON* nccomm,
     if(xnode->etype == NC_STRING || xnode->etype == NC_URL) hasstringdim = 1;
 
     /* Get the mode */
-    mode = oc_data_mode(conn,currentcontent);
+    ocstat = oc_data_mode(conn,currentcontent,&mode);
+    if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
 
 #ifdef DEBUG2
-fprintf(stderr,"moveto: nctype=%d depth=%d dimindex=%d mode=%s",
-        xnode->nctype, depth,dimindex,oc_data_modestring(mode));
+fprintf(stderr,"moveto: nctype=%d depth=%d dimindex=%d",
+        xnode->nctype, (int)depth, (int)dimindex);
 fprintf(stderr," segment=%s hasstringdim=%d\n",
 		dcetostring((DCEnode*)segment),hasstringdim);
 #endif
@@ -598,16 +596,15 @@ fprintf(stderr," segment=%s hasstringdim=%d\n",
     case NC_Atomic:
 
         if(hasstringdim)
-	    ncstat = extractstring(nccomm, xgetvar, xnode, segment, dimindex, conn, currentcontent, memory);
-	else
-	    ncstat = extract(nccomm, xgetvar, xnode, segment, dimindex, conn, currentcontent, memory);
-	break;
+            ncstat = extractstring(nccomm, xgetvar, xnode, segment, dimindex, conn, currentcontent, memory);
+        else
+            ncstat = extract(nccomm, xgetvar, xnode, segment, dimindex, conn, currentcontent, memory);
+        break;
 
     }
 
 done:
     oc_data_free(conn,dimcontent);
-    oc_data_free(conn,fieldcontent);
     oc_data_free(conn,reccontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     if(odom) dapodom_free(odom);
@@ -618,7 +615,7 @@ static NCerror
 movetofield(NCDAPCOMMON* nccomm,
 	OCdatanode currentcontent,
 	NClist* path,
-        int depth, /* depth is position in segment list*/
+        size_t depth, /* depth is position in segment list*/
 	Getvara* xgetvar,
         size_t dimindex, /* dimindex is position in xgetvar->slices*/
 	struct NCMEMORY* memory,
@@ -629,11 +626,9 @@ movetofield(NCDAPCOMMON* nccomm,
     size_t fieldindex,gridindex;
     OClink conn = nccomm->oc.conn;
     CDFnode* xnode = (CDFnode*)nclistget(path,depth);
-    OCdatanode reccontent = NULL;
-    OCdatanode dimcontent = NULL;
     OCdatanode fieldcontent = NULL;
     CDFnode* xnext;
-    int newdepth;
+    size_t newdepth;
     int ffield;
 
     /* currentcontent points to the grid/dataset/structure/record instance */
@@ -677,9 +672,7 @@ movetofield(NCDAPCOMMON* nccomm,
 		     segments);
 
 done:
-    oc_data_free(conn,dimcontent);
     oc_data_free(conn,fieldcontent);
-    oc_data_free(conn,reccontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     return THROW(ncstat);
 }
@@ -813,8 +806,8 @@ fprintf(stderr,"\n");
 	      => we have to use odometer on leading prefix.
            If conversion required, then read one-by-one
 	*/
-	int safeindex = dcesafeindex(segment,0,rank0);
-	assert(safeindex >= 0 && safeindex <= rank0);
+	size_t safeindex = dcesafeindex(segment,0,rank0);
+	assert(safeindex <= rank0);
 
 	if(!requireconversion && safeindex == 0) { /* can read whole thing */
             size_t internlen;
@@ -890,7 +883,6 @@ static NCerror
 slicestring(OClink conn, char* stringmem, DCEslice* slice, struct NCMEMORY* memory)
 {
     size_t stringlen;
-    unsigned int i;
     NCerror ncstat = NC_NOERR;
     char* lastchar;
     size_t charcount; /* number of characters inserted into memory */
@@ -905,12 +897,12 @@ slicestring(OClink conn, char* stringmem, DCEslice* slice, struct NCMEMORY* memo
 fprintf(stderr,"moveto: slicestring: string/%lu=%s\n",stringlen,stringmem);
 fprintf(stderr,"slicestring: %lu string=|%s|\n",stringlen,stringmem);
 fprintf(stderr,"slicestring: slice=[%lu:%lu:%lu/%lu]\n",
-slice->first,slice->stride,slice->stop,slice->declsize);
+slice->first,slice->stride,slice->last,slice->declsize);
 #endif
 
     /* Stride across string; if we go past end of string, then pad*/
     charcount = 0;
-    for(i=slice->first;i<slice->length;i+=slice->stride) {
+    for(size_t i=slice->first;i<slice->length;i+=slice->stride) {
         if(i < stringlen)
             *memory->next = stringmem[i];
         else /* i >= stringlen*/
@@ -943,7 +935,7 @@ extractstring(
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
-    int i;
+    size_t i;
     size_t rank0;
     NClist* strings = NULL;
     Dapodometer* odom = NULL;
@@ -1064,9 +1056,9 @@ It is assumed that both trees have been re-struct'ed if necessary.
 */
 
 static NCerror
-attachr(CDFnode* xnode, NClist* patternpath, int depth)
+attachr(CDFnode* xnode, NClist* patternpath, size_t depth)
 {
-    unsigned int i,plen,lastnode,gridable;
+    size_t i,plen,lastnode,gridable;
     NCerror ncstat = NC_NOERR;
     CDFnode* patternpathnode;
     CDFnode* patternpathnext;
@@ -1137,7 +1129,7 @@ attachsubsetr(CDFnode* target, CDFnode* pattern)
 {
     unsigned int i;
     NCerror ncstat = NC_NOERR;
-    int fieldindex;
+    size_t fieldindex;
 
 #ifdef DEBUG2
 fprintf(stderr,"attachsubsetr: attach: target=%s pattern=%s\n",
