@@ -7,6 +7,8 @@
 #include "ncd2dispatch.h"
 #include "ncrc.h"
 #include "ncoffsets.h"
+#include "netcdf_dispatch.h"
+#include <stddef.h>
 #ifdef DEBUG2
 #include "dapdump.h"
 #endif
@@ -175,8 +177,13 @@ NCD2_def_var_filter,
 NCD2_set_var_chunk_cache,
 NCD2_get_var_chunk_cache,
 
-NC_NOTNC4_filter_actions,
+NC_NOOP_inq_var_filter_ids,
+NC_NOOP_inq_var_filter_info,
 
+NC_NOTNC4_def_var_quantize,
+NC_NOTNC4_inq_var_quantize,
+
+NC_NOOP_inq_filter_avail,
 };
 
 const NC_Dispatch* NCD2_dispatch_table = NULL; /* moved here from ddispatch.c */
@@ -188,9 +195,8 @@ NCD2_initialize(void)
     ncd2initialized = 1;
 #ifdef DEBUG
     /* force logging to go to stderr */
-    nclogclose();
-    if(nclogopen(NULL))
-        ncsetlogging(1); /* turn it on */
+    ncsetlogging(1); /* turn it on */
+    nclogopen(NULL))
 #endif
     return NC_NOERR;
 }
@@ -346,7 +352,7 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
 
     /* fail if we are unconstrainable but have constraints */
     if(FLAGSET(dapcomm->controls,NCF_UNCONSTRAINABLE)) {
-	if(dapcomm->oc.url->query != NULL) {
+	if(dapcomm->oc.url != NULL && dapcomm->oc.url->query != NULL) {
 	    nclog(NCLOGWARN,"Attempt to constrain an unconstrainable data source: %s",
 		   dapcomm->oc.url->query);
 	    ncstat = THROW(NC_EDAPCONSTRAINT);
@@ -378,7 +384,7 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
 	    nc_set_default_format(old,&new); /* restore */
 	    dapcomm->substrate.realfile = ((ncflags & NC_DISKLESS) != 0);
 	    dapcomm->substrate.filename = strdup(tmpname);
-	    if(tmpname == NULL) ncstat = NC_ENOMEM;
+	    if(dapcomm->substrate.filename == NULL) ncstat = NC_ENOMEM;
 	    dapcomm->substrate.nc3id = nc3id;
 	}
         if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
@@ -393,7 +399,9 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
      /* Parse constraints to make sure they are syntactically correct */
      ncstat = dapparsedapconstraints(dapcomm,dapcomm->oc.url->query,dapcomm->oc.dapconstraint);
      if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
-
+#ifdef DEBUG2
+fprintf(stderr,"ce=%s\n",dumpconstraint(dapcomm->oc.dapconstraint));
+#endif
     /* Construct a url for oc minus any constraint and params*/
     dapcomm->oc.urltext = ncuribuild(dapcomm->oc.url,NULL,NULL,NCURIBASE);
 
@@ -401,9 +409,8 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
     ocstat = oc_open(dapcomm->oc.urltext,&dapcomm->oc.conn);
     if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
 
-#ifdef DEBUG1
-    (void)oc_trace_curl(dapcomm->oc.conn);
-#endif
+    if(getenv("CURLOPT_VERBOSE") != NULL)
+        (void)oc_trace_curl(dapcomm->oc.conn);
 
     nullfree(dapcomm->oc.urltext); /* clean up */
     dapcomm->oc.urltext = NULL;
@@ -413,12 +420,7 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
 
     /* Turn on logging; only do this after oc_open*/
     if((value = dapparamvalue(dapcomm,"log")) != NULL) {
-	ncloginit();
-        if(nclogopen(value))
-	    ncsetlogging(1);
-	ncloginit();
-        if(nclogopen(value))
-	    ncsetlogging(1);
+        ncsetloglevel(NCLOGNOTE);
     }
 
     /* fetch and build the unconstrained DDS for use as
@@ -643,7 +645,6 @@ done:
 static NCerror
 builddims(NCDAPCOMMON* dapcomm)
 {
-    int i;
     NCerror ncstat = NC_NOERR;
     int dimid;
     NClist* dimset = NULL;
@@ -655,12 +656,13 @@ builddims(NCDAPCOMMON* dapcomm)
 
     /* Sort by fullname just for the fun of it */
     for(;;) {
-	int last = nclistlength(dimset) - 1;
+	const size_t last = nclistlength(dimset);
+	if (last == 0) break;
 	int swap = 0;
-        for(i=0;i<last;i++) {
+	for(size_t i=0;i<last-1;i++) {
 	    CDFnode* dim1 = (CDFnode*)nclistget(dimset,i);
 	    CDFnode* dim2 = (CDFnode*)nclistget(dimset,i+1);
-   	    if(strcmp(dim1->ncfullname,dim2->ncfullname) > 0) {
+	    if(strcmp(dim1->ncfullname,dim2->ncfullname) > 0) {
 		nclistset(dimset,i,(void*)dim2);
 		nclistset(dimset,i+1,(void*)dim1);
 		swap = 1;
@@ -693,7 +695,7 @@ builddims(NCDAPCOMMON* dapcomm)
 
     }
 
-    for(i=0;i<nclistlength(dimset);i++) {
+    for(size_t i=0;i<nclistlength(dimset);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(dimset,i);
         if(dim->dim.basedim != NULL) continue; /* handle below */
 	if(DIMFLAG(dim,CDFDIMRECORD)) continue; /* defined above */
@@ -711,7 +713,7 @@ fprintf(stderr,"define: dim: %s=%ld\n",dim->ncfullname,(long)dim->dim.declsize);
 
     /* Make all duplicate dims have same dimid as basedim*/
     /* (see computecdfdimnames)*/
-    for(i=0;i<nclistlength(dimset);i++) {
+    for(size_t i=0;i<nclistlength(dimset);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(dimset,i);
         if(dim->dim.basedim != NULL) {
 	    dim->ncid = dim->dim.basedim->ncid;
@@ -727,7 +729,7 @@ done:
 static NCerror
 buildvars(NCDAPCOMMON* dapcomm)
 {
-    int i,j;
+    size_t i,j;
     NCerror ncstat = NC_NOERR;
     int varid;
     NClist* varnodes = dapcomm->cdf.ddsroot->tree->varnodes;
@@ -737,7 +739,7 @@ buildvars(NCDAPCOMMON* dapcomm)
     for(i=0;i<nclistlength(varnodes);i++) {
 	CDFnode* var = (CDFnode*)nclistget(varnodes,i);
         int dimids[NC_MAX_VAR_DIMS];
-	unsigned int ncrank;
+	size_t ncrank;
         NClist* vardims = NULL;
 
 	if(var->invisible) continue;
@@ -772,7 +774,7 @@ fprintf(stderr,"\n");
         ncstat = nc_def_var(dapcomm->substrate.nc3id,
 		        definename,
                         var->externaltype,
-                        ncrank,
+                        (int)ncrank,
                         (ncrank==0?NULL:dimids),
                         &varid);
 	nullfree(definename);
@@ -804,7 +806,7 @@ fprintf(stderr,"\n");
 		NCattribute* att = (NCattribute*)nclistget(var->attributes,j);
 		char* val = NULL;
 		/* Check for _FillValue/Variable mismatch */
-		if(strcmp(att->name,"_FillValue")==0) {
+		if(strcmp(att->name,NC_FillValue)==0) {
 		    /* Special case var is byte, fillvalue is int16 and 
 			unsignedattr == 0;
 			This exception is needed because DAP2 byte type
@@ -821,13 +823,13 @@ fprintf(stderr,"\n");
 			if(val) free(val);
 			nclistpush(unsignedatt->values,strdup("false"));
 		    } else if(att->etype != var->etype) {/* other mismatches */
-			/* Log a message */
-	                nclog(NCLOGERR,"_FillValue/Variable type mismatch: variable=%s",var->ncbasename);
 			/* See if mismatch is allowed */
 			if(FLAGSET(dapcomm->controls,NCF_FILLMISMATCH)) {
 			    /* Forcibly change the attribute type to match */
 			    att->etype = var->etype;
 			} else {
+	  		    /* Log a message */
+	                    nclog(NCLOGWARN,"_FillValue/Variable type mismatch: variable=%s",var->ncbasename);
 			    ncstat = NC_EBADTYPE; /* fail */
 			    goto done;
 			}
@@ -848,7 +850,7 @@ done:
 static NCerror
 buildglobalattrs(NCDAPCOMMON* dapcomm, CDFnode* root)
 {
-    int i;
+    size_t i;
     NCerror ncstat = NC_NOERR;
     const char* txt;
     char *nltxt, *p;
@@ -929,9 +931,9 @@ done:
 static NCerror
 buildattribute(NCDAPCOMMON* dapcomm, CDFnode* var, NCattribute* att)
 {
-    int i;
+    size_t i;
     NCerror ncstat = NC_NOERR;
-    unsigned int nvalues = nclistlength(att->values);
+    size_t nvalues = nclistlength(att->values);
     int varid = (var == NULL ? NC_GLOBAL : var->ncid);
     void* mem = NULL;
 
@@ -965,7 +967,7 @@ buildattribute(NCDAPCOMMON* dapcomm, CDFnode* var, NCattribute* att)
         if(ncstat) goto done;
     } else {
 	nc_type atype;
-	unsigned int typesize;
+	size_t typesize;
 	atype = nctypeconvert(dapcomm,att->etype);
 	typesize = nctypesizeof(atype);
 	if (nvalues > 0) {
@@ -1039,7 +1041,7 @@ NCD2_inq_format_extended(int ncid, int* formatp, int* modep)
 NCerror
 computecdfdimnames(NCDAPCOMMON* nccomm)
 {
-    int i,j;
+    size_t i,j;
     char tmp[NC_MAX_NAME*2];
     NClist* conflicts = nclistnew();
     NClist* varnodes = nccomm->cdf.ddsroot->tree->varnodes;
@@ -1115,7 +1117,7 @@ fprintf(stderr,"conflict: %s[%lu] %s[%lu]\n",
 	/* Give  all the conflicting dimensions an index */
 	for(j=0;j<nclistlength(conflicts);j++) {
 	    CDFnode* dim = (CDFnode*)nclistget(conflicts,j);
-	    dim->dim.index1 = j+1;
+	    dim->dim.index1 = (int)j+1;
 	}
     }
     nclistfree(conflicts);
@@ -1227,7 +1229,7 @@ paramlookup(NCDAPCOMMON* state, const char* key)
 {
     const char* value = NULL;
     if(state == NULL || key == NULL || state->oc.url == NULL) return NULL;
-    value = ncurilookup(state->oc.url,key);
+    value = ncurifragmentlookup(state->oc.url,key);
     return value;
 }
 
@@ -1239,9 +1241,10 @@ paramlookup(NCDAPCOMMON* state, const char* key)
 static NCerror
 applyclientparams(NCDAPCOMMON* nccomm)
 {
-    int i,len;
-    int dfaltstrlen = DEFAULTSTRINGLENGTH;
-    int dfaltseqlim = DEFAULTSEQLIMIT;
+    size_t i;
+    unsigned int len;
+    unsigned int dfaltstrlen = DEFAULTSTRINGLENGTH;
+    unsigned int dfaltseqlim = DEFAULTSEQLIMIT;
     const char* value;
     char tmpname[NC_MAX_NAME+32];
     char* pathstr = NULL;
@@ -1284,7 +1287,7 @@ applyclientparams(NCDAPCOMMON* nccomm)
 	dfaltseqlim = 0;
     value = paramlookup(nccomm,"limit");
     if(value != NULL && strlen(value) != 0) {
-        if(sscanf(value,"%d",&len) && len > 0) dfaltseqlim = len;
+        if(sscanf(value,"%u",&len) && len > 0) dfaltseqlim = len;
     }
     nccomm->cdf.defaultsequencelimit = dfaltseqlim;
 
@@ -1293,7 +1296,7 @@ applyclientparams(NCDAPCOMMON* nccomm)
     if(value == NULL) 
         value = paramlookup(nccomm,"maxstrlen");
     if(value != NULL && strlen(value) != 0) {
-        if(sscanf(value,"%d",&len) && len > 0) dfaltstrlen = len;
+        if(sscanf(value,"%u",&len) && len > 0) dfaltstrlen = len;
     } 
     nccomm->cdf.defaultstringlength = dfaltstrlen;
 
@@ -1308,8 +1311,8 @@ applyclientparams(NCDAPCOMMON* nccomm)
 	strlcat(tmpname,pathstr,sizeof(tmpname));
 	value = paramlookup(nccomm,tmpname);
 	if(value == NULL) {
-	    strcpy(tmpname,"maxstrlen_");
-	    strncat(tmpname,pathstr,NC_MAX_NAME);
+	    strncpy(tmpname,"maxstrlen_",sizeof(tmpname));
+	    strlcat(tmpname,pathstr,sizeof(tmpname));
 	    value = paramlookup(nccomm,tmpname);
         }
 	nullfree(pathstr);
@@ -1363,12 +1366,12 @@ applyclientparams(NCDAPCOMMON* nccomm)
 static void
 computedimindexanon(CDFnode* dim, CDFnode* var)
 {
-    int i;
+    size_t i;
     NClist* dimset = var->array.dimsetall;
     for(i=0;i<nclistlength(dimset);i++) {
 	CDFnode* candidate = (CDFnode*)nclistget(dimset,i);
         if(dim == candidate) {
-	   dim->dim.index1=i+1;
+           dim->dim.index1 = (int)i+1;
 	   return;
 	}
     }
@@ -1378,7 +1381,7 @@ computedimindexanon(CDFnode* dim, CDFnode* var)
 static void
 replacedims(NClist* dims)
 {
-    int i;
+    size_t i;
     for(i=0;i<nclistlength(dims);i++) {
         CDFnode* dim = (CDFnode*)nclistget(dims,i);
 	CDFnode* basedim = dim->dim.basedim;
@@ -1406,7 +1409,7 @@ equivalentdim(CDFnode* basedim, CDFnode* dupdim)
 static void
 getalldimsa(NClist* dimset, NClist* alldims)
 {
-    int i;
+    size_t i;
     for(i=0;i<nclistlength(dimset);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(dimset,i);
 	if(!nclistcontains(alldims,(void*)dim)) {
@@ -1425,7 +1428,7 @@ fprintf(stderr,"getalldims: %s[%lu]\n",
 NClist*
 getalldims(NCDAPCOMMON* nccomm, int visibleonly)
 {
-    int i;
+    size_t i;
     NClist* alldims = nclistnew();
     NClist* varnodes = nccomm->cdf.ddsroot->tree->varnodes;
 
@@ -1449,7 +1452,7 @@ addstringdims(NCDAPCOMMON* dapcomm)
        in DODS{...} attribute set or defaulting to the variable name.
        All such dimensions are global.
     */
-    int i;
+    size_t i;
     NClist* varnodes = dapcomm->cdf.ddsroot->tree->varnodes;
     CDFnode* globalsdim = NULL;
     char dimname[4096];
@@ -1513,7 +1516,7 @@ addstringdims(NCDAPCOMMON* dapcomm)
 static NCerror
 defrecorddim(NCDAPCOMMON* dapcomm)
 {
-    unsigned int i;
+    size_t i;
     NCerror ncstat = NC_NOERR;
     NClist* basedims;
 
@@ -1589,7 +1592,7 @@ fail:
 static NCerror
 showprojection(NCDAPCOMMON* dapcomm, CDFnode* var)
 {
-    int i,rank;
+    size_t i,rank;
     NCerror ncstat = NC_NOERR;
     NCbytes* projection = ncbytesnew();
     NClist* path = nclistnew();
@@ -1628,7 +1631,6 @@ getseqdimsize(NCDAPCOMMON* dapcomm, CDFnode* seq, size_t* sizep)
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
     OClink conn = dapcomm->oc.conn;
-    OCdatanode rootcontent = NULL;
     OCddsnode ocroot;
     CDFnode* dxdroot;
     CDFnode* xseq;
@@ -1687,7 +1689,6 @@ fprintf(stderr,"sequencesize: %s = %lu\n",seq->ocname,(unsigned long)seqsize);
 
 fail:
     ncbytesfree(seqcountconstraints);
-    oc_data_free(conn,rootcontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     return ncstat;
 }
@@ -1719,7 +1720,6 @@ countsequence(NCDAPCOMMON* dapcomm, CDFnode* xseq, size_t* sizep)
 {
     unsigned int i;
     NClist* path = nclistnew();
-    int index;
     OCerror ocstat = OC_NOERR;
     NCerror ncstat = NC_NOERR;
     OClink conn = dapcomm->oc.conn;
@@ -1760,7 +1760,7 @@ countsequence(NCDAPCOMMON* dapcomm, CDFnode* xseq, size_t* sizep)
 	    }
 	    /* get next node in path; structure/dataset => exists */
 	    next = (CDFnode*)nclistget(path,i+1);
-	    index = fieldindex(current,next);
+	    size_t index = fieldindex(current,next);
             /* Move to appropriate field */
 	    ocstat = oc_data_ithfield(conn,data,index,&nextdata);
 	    if(ocstat) goto done;
@@ -1849,7 +1849,7 @@ make sure we always have a constraint.
 static NCerror
 computeseqcountconstraints(NCDAPCOMMON* dapcomm, CDFnode* seq, NCbytes* seqcountconstraints)
 {
-    int i,j;
+    size_t i,j;
     NClist* path = NULL;
     CDFnode* var = NULL;
 
@@ -1876,7 +1876,7 @@ computeseqcountconstraints(NCDAPCOMMON* dapcomm, CDFnode* seq, NCbytes* seqcount
 		ncbytescat(seqcountconstraints,tmp);
 	    }
 	} else if(nclistlength(node->array.dimset0) > 0) {
-	    int ndims = nclistlength(node->array.dimset0);
+	    size_t ndims = nclistlength(node->array.dimset0);
 	    for(j=0;j<ndims;j++) {
 		CDFnode* dim = (CDFnode*)nclistget(node->array.dimset0,j);
 		if(DIMFLAG(dim,CDFDIMSTRING)) {
@@ -1961,7 +1961,7 @@ computeseqcountconstraintsr(NCDAPCOMMON* dapcomm, CDFnode* node, CDFnode** candi
     } else if(candidate != NULL && candidatep != NULL) {
 	*candidatep = candidate;
     } else { /* compound != NULL by construction */
-	/* recurse on a nested grids or strucures */
+	/* recurse on a nested grids or structures */
         computeseqcountconstraintsr(dapcomm,compound,candidatep);
     }
 }
@@ -1987,8 +1987,8 @@ cdftotalsize(NClist* dimensions)
 static void
 estimatevarsizes(NCDAPCOMMON* dapcomm)
 {
-    int ivar;
-    unsigned int rank;
+    size_t ivar;
+    size_t rank;
     size_t totalsize = 0;
 
     for(ivar=0;ivar<nclistlength(dapcomm->cdf.ddsroot->tree->varnodes);ivar++) {
@@ -2145,19 +2145,19 @@ fail:
 static NCerror
 suppressunusablevars(NCDAPCOMMON* dapcomm)
 {
-    int i,j;
+    size_t i;
     int found = 1;
     NClist* path = nclistnew();
 
     while(found) {
 	found = 0;
 	/* Walk backwards to aid removal semantics */
-	for(i=nclistlength(dapcomm->cdf.ddsroot->tree->varnodes)-1;i>=0;i--) {
+	for(i = nclistlength(dapcomm->cdf.ddsroot->tree->varnodes); i-->0;) {
 	    CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->varnodes,i);
 	    /* See if this var is under an unusable sequence */
 	    nclistclear(path);
 	    collectnodepath(var,path,WITHOUTDATASET);
-	    for(j=0;j<nclistlength(path);j++) {
+	    for(size_t j=0;j<nclistlength(path);j++) {
 		CDFnode* node = (CDFnode*)nclistget(path,j);
 		if(node->nctype == NC_Sequence
 		   && !node->usesequence) {
@@ -2184,7 +2184,7 @@ make them invisible.
 static NCerror
 fixzerodims(NCDAPCOMMON* dapcomm)
 {
-    int i,j;
+    size_t i,j;
     for(i=0;i<nclistlength(dapcomm->cdf.ddsroot->tree->varnodes);i++) {
 	CDFnode* var = (CDFnode*)nclistget(dapcomm->cdf.ddsroot->tree->varnodes,i);
         NClist* ncdims = var->array.dimsetplus;
@@ -2204,6 +2204,8 @@ fixzerodims(NCDAPCOMMON* dapcomm)
 static void
 applyclientparamcontrols(NCDAPCOMMON* dapcomm)
 {
+    const char* value = NULL;
+
     /* clear the flags */
     CLRFLAG(dapcomm->controls,NCF_CACHE);
     CLRFLAG(dapcomm->controls,NCF_SHOWFETCH);
@@ -2211,6 +2213,8 @@ applyclientparamcontrols(NCDAPCOMMON* dapcomm)
     CLRFLAG(dapcomm->controls,NCF_NCDAP);
     CLRFLAG(dapcomm->controls,NCF_PREFETCH);
     CLRFLAG(dapcomm->controls,NCF_PREFETCH_EAGER);
+    CLRFLAG(dapcomm->controls,NCF_ENCODE_PATH);
+    CLRFLAG(dapcomm->controls,NCF_ENCODE_QUERY);
 
     /* Turn on any default on flags */
     SETFLAG(dapcomm->controls,DFALT_ON_FLAGS);
@@ -2245,8 +2249,31 @@ applyclientparamcontrols(NCDAPCOMMON* dapcomm)
     else if(dapparamcheck(dapcomm,"nofillmismatch",NULL))
 	CLRFLAG(dapcomm->controls,NCF_FILLMISMATCH);
 
+    if((value=dapparamvalue(dapcomm,"encode")) != NULL) {
+	size_t i;
+	NClist* encode = nclistnew();
+	if(dapparamparselist(value,',',encode)) 
+            nclog(NCLOGERR,"Malformed encode parameter: %s",value);
+	else {
+	    /* First, turn off all the encode flags */
+            CLRFLAG(dapcomm->controls,NCF_ENCODE_PATH|NCF_ENCODE_QUERY);
+	    for(i=0;i<nclistlength(encode);i++) {
+	        const char* s = nclistget(encode,i);
+	        if(strcmp(s,"path")==0)
+	            SETFLAG(dapcomm->controls,NCF_ENCODE_PATH);
+	        else if(strcmp(s,"query")==0)
+	            SETFLAG(dapcomm->controls,NCF_ENCODE_QUERY);
+	        else if(strcmp(s,"all")==0)
+	            SETFLAG(dapcomm->controls,NCF_ENCODE_PATH|NCF_ENCODE_QUERY);
+	        else if(strcmp(s,"none")==0)
+	            CLRFLAG(dapcomm->controls,NCF_ENCODE_PATH|NCF_ENCODE_QUERY);
+	    }
+            nclistfreeall(encode);
+	}
+    } else { /* Set defaults */
+	SETFLAG(dapcomm->controls,NCF_ENCODE_QUERY);
+    }
     nclog(NCLOGNOTE,"Caching=%d",FLAGSET(dapcomm->controls,NCF_CACHE));
-
 }
 
 /*
@@ -2837,3 +2864,23 @@ NCD2_get_var_chunk_cache(int ncid, int p2, size_t* p3, size_t* p4, float* p5)
     return THROW(ret);
 }
 
+/* Get substrate NC* object.
+   This function breaks the abstraction, but is necessary for
+   code that accesses the underlying netcdf-4 metadata objects.
+   Used in: NC_reclaim_data[_all]
+            NC_copy_data[_all]
+*/
+
+NC*
+NCD2_get_substrate(NC* nc)
+{
+    NC* subnc = NULL;
+    /* Iff nc->dispatch is the DAP2 dispatcher, then do a level of indirection */
+    if(USED2INFO(nc)) {
+        NCDAPCOMMON* d2 = (NCDAPCOMMON*)nc->dispatchdata;
+        /* Find pointer to NC struct for this file. */
+        (void)NC_check_id(d2->substrate.nc3id,&subnc);
+    } else subnc = nc;
+    return subnc;
+}
+
