@@ -6,58 +6,62 @@
 #ifndef D4UTIL_H
 #define D4UTIL_H 1
 
-#ifdef HAVE_MEMMOVE
-#define d4memmove(dst,src,n) memmove(dst,src,n)
-#else
-#define d4memmove(dst,src,n) localmemmove(dst,src,n)
-#endif
-
 /* This is intended to be big enough to work as
    an offset/position/size for a file or a memory block.
 */
 typedef unsigned long long d4size_t;
 
-/* Define an counted memory marker */
+/* Define a (size, memory) pair */
 typedef struct D4blob {d4size_t size; void* memory;} D4blob;
 
+/* Empty blob constant */
+#define NULLBLOB(blob) {blob.size = 0; blob.memory = NULL;}
+
+#define OFFSET2BLOB(blob,offset) do{(blob).size = (d4size_t)((offset)->limit - (offset)->base); (blob).memory = (offset)->base; }while(0)
+#define BLOB2OFFSET(offset,blob) do{\
+(offset)->base = (blob).memory; \
+(offset)->limit = ((char*)(blob).memory) + (blob).size; \
+(offset)->offset = (offset)->base; \
+} while(0)
+
 /**************************************************/
 
 /* signature: void swapinline16(void* ip) */
 #define swapinline16(ip) \
 { \
-    union {char b[2]; unsigned short i;} u; \
+    char b[2]; \
     char* src = (char*)(ip); \
-    u.b[0] = src[1]; \
-    u.b[1] = src[0]; \
-    *((unsigned short*)ip) = u.i; \
+    b[0] = src[1]; \
+    b[1] = src[0]; \
+    memcpy(ip, b, 2); \
 }
 
 /* signature: void swapinline32(void* ip) */
 #define swapinline32(ip) \
 { \
-    union {char b[4]; unsigned int i;} u; \
+    char b[4]; \
     char* src = (char*)(ip); \
-    u.b[0] = src[3]; \
-    u.b[1] = src[2]; \
-    u.b[2] = src[1]; \
-    u.b[3] = src[0]; \
-    *((unsigned int*)ip) = u.i; \
+    b[0] = src[3]; \
+    b[1] = src[2]; \
+    b[2] = src[1]; \
+    b[3] = src[0]; \
+    memcpy(ip, b, 4); \
 }
 
 /* signature: void swapinline64(void* ip) */
 #define swapinline64(ip) \
 { \
-    union {char b[8]; unsigned long long i;} u; \
+    char b[8]; \
     char* src = (char*)(ip); \
-    u.b[0] = src[7]; \
-    u.b[1] = src[6]; \
-    u.b[2] = src[5]; \
-    u.b[3] = src[4]; \
-    u.b[4] = src[3]; \
-    u.b[5] = src[2]; \
-    u.b[6] = src[1]; \
-    u.b[7] = src[0]; \
-    *((unsigned long long*)ip) = u.i; \
+    b[0] = src[7]; \
+    b[1] = src[6]; \
+    b[2] = src[5]; \
+    b[3] = src[4]; \
+    b[4] = src[3]; \
+    b[5] = src[2]; \
+    b[6] = src[1]; \
+    b[7] = src[0]; \
+    memcpy(ip, b, 8); \
 }
 
 /***************************************************/
@@ -72,6 +76,6 @@ typedef struct D4blob {d4size_t size; void* memory;} D4blob;
 #define HASANY   (HASNIL|HASSEQ|HASSTR|HASOPTFIX|HASOPVAR)
 /***************************************************/
 
-extern int ncd4__testurl(const char* parth, char** basename);
+extern int ncd4__testurl(const char* path, char** basename);
 
 #endif /*D4UTIL_H*/
