@@ -3,6 +3,7 @@
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
+#include <stddef.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -43,7 +44,7 @@ static const NC_reservedatt NCD4_reserved[] = {
     {D4CHECKSUMATTR, READONLYFLAG|NAMEONLYFLAG},      /*_DAP4_Checksum_CRC32*/
     {D4LEATTR, READONLYFLAG|NAMEONLYFLAG},            /*_DAP4_Little_Endian*/
     /* Also need to include the provenance attributes */
-    {NCPROPS, READONLYFLAG|NAMEONLYFLAG|MATERIALIZEDFLAG},		/*_NCProperties*/
+    {NCPROPS, READONLYFLAG|NAMEONLYFLAG},		/*_NCProperties*/
     {NULL, 0}
 };
 
@@ -62,7 +63,6 @@ NCD4_initialize(void)
     ncloginit();
 #ifdef D4DEBUG
     /* force logging to go to stderr */
-    nclogclose();
     if(nclogopen(NULL))
         ncsetlogging(1); /* turn it on */
 #endif
@@ -244,7 +244,7 @@ NCD4_def_var_endian(int ncid, int p2, int p3)
 }
 
 static int
-NCD4_def_var_filter(int ncid, int varid, unsigned int id, size_t n, const unsigned int* parms)
+NCD4_def_var_filter(int ncid, int varid, unsigned int id, size_t n, const unsigned int* params)
 {
     return (NC_EPERM);
 }
@@ -389,11 +389,11 @@ NCD4_inq_attname(int ncid, int varid, int attnum, char* name)
     const NC_reservedatt* rsvp = NULL;
     
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    substrateid = makenc4id(ncp,ncid);
+    ret = nc_inq_attname(substrateid, varid, attnum, name);
     /* Is this a reserved attribute name? */
     if(name && (rsvp = NCD4_lookupreserved(name)))
 	return NC_EATTMETA;
-    substrateid = makenc4id(ncp,ncid);
-    ret = nc_inq_attname(substrateid, varid, attnum, name);
     return (ret);
 }
 
@@ -842,13 +842,13 @@ NCD4_inq_dim(int ncid, int dimid, char* name, size_t* lenp)
     NC* ncp;
     NCD4INFO* info;
     NCD4meta* meta;
-    int i;
+    size_t i;
     NCD4node* dim = NULL;
 
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR)
 	goto done;
     info = (NCD4INFO*)ncp->dispatchdata;
-    meta = info->substrate.metadata;
+    meta = info->dmrmetadata;
 
     /* Locate the dimension specified by dimid */
     for(i=0;i<nclistlength(meta->allnodes);i++) {
@@ -872,17 +872,16 @@ static int
 ncd4_get_att_reserved(NC* ncp, int ncid, int varid, const char* name, void* value, nc_type t, const NC_reservedatt* rsvp)
 {
     int ret = NC_NOERR;
-    NCD4INFO* info = (NCD4INFO*)(ncp->dispatchdata);
-    NCD4meta* meta = info->substrate.metadata;
     NCD4node* var = NULL;
 
+    if((ret=NCD4_findvar(ncp,ncid,varid,&var,NULL))) goto done;
+
     if(strcmp(rsvp->name,D4CHECKSUMATTR)==0) {
 	unsigned int* ip = (unsigned int*)value;
 	if(varid == NC_GLOBAL)
             {ret = NC_EBADID; goto done;}
 	if(t != NC_UINT) {ret = NC_EBADTYPE; goto done;}
-        if((ret=NCD4_findvar(ncp,ncid,varid,&var,NULL))) goto done;
-	if(var->data.remotechecksummed == 0)
+	if(var->data.checksumattr == 0)
 	    {ret = NC_ENOTATT; goto done;} 
 	*ip = (var->data.remotechecksum);
     } else if(strcmp(rsvp->name,D4LEATTR)==0) {
@@ -890,7 +889,7 @@ ncd4_get_att_reserved(NC* ncp, int ncid, int varid, const char* name, void* valu
 	if(varid != NC_GLOBAL)
             {ret = NC_EBADID; goto done;}
 	if(t != NC_INT) {ret = NC_EBADTYPE; goto done;}
-	*ip = (meta->serial.remotelittleendian?1:0);
+	*ip = (var->data.response->remotelittleendian?1:0);
     }
 done:
     return THROW(ret);
@@ -906,7 +905,7 @@ ncd4_inq_att_reserved(NC* ncp, int ncid, int varid, const char* name, nc_type* x
 	if(varid == NC_GLOBAL)
             {ret = NC_EBADID; goto done;}
         if((ret=NCD4_findvar(ncp,ncid,varid,&var,NULL))) goto done;
-	if(var->data.remotechecksummed == 0)
+	if(var->data.checksumattr == 0)
 	    {ret = NC_ENOTATT; goto done;} 
 	if(xtypep) *xtypep = NC_UINT;
 	if(lenp) *lenp = 1;
@@ -926,7 +925,7 @@ static int
 globalinit(void)
 {
     int stat = NC_NOERR;
-    return stat;
+    return THROW(stat);
 }
 
 /**************************************************/
