@@ -40,13 +40,59 @@
 
 #undef DEBUG
 
-extern int NC_initialized; /**< True when dispatch table is initialized. */
+#ifndef nulldup
+ #define nulldup(s) ((s)?strdup(s):NULL)
+#endif
 
-/* User-defined formats. */
-NC_Dispatch *UDF0_dispatch_table = NULL;
-char UDF0_magic_number[NC_MAX_MAGIC_NUMBER_LEN + 1] = "";
-NC_Dispatch *UDF1_dispatch_table = NULL;
-char UDF1_magic_number[NC_MAX_MAGIC_NUMBER_LEN + 1] = "";
+
+/* User-defined formats - expanded from 2 to 10 slots.
+ * These arrays store dispatch tables and magic numbers for up to 10 custom formats.
+ * Array-based design replaces the previous individual UDF0/UDF1 global variables. */
+NC_Dispatch *UDF_dispatch_tables[NC_MAX_UDF_FORMATS] = {NULL};
+char UDF_magic_numbers[NC_MAX_UDF_FORMATS][NC_MAX_MAGIC_NUMBER_LEN + 1] = {{0}};
+
+/** Helper function to convert NC_UDFn mode flag to array index (0-9).
+ * @param mode_flag The mode flag (NC_UDF0 through NC_UDF9)
+ * @return Array index 0-9, or -1 if not a valid UDF flag or multiple UDF flags set
+ * @note UDF flags are not sequential in bit positions due to conflicts with
+ *       NC_NOATTCREORD (0x20000) and NC_NODIMSCALE_ATTACH (0x40000) */
+static int udf_mode_to_index(int mode_flag) {
+    int count = 0;
+    int index = -1;
+    
+    /* Count how many UDF flags are set and remember the index */
+    if (fIsSet(mode_flag, NC_UDF0)) { count++; index = 0; }
+    if (fIsSet(mode_flag, NC_UDF1)) { count++; index = 1; }
+    if (fIsSet(mode_flag, NC_UDF2)) { count++; index = 2; }
+    if (fIsSet(mode_flag, NC_UDF3)) { count++; index = 3; }
+    if (fIsSet(mode_flag, NC_UDF4)) { count++; index = 4; }
+    if (fIsSet(mode_flag, NC_UDF5)) { count++; index = 5; }
+    if (fIsSet(mode_flag, NC_UDF6)) { count++; index = 6; }
+    if (fIsSet(mode_flag, NC_UDF7)) { count++; index = 7; }
+    if (fIsSet(mode_flag, NC_UDF8)) { count++; index = 8; }
+    if (fIsSet(mode_flag, NC_UDF9)) { count++; index = 9; }
+    
+    /* Only one UDF flag should be set at a time */
+    if (count != 1)
+        return -1;
+    
+    return index;
+}
+
+/** Helper function to convert NC_FORMATX_UDFn to array index (0-9).
+ * @param formatx The format constant (NC_FORMATX_UDF0 through NC_FORMATX_UDF9)
+ * @return Array index 0-9, or -1 if not a valid UDF format constant
+ * @note Handles the gap in format numbering: UDF0=8, UDF1=9, UDF2=11, UDF3=12, etc.
+ *       (NC_FORMATX_NCZARR=10 occupies the slot between UDF1 and UDF2) */
+static int udf_formatx_to_index(int formatx) {
+    /* UDF0 and UDF1 are sequential (8, 9) */
+    if (formatx >= NC_FORMATX_UDF0 && formatx <= NC_FORMATX_UDF1)
+        return formatx - NC_FORMATX_UDF0;
+    /* UDF2-UDF9 start at 11 (skipping NCZARR at 10) */
+    if (formatx >= NC_FORMATX_UDF2 && formatx <= NC_FORMATX_UDF9)
+        return formatx - NC_FORMATX_UDF2 + 2;
+    return -1;
+}
 
 /**************************************************/
 
@@ -119,9 +165,9 @@ char UDF1_magic_number[NC_MAX_MAGIC_NUMBER_LEN + 1] = "";
 int
 nc_def_user_format(int mode_flag, NC_Dispatch *dispatch_table, char *magic_number)
 {
+    int udf_index;
+
     /* Check inputs. */
-    if (mode_flag != NC_UDF0 && mode_flag != NC_UDF1)
-        return NC_EINVAL;
     if (!dispatch_table)
         return NC_EINVAL;
     if (magic_number && strlen(magic_number) > NC_MAX_MAGIC_NUMBER_LEN)
@@ -130,21 +176,25 @@ nc_def_user_format(int mode_flag, NC_Dispatch *dispatch_table, char *magic_numbe
     /* Check the version of the dispatch table provided. */
     if (dispatch_table->dispatch_version != NC_DISPATCH_VERSION)
         return NC_EINVAL;
+    /* user defined magic numbers not allowed with netcdf3 modes */ 
+    if (magic_number && (fIsSet(mode_flag, NC_64BIT_OFFSET) ||
+                         fIsSet(mode_flag, NC_64BIT_DATA) ||
+                        (fIsSet(mode_flag, NC_CLASSIC_MODEL) &&
+                        !fIsSet(mode_flag, NC_NETCDF4))))
+        return NC_EINVAL;
+
+    /* Get the UDF index from the mode flag. */
+    udf_index = udf_mode_to_index(mode_flag);
+    if (udf_index < 0 || udf_index >= NC_MAX_UDF_FORMATS)
+        return NC_EINVAL;
 
     /* Retain a pointer to the dispatch_table and a copy of the magic
      * number, if one was provided. */
-    switch(mode_flag)
-    {
-    case NC_UDF0:
-        UDF0_dispatch_table = dispatch_table;
-        if (magic_number)
-            strncpy(UDF0_magic_number, magic_number, NC_MAX_MAGIC_NUMBER_LEN);
-        break;
-    case NC_UDF1:
-        UDF1_dispatch_table = dispatch_table;
-        if (magic_number)
-            strncpy(UDF1_magic_number, magic_number, NC_MAX_MAGIC_NUMBER_LEN);
-        break;
+    UDF_dispatch_tables[udf_index] = dispatch_table;
+    if (magic_number) {
+        strncpy(UDF_magic_numbers[udf_index], magic_number, NC_MAX_MAGIC_NUMBER_LEN);
+        /* Ensure null-termination since strncpy doesn't guarantee it if source >= max length */
+        UDF_magic_numbers[udf_index][NC_MAX_MAGIC_NUMBER_LEN] = '\0';
     }
 
     return NC_NOERR;
@@ -169,25 +219,18 @@ nc_def_user_format(int mode_flag, NC_Dispatch *dispatch_table, char *magic_numbe
 int
 nc_inq_user_format(int mode_flag, NC_Dispatch **dispatch_table, char *magic_number)
 {
-    /* Check inputs. */
-    if (mode_flag != NC_UDF0 && mode_flag != NC_UDF1)
+    int udf_index;
+
+    /* Get the UDF index from the mode flag. */
+    udf_index = udf_mode_to_index(mode_flag);
+    if (udf_index < 0 || udf_index >= NC_MAX_UDF_FORMATS)
         return NC_EINVAL;
 
-    switch(mode_flag)
-    {
-    case NC_UDF0:
-        if (dispatch_table)
-            *dispatch_table = UDF0_dispatch_table;
-        if (magic_number)
-            strncpy(magic_number, UDF0_magic_number, NC_MAX_MAGIC_NUMBER_LEN);
-        break;
-    case NC_UDF1:
-        if (dispatch_table)
-            *dispatch_table = UDF1_dispatch_table;
-        if (magic_number)
-            strncpy(magic_number, UDF1_magic_number, NC_MAX_MAGIC_NUMBER_LEN);
-        break;
-    }
+    /* Return the dispatch table and magic number. */
+    if (dispatch_table)
+        *dispatch_table = UDF_dispatch_tables[udf_index];
+    if (magic_number)
+        strncpy(magic_number, UDF_magic_numbers[udf_index], NC_MAX_MAGIC_NUMBER_LEN);
 
     return NC_NOERR;
 }
@@ -1739,6 +1782,13 @@ nc_inq_type(int ncid, nc_type xtype, char *name, size_t *size)
     return ncp->dispatch->inq_type(ncid,xtype,name,size);
 }
 
+/** \defgroup dispatch dispatch functions. */
+/** \{
+
+\ingroup dispatch
+*/
+
+
 /**
    Check the create mode parameter for sanity.
 
@@ -1759,7 +1809,7 @@ static int
 check_create_mode(int mode)
 {
     int mode_format;
-    int mmap = 0;
+    int use_mmap = 0;
     int inmemory = 0;
     int diskless = 0;
 
@@ -1770,17 +1820,17 @@ check_create_mode(int mode)
     if (mode_format && (mode_format & (mode_format - 1)))
         return NC_EINVAL;
 
-    mmap = ((mode & NC_MMAP) == NC_MMAP);
+    use_mmap = ((mode & NC_MMAP) == NC_MMAP);
     inmemory = ((mode & NC_INMEMORY) == NC_INMEMORY);
     diskless = ((mode & NC_DISKLESS) == NC_DISKLESS);
 
     /* NC_INMEMORY and NC_DISKLESS and NC_MMAP are all mutually exclusive */
     if(diskless && inmemory) return NC_EDISKLESS;
-    if(diskless && mmap) return NC_EDISKLESS;
-    if(inmemory && mmap) return NC_EINMEMORY;
+    if(diskless && use_mmap) return NC_EDISKLESS;
+    if(inmemory && use_mmap) return NC_EINMEMORY;
 
     /* mmap is not allowed for netcdf-4 */
-    if(mmap && (mode & NC_NETCDF4)) return NC_EINVAL;
+    if(use_mmap && (mode & NC_NETCDF4)) return NC_EINVAL;
 
 #ifndef USE_NETCDF4
     /* If the user asks for a netCDF-4 file, and the library was built
@@ -1834,19 +1884,16 @@ NC_create(const char *path0, int cmode, size_t initialsz,
 
     TRACE(nc_create);
     if(path0 == NULL)
-        return NC_EINVAL;
+        {stat = NC_EINVAL; goto done;}
 
     /* Check mode flag for sanity. */
-    if ((stat = check_create_mode(cmode)))
-        return stat;
+    if ((stat = check_create_mode(cmode))) goto done;
 
     /* Initialize the library. The available dispatch tables
      * will depend on how netCDF was built
      * (with/without netCDF-4, DAP, CDMREMOTE). */
-    if(!NC_initialized)
-    {
-        if ((stat = nc_initialize()))
-            return stat;
+    if(!NC_initialized) {
+        if ((stat = nc_initialize())) goto done;
     }
 
     {
@@ -1858,10 +1905,7 @@ NC_create(const char *path0, int cmode, size_t initialsz,
 
     memset(&model,0,sizeof(model));
     newpath = NULL;
-    if((stat = NC_infermodel(path,&cmode,1,useparallel,NULL,&model,&newpath))) {
-	nullfree(newpath);
-        goto done;
-    }
+    if((stat = NC_infermodel(path,&cmode,1,useparallel,NULL,&model,&newpath))) goto done;
     if(newpath) {
         nullfree(path);
         path = newpath;
@@ -1879,7 +1923,7 @@ NC_create(const char *path0, int cmode, size_t initialsz,
     if (model.impl == NC_FORMATX_PNETCDF)
     {stat = NC_ENOTBUILT; goto done;}
 #endif
-#ifndef ENABLE_CDF5
+#ifndef NETCDF_ENABLE_CDF5
     if (model.impl == NC_FORMATX_NC3 && (cmode & NC_64BIT_DATA))
     {stat = NC_ENOTBUILT; goto done;}
 #endif
@@ -1898,13 +1942,33 @@ NC_create(const char *path0, int cmode, size_t initialsz,
 #endif
 #ifdef USE_NETCDF4
     case NC_FORMATX_UDF0:
-        dispatcher = UDF0_dispatch_table;
-        break;
     case NC_FORMATX_UDF1:
-        dispatcher = UDF1_dispatch_table;
+    case NC_FORMATX_UDF2:
+    case NC_FORMATX_UDF3:
+    case NC_FORMATX_UDF4:
+    case NC_FORMATX_UDF5:
+    case NC_FORMATX_UDF6:
+    case NC_FORMATX_UDF7:
+    case NC_FORMATX_UDF8:
+    case NC_FORMATX_UDF9:
+        {
+            /* Convert format constant to array index and validate */
+            int udf_index = udf_formatx_to_index(model.impl);
+            if (udf_index < 0 || udf_index >= NC_MAX_UDF_FORMATS) {
+                stat = NC_EINVAL;
+                goto done;
+            }
+            /* Get the dispatch table for this UDF slot */
+            dispatcher = UDF_dispatch_tables[udf_index];
+            /* Ensure the UDF format has been registered via nc_def_user_format() */
+            if (!dispatcher) {
+                stat = NC_ENOTBUILT;
+                goto done;
+            }
+        }
         break;
 #endif /* USE_NETCDF4 */
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
     case NC_FORMATX_NCZARR:
         dispatcher = NCZ_dispatch_table;
 	break;
@@ -1913,7 +1977,7 @@ NC_create(const char *path0, int cmode, size_t initialsz,
         dispatcher = NC3_dispatch_table;
         break;
     default:
-        return NC_ENOTNC;
+        {stat = NC_ENOTNC; goto done;}
     }
 
     /* Create the NC* instance and insert its dispatcher and model */
@@ -1932,6 +1996,7 @@ NC_create(const char *path0, int cmode, size_t initialsz,
     }
 done:
     nullfree(path);
+    nullfree(newpath);
     return stat;
 }
 
@@ -1967,7 +2032,7 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
     const NC_Dispatch* dispatcher = NULL;
     int inmemory = 0;
     int diskless = 0;
-    int mmap = 0;
+    int use_mmap = 0;
     char* path = NULL;
     NCmodel model;
     char* newpath = NULL;
@@ -1975,25 +2040,25 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
     TRACE(nc_open);
     if(!NC_initialized) {
         stat = nc_initialize();
-        if(stat) return stat;
+        if(stat) goto done;
     }
 
     /* Check inputs. */
     if (!path0)
-        return NC_EINVAL;
+        {stat = NC_EINVAL; goto done;}
 
     /* Capture the inmemory related flags */
-    mmap = ((omode & NC_MMAP) == NC_MMAP);
+    use_mmap = ((omode & NC_MMAP) == NC_MMAP);
     diskless = ((omode & NC_DISKLESS) == NC_DISKLESS);
     inmemory = ((omode & NC_INMEMORY) == NC_INMEMORY);
 
     /* NC_INMEMORY and NC_DISKLESS and NC_MMAP are all mutually exclusive */
     if(diskless && inmemory) {stat = NC_EDISKLESS; goto done;}
-    if(diskless && mmap) {stat = NC_EDISKLESS; goto done;}
-    if(inmemory && mmap) {stat = NC_EINMEMORY; goto done;}
+    if(diskless && use_mmap) {stat = NC_EDISKLESS; goto done;}
+    if(inmemory && use_mmap) {stat = NC_EINMEMORY; goto done;}
 
     /* mmap is not allowed for netcdf-4 */
-    if(mmap && (omode & NC_NETCDF4)) {stat = NC_EINVAL; goto done;}
+    if(use_mmap && (omode & NC_NETCDF4)) {stat = NC_EINVAL; goto done;}
 
     /* Attempt to do file path conversion: note that this will do
        nothing if path is a 'file:...' url, so it will need to be
@@ -2026,6 +2091,7 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
     }
 
     /* Suppress unsupported formats */
+#if 0
     /* (should be more compact, table-driven, way to do this) */
     {
 	int hdf5built = 0;
@@ -2036,20 +2102,23 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
 	int nczarrbuilt = 0;
 #ifdef USE_NETCDF4
         hdf5built = 1;
+#endif
 #ifdef USE_HDF4
         hdf4built = 1;
 #endif
-#endif
-#ifdef ENABLE_CDF5
+#ifdef NETCDF_ENABLE_CDF5
         cdf5built = 1;
 #endif
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
 	nczarrbuilt = 1;
 #endif
-        if(UDF0_dispatch_table != NULL)
-            udf0built = 1;
-        if(UDF1_dispatch_table != NULL)
-            udf1built = 1;
+        /* Check which UDF formats are built (i.e., have been registered).
+         * A UDF format is considered "built" if its dispatch table is non-NULL. */
+        int udfbuilt[NC_MAX_UDF_FORMATS] = {0};
+        for(int i = 0; i < NC_MAX_UDF_FORMATS; i++) {
+            if(UDF_dispatch_tables[i] != NULL)
+                udfbuilt[i] = 1;
+        }
 
         if(!hdf5built && model.impl == NC_FORMATX_NC4)
         {stat = NC_ENOTBUILT; goto done;}
@@ -2059,25 +2128,70 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
         {stat = NC_ENOTBUILT; goto done;}
 	if(!nczarrbuilt && model.impl == NC_FORMATX_NCZARR)
         {stat = NC_ENOTBUILT; goto done;}
-        if(!udf0built && model.impl == NC_FORMATX_UDF0)
-        {stat = NC_ENOTBUILT; goto done;}
-        if(!udf1built && model.impl == NC_FORMATX_UDF1)
-        {stat = NC_ENOTBUILT; goto done;}
+        /* Check all UDF formats - ensure the requested format has been registered */
+        for(int i = 0; i < NC_MAX_UDF_FORMATS; i++) {
+            /* Convert array index to format constant (handles gap: UDF0=8, UDF1=9, UDF2=11...) */
+            int formatx = (i <= 1) ? (NC_FORMATX_UDF0 + i) : (NC_FORMATX_UDF2 + i - 2);
+            if(!udfbuilt[i] && model.impl == formatx)
+                {stat = NC_ENOTBUILT; goto done;}
+        }
     }
+#else
+    {
+	unsigned built = 0 /* leave off the trailing semicolon so we can build constant */
+		| (1<<NC_FORMATX_NC3) /* NC3 always supported */
+#ifdef USE_HDF5
+		| (1<<NC_FORMATX_NC_HDF5)
+#endif
+#ifdef USE_HDF4
+		| (1<<NC_FORMATX_NC_HDF4)
+#endif
+#ifdef NETCDF_ENABLE_NCZARR
+		| (1<<NC_FORMATX_NCZARR)
+#endif
+#ifdef NETCDF_ENABLE_DAP
+		| (1<<NC_FORMATX_DAP2)
+#endif
+#ifdef NETCDF_ENABLE_DAP4
+		| (1<<NC_FORMATX_DAP4)
+#endif
+#ifdef USE_PNETCDF
+		| (1<<NC_FORMATX_PNETCDF)
+#endif
+		; /* end of the built flags */
+        /* Add UDF formats to built flags - set bit for each registered UDF format.
+         * Use unsigned literal (1U) to avoid integer overflow for higher bit positions. */
+        for(int i = 0; i < NC_MAX_UDF_FORMATS; i++) {
+            if(UDF_dispatch_tables[i] != NULL) {
+                /* Convert array index to format constant */
+                int formatx = (i <= 1) ? (NC_FORMATX_UDF0 + i) : (NC_FORMATX_UDF2 + i - 2);
+                built |= (1U << formatx);
+            }
+        }
+	/* Verify the requested format is available */
+	if((built & (1U << model.impl)) == 0)
+            {stat = NC_ENOTBUILT; goto done;}
+#ifndef NETCDF_ENABLE_CDF5
+	/* Special case because there is no separate CDF5 dispatcher */
+        if(model.impl == NC_FORMATX_NC3 && (omode & NC_64BIT_DATA))
+            {stat = NC_ENOTBUILT; goto done;}
+#endif
+    }
+#endif
     /* Figure out what dispatcher to use */
     if (!dispatcher) {
         switch (model.impl) {
-#ifdef ENABLE_DAP
+#ifdef NETCDF_ENABLE_DAP
         case NC_FORMATX_DAP2:
             dispatcher = NCD2_dispatch_table;
             break;
 #endif
-#ifdef ENABLE_DAP4
+#ifdef NETCDF_ENABLE_DAP4
         case NC_FORMATX_DAP4:
             dispatcher = NCD4_dispatch_table;
             break;
 #endif
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
 	case NC_FORMATX_NCZARR:
 	    dispatcher = NCZ_dispatch_table;
 	    break;
@@ -2099,10 +2213,30 @@ NC_open(const char *path0, int omode, int basepe, size_t *chunksizehintp,
 #endif
 #ifdef USE_NETCDF4
         case NC_FORMATX_UDF0:
-            dispatcher = UDF0_dispatch_table;
-            break;
         case NC_FORMATX_UDF1:
-            dispatcher = UDF1_dispatch_table;
+        case NC_FORMATX_UDF2:
+        case NC_FORMATX_UDF3:
+        case NC_FORMATX_UDF4:
+        case NC_FORMATX_UDF5:
+        case NC_FORMATX_UDF6:
+        case NC_FORMATX_UDF7:
+        case NC_FORMATX_UDF8:
+        case NC_FORMATX_UDF9:
+            {
+                /* Convert format constant to array index and validate */
+                int udf_index = udf_formatx_to_index(model.impl);
+                if (udf_index < 0 || udf_index >= NC_MAX_UDF_FORMATS) {
+                    stat = NC_EINVAL;
+                    goto done;
+                }
+                /* Get the dispatch table for this UDF slot */
+                dispatcher = UDF_dispatch_tables[udf_index];
+                /* Ensure the UDF format has been registered via nc_def_user_format() */
+                if (!dispatcher) {
+                    stat = NC_ENOTBUILT;
+                    goto done;
+                }
+            }
             break;
 #endif /* USE_NETCDF4 */
         case NC_FORMATX_NC3:
@@ -2158,8 +2292,8 @@ int
 nc__pseudofd(void)
 {
     if(pseudofd == 0)  {
-        int maxfd = 32767; /* default */
 #ifdef HAVE_GETRLIMIT
+        int maxfd = 32767; /* default */
         struct rlimit rl;
         if(getrlimit(RLIMIT_NOFILE,&rl) == 0) {
             if(rl.rlim_max != RLIM_INFINITY)
@@ -2172,3 +2306,4 @@ nc__pseudofd(void)
     }
     return pseudofd++;
 }
+/** \} */
