@@ -229,7 +229,7 @@ the top-level instances to copy any nested data such as vlen or
 strings or such.
 
 Assumes it is passed a pointer to count instances of xtype and a
-space into which to copy the instance.  Copys any nested data
+space into which to copy the instance.  Copies any nested data
 by calling malloc().
 
 WARNING: DOES NOT ALLOCATE THE TOP-LEVEL MEMORY (see the
@@ -439,8 +439,8 @@ copy_datar(NC_FILE_INFO_T* file, NC_TYPE_INFO_T* utype, Position src, Position d
 
 	    /* optimize: string field type */
 	    if(field->nc_typeid == NC_STRING) {
-	        char** srcstrvec = (char**)src.memory;
-	        char** dststrvec = (char**)dst.memory;
+	        char** srcstrvec = (char**)fsrc.memory;
+	        char** dststrvec = (char**)fdst.memory;
 		for(i=0;i<arraycount;i++) 
 		    {if(srcstrvec[i] != NULL) {dststrvec[i] = strdup(srcstrvec[i]);} else {dststrvec[i] = NULL;}}
 		continue; /* move to next field */
