@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncgen/offsets.c,v 1.1 2009/09/25 18:22:40 dmh Exp $
  *********************************************************************/
@@ -20,7 +20,7 @@
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
-This code is a variantion of the H5detect.c code from HDF5.
+This code is a variation of the H5detect.c code from HDF5.
 Author: D. Heimbigner 10/7/2008
 */
 
@@ -75,7 +75,7 @@ The assumptions appear to be as follows:
    depends only on the types that precede it in the struct.
 2. the compiler does not reorder fields.
 3. arrays are tightly packed.
-4. nested structs are alignd according to their first member
+4. nested structs are aligned according to their first member
    (this actually follows from C language requirement that
     a struct can legally be cast to an instance of its first member).
 Given the alignments for the various common primitive types,
@@ -105,18 +105,17 @@ char* ctypenames[NCTYPES] = {
 
 static NCtypealignvec vec[NC_NCTYPES];
 static NCtypealignset set;
-int NC_alignments_computed = 0;
+static int NC_alignments_computed = 0;
 
 /* Argument is a netcdf type class, except compound|ENUM  */
-size_t
-NC_class_alignment(int ncclass)
+int
+NC_class_alignment(int ncclass, size_t* alignp)
 {
+    int stat = NC_NOERR;
     NCalignment* align = NULL;
     int index = 0;
-    if(!NC_alignments_computed) {
+    if(!NC_alignments_computed)
 	NC_compute_alignments();
-	NC_alignments_computed = 1;
-    }
     switch (ncclass) {
       case NC_BYTE: index = NC_UCHARINDEX; break;
       case NC_CHAR: index = NC_CHARINDEX; break;
@@ -137,13 +136,14 @@ NC_class_alignment(int ncclass)
       case NC_COMPOUND: /* fall thru */
       default:
 	nclog(NCLOGERR,"nc_class_alignment: class code %d cannot be aligned",ncclass);
-	return 0;
+	goto done;
     }
     align = &vec[index];
-    return align->alignment;
+    if(alignp) *alignp = align->alignment;
+done:
+    return stat;
 }
 
-
 void
 NC_compute_alignments(void)
 {
