@@ -26,6 +26,7 @@ See COPYRIGHT for license information.
 #include "nc4internal.h"
 #include "ncs3sdk.h"
 #include "ncdispatch.h"
+#include "ncutil.h"
 
 #undef NOREAD
 
@@ -52,7 +53,7 @@ static int rcsearch(const char* prefix, const char* rcname, char** pathp);
 static void rcfreeentries(NClist* rc);
 static void rcfreeentry(NCRCentry* t);
 #ifdef DRCDEBUG
-static void storedump(char* msg, NClist* entrys);
+static void storedump(char* msg, NClist* entries);
 #endif
 
 /* Define default rc files and aliases, also defines load order*/
@@ -136,16 +137,14 @@ This is set by the environment variable NC_TEST_AWS_DIR.
 void
 ncrc_initialize(void)
 {
-    int stat = NC_NOERR;
-    NCglobalstate* ncg = NULL;
-
     if(NCRCinitialized) return;
     NCRCinitialized = 1; /* prevent recursion */
 
-    ncg = NC_getglobalstate();
-
 #ifndef NOREAD
-    /* Load entrys */
+    {
+    int stat = NC_NOERR;
+    NCglobalstate* ncg = NC_getglobalstate();
+    /* Load entries */
     if((stat = NC_rcload())) {
         nclog(NCLOGWARN,".rc loading failed");
     }
@@ -153,6 +152,7 @@ ncrc_initialize(void)
     if((stat = NC_aws_load_credentials(ncg))) {
         nclog(NCLOGWARN,"AWS config file not loaded");
     }
+    }
 #endif
 }
 
@@ -180,7 +180,6 @@ NC_rcclear(NCRCinfo* info)
     nullfree(info->rchome);
     rcfreeentries(info->entries);
     NC_s3freeprofilelist(info->s3profiles);
-
 }
 
 static void
@@ -329,7 +328,7 @@ NC_set_rcfile(const char* rcfile)
         goto done;
     }
     fclose(f);
-    nullfree(globalstate->rcinfo->rcfile);
+    NC_rcclear(globalstate->rcinfo);
     globalstate->rcinfo->rcfile = strdup(rcfile);
     /* Clear globalstate->rcinfo */
     NC_rcclear(&globalstate->rcinfo);
