@@ -4,6 +4,7 @@
  *********************************************************************/
 
 #include "config.h"
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -11,6 +12,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -25,45 +27,55 @@
 #include "ncbytes.h"
 #include "nclist.h"
 #include "nclog.h"
-#include "ncrc.h"
 #include "ncpathmgr.h"
+#include "ncutil.h"
 
 #define NC_MAX_PATH 4096
-#ifndef nulldup
- #define nulldup(x) ((x)?strdup(x):(x))
-#endif
+
 /**************************************************/
 /** \internal
  * Provide a hidden interface to allow utilities
- * to check if a given path name is really an ncdap4 url.
- * If no, return null, else return basename of the url
- * minus any extension.
+ * to check if a given path name is really a url.
+ * If no, return null, else return basename of the url path
+ * minus any extension in basenamep.
+ * If is a url and the protocol is file:, set isfilep.
+ * @return 0 if not URL, 1 if URL
  */
 
 int
-NC__testurl(const char* path, char** basenamep)
+NC__testurl(const char* path, char** basenamep, int* isfilep)
 {
-    NCURI* uri;
-    int ok = NC_NOERR;
-    if(ncuriparse(path,&uri))
-	ok = NC_EURL;
-    else {
-	char* slash = (uri->path == NULL ? NULL : strrchr(uri->path, '/'));
-	char* dot;
-	if(slash == NULL) slash = (char*)path; else slash++;
-        slash = nulldup(slash);
-        if(slash == NULL)
-            dot = NULL;
-        else
-            dot = strrchr(slash, '.');
-        if(dot != NULL &&  dot != slash) *dot = '\0';
-        if(basenamep)
-            *basenamep=slash;
-        else if(slash)
-            free(slash);
-    }
+    int stat = NC_NOERR;
+    NCURI* uri = NULL;
+    int isurl = 1;
+    int isfile = 0;
+    const char* uripath = NULL;
+    char* slash = NULL;
+    char* dot = NULL;
+
+    /* Parse url; if fails or returns null URL, then assume path is not a URL */
+    stat = ncuriparse(path,&uri);
+    if(stat || uri == NULL) {isurl = 0; isfile = 0; goto done;} /* not a url */
+    isurl = 1;
+    if(strcmp(uri->protocol,"file")==0) isfile = 1;
+    /* Extract the basename of the URL */
+    if(uri->path == NULL)
+        uripath = "/";
+    else
+        uripath = uri->path;
+    slash = (char*)strrchr(uripath, '/');
+    if(slash == NULL) slash = (char*)uripath; else slash++;
+    slash = nulldup(slash);
+    assert(slash != NULL);
+    dot = strrchr(slash, '.');
+    if(dot != NULL &&  dot != slash) *dot = '\0';
+    if(basenamep)
+	{*basenamep=slash; slash = NULL;}
+done:
+    if(isfilep) *isfilep = isfile;
+    nullfree(slash);
     ncurifree(uri);
-    return ok;
+    return isurl;
 }
 
 /** \internal Return 1 if this machine is little endian */
@@ -202,12 +214,13 @@ Wrap mktmp and return the generated path,
 or null if failed.
 Base is the base file path. XXXXX is appended
 to allow mktmp add its unique id.
-Return the generated path.
+Return any error
 */
 
-char*
-NC_mktmp(const char* base)
+int
+NC_mktmp(const char* base, char** tmpp)
 {
+    int ret = NC_NOERR;
     int fd = -1;
     char* tmp = NULL;
     size_t len;
@@ -238,19 +251,21 @@ NC_mktmp(const char* base)
         snprintf(spid,sizeof(spid),"%06d",rno);
         strlcat(tmp,spid,len);
         fd=NCopen3(tmp,O_RDWR|O_CREAT, _S_IREAD|_S_IWRITE);
-	if(fd >= 0) break; /* sucess */
+	if(fd >= 0) break; /* success */
 	fd = -1; /* try again */
     }
 #endif /* !HAVE_MKSTEMP */
     if(fd < 0) {
         nclog(NCLOGERR, "Could not create temp file: %s",tmp);
+	ret = errno; errno = 0;
         nullfree(tmp);
 	tmp = NULL;
         goto done;
     }
 done:
     if(fd >= 0) close(fd);
-    return tmp;
+    if(tmpp) {*tmpp = tmp;}
+    return ret;
 }
 
 /** \internal */
@@ -279,24 +294,26 @@ done:
 int
 NC_readfileF(FILE* stream, NCbytes* content, long long amount)
 {
+#define READ_BLOCK_SIZE 4194304
     int ret = NC_NOERR;
     long long red = 0;
-    char part[1024];
+    char *part = (char*) malloc(READ_BLOCK_SIZE);
 
     while(amount < 0 || red < amount) {
-	size_t count = fread(part, 1, sizeof(part), stream);
+	size_t count = fread(part, 1, READ_BLOCK_SIZE, stream);
 	if(ferror(stream)) {ret = NC_EIO; goto done;}
 	if(count > 0) ncbytesappendn(content,part,(unsigned long)count);
-	red += count;
+	red += (long long)count;
     if (feof(stream)) break;
     }
     /* Keep only amount */
     if(amount >= 0) {
-	if(red > amount) ncbytessetlength(content,amount); /* read too much */
+	if(red > amount) ncbytessetlength(content, (unsigned long)amount); /* read too much */
 	if(red < amount) ret = NC_ETRUNC; /* |file| < amount */
     }
     ncbytesnull(content);
 done:
+    free(part);
     return ret;
 }
 
@@ -376,7 +393,7 @@ NC_testmode(NCURI* uri, const char* tag)
 {
     int stat = NC_NOERR;
     int found = 0;
-    int i;
+    size_t i;
     const char* modestr = NULL;
     NClist* modelist = NULL;
 
@@ -394,10 +411,48 @@ done:
     return found;
 }
 
-#if ! defined __INTEL_COMPILER 
+/** \internal
+Add tag to fragment mode list unless already present.
+*/
+int
+NC_addmodetag(NCURI* uri, const char* tag)
+{
+    int stat = NC_NOERR;
+    int found = 0;
+    const char* modestr = NULL;
+    char* modevalue = NULL;
+    NClist* modelist = NULL;
+
+    modestr = ncurifragmentlookup(uri,"mode");
+    if(modestr != NULL) {
+        /* Parse mode str */
+        if((stat = NC_getmodelist(modestr,&modelist))) goto done;
+    } else
+        modelist = nclistnew();
+    /* Search for tag */
+    for(size_t i=0;i<nclistlength(modelist);i++) {
+        const char* mode = (const char*)nclistget(modelist,i);
+	if(strcasecmp(mode,tag)==0) {found = 1; break;}
+    }
+    /* If not found, then add to modelist */
+    if(!found) nclistpush(modelist,strdup(tag));
+    /* Convert modelist back to string */
+    if((stat=NC_joinwith(modelist,",",NULL,NULL,&modevalue))) goto done;
+    /* modify the url */
+    if((stat=ncurisetfragmentkey(uri,"mode",modevalue))) goto done;
+
+done:
+    nclistfreeall(modelist);
+    nullfree(modevalue);
+    return stat;
+}
+
+#if ! defined __INTEL_COMPILER
 #if defined __APPLE__ 
 /** \internal */
 
+#if ! defined HAVE_DECL_ISINF
+
 int isinf(double x)
 {
     union { unsigned long long u; double f; } ieee754;
@@ -406,6 +461,9 @@ int isinf(double x)
            ( (unsigned)ieee754.u == 0 );
 }
 
+#endif /* HAVE_DECL_ISINF */
+
+#if ! defined HAVE_DECL_ISNAN
 /** \internal */
 int isnan(double x)
 {
@@ -415,6 +473,8 @@ int isnan(double x)
            ( (unsigned)ieee754.u != 0 ) > 0x7ff00000;
 }
 
+#endif /* HAVE_DECL_ISNAN */
+
 #endif /*APPLE*/
 #endif /*!_INTEL_COMPILER*/
 
@@ -439,9 +499,9 @@ NC_split_delim(const char* arg, char delim, NClist* segments)
         len = (q - p);
 	if(len == 0)
 	    {stat = NC_EURL; goto done;}
-	if((seg = malloc(len+1)) == NULL)
+	if((seg = malloc((size_t)len+1)) == NULL)
 	    {stat = NC_ENOMEM; goto done;}
-	memcpy(seg,p,len);
+	memcpy(seg,p,(size_t)len);
 	seg[len] = '\0';
 	nclistpush(segments,seg);
 	seg = NULL; /* avoid mem errors */
@@ -456,9 +516,51 @@ done:
 /** \internal concat the the segments with each segment preceded by '/' */
 int
 NC_join(NClist* segments, char** pathp)
+{
+    return NC_joinwith(segments,"/","/",NULL,pathp);
+}
+
+/** \internal
+Concat the the segments with separator.
+@param segments to join
+@param sep to use between segments
+@param prefix put at front of joined string: NULL => no prefix
+@param suffix put at end of joined string: NULL => no suffix
+@param pathp return the join in this
+*/
+int
+NC_joinwith(NClist* segments, const char* sep, const char* prefix, const char* suffix, char** pathp)
 {
     int stat = NC_NOERR;
-    int i;
+    size_t i;
+    NCbytes* buf = NULL;
+    size_t seplen = nulllen(sep);
+
+    if(segments == NULL)
+	{stat = NC_EINVAL; goto done;}
+    if((buf = ncbytesnew())==NULL)
+	{stat = NC_ENOMEM; goto done;}
+    if(prefix) ncbytescat(buf,prefix);
+    for(i=0;i<nclistlength(segments);i++) {
+	const char* seg = nclistget(segments,i);
+	if(i>0 && strncmp(seg,sep,seplen)!=0)
+	    ncbytescat(buf,sep);
+	ncbytescat(buf,seg);
+    }
+    if(suffix) ncbytescat(buf,suffix);
+    if(pathp) *pathp = ncbytesextract(buf);
+done:
+    ncbytesfree(buf);
+    return stat;
+}
+
+#if 0
+/* concat the the segments with each segment preceded by '/' */
+int
+NC_join(NClist* segments, char** pathp)
+{
+    int stat = NC_NOERR;
+    size_t i;
     NCbytes* buf = NULL;
 
     if(segments == NULL)
@@ -471,7 +573,7 @@ NC_join(NClist* segments, char** pathp)
 	const char* seg = nclistget(segments,i);
 	if(seg[0] != '/')
 	    ncbytescat(buf,"/");
-	ncbytescat(buf,seg);
+	ncbytescat(buf,seg);		
     }
 
 done:
@@ -479,5 +581,63 @@ done:
 	if(pathp) *pathp = ncbytesextract(buf);
     }
     ncbytesfree(buf);
-    return stat;
+    return THROW(stat);
 }
+#endif
+
+#if 0
+static int
+extendenvv(char*** envvp, int amount, int* oldlenp)
+{
+    char** envv = *envvp;
+    char** p;
+    int len;
+    for(len=0,p=envv;*p;p++) len++;
+    *oldlenp = len;
+    if((envv = (char**)malloc((amount+len+1)*sizeof(char*)))==NULL) return NC_ENOMEM;
+    memcpy(envv,*envvp,sizeof(char*)*len);
+    envv[len] = NULL;
+    nullfree(*envvp);
+    *envvp = envv; envv = NULL;
+    return NC_NOERR;
+}
+#endif
+
+static int
+nc_compare(const void* arg1, const void* arg2)
+{
+    char* n1 = *((char**)arg1);
+    char* n2 = *((char**)arg2);
+    return strcmp(n1,n2);
+}
+
+/* quick sort a list of strings */
+void
+NC_sortenvv(size_t n, char** envv)
+{
+    if(n <= 1) return;
+    qsort(envv, n, sizeof(char*), nc_compare);
+#if 0
+{int i;
+for(i=0;i<n;i++)
+fprintf(stderr,">>> sorted: [%d] %s\n",i,(const char*)envv[i]);
+}
+#endif
+}
+
+void
+NC_freeenvv(size_t n, char** envv)
+{
+    size_t i;
+    char** p;
+    if(envv == NULL) return;
+    if(n < 0)
+       {for(n=0, p = envv; *p; n++) {}; /* count number of strings */}
+    for(i=0;i<n;i++) {
+        if(envv[i]) {
+	    free(envv[i]);
+	}
+    }
+    free(envv);    
+}
+
