@@ -23,7 +23,7 @@ static int
 ncbytesfail(void)
 {
     fflush(stdout);
-    fprintf(stderr,"bytebuffer failure\n");
+    fprintf(stderr,"NCbytes failure\n");
     fflush(stderr);
 #ifdef NCBYTESDEBUG
     abort();
@@ -39,7 +39,7 @@ ncbytesnew(void)
   bb->alloc=0;
   bb->length=0;
   bb->content=NULL;
-  bb->nonextendible = 0;
+  bb->extendible = 1;
   return bb;
 }
 
@@ -50,9 +50,9 @@ ncbytessetalloc(NCbytes* bb, unsigned long sz)
   if(bb == NULL) return ncbytesfail();
   if(sz == 0) {sz = (bb->alloc?2*bb->alloc:DEFAULTALLOC);}
   if(bb->alloc >= sz) return TRUE;
-  if(bb->nonextendible) return ncbytesfail();
+  if(!bb->extendible) return ncbytesfail();
   newcontent=(char*)calloc(sz,sizeof(char));
-  if(newcontent == NULL) return FALSE;
+  if(newcontent == NULL) ncbytesfail();
   if(bb->alloc > 0 && bb->length > 0 && bb->content != NULL) {
     memcpy((void*)newcontent,(void*)bb->content,sizeof(char)*bb->length);
   }
@@ -62,11 +62,11 @@ ncbytessetalloc(NCbytes* bb, unsigned long sz)
   return TRUE;
 }
 
-void
+EXTERNL void
 ncbytesfree(NCbytes* bb)
 {
   if(bb == NULL) return;
-  if(!bb->nonextendible && bb->content != NULL) free(bb->content);
+  if(bb->extendible && bb->content != NULL) free(bb->content);
   free(bb);
 }
 
@@ -110,12 +110,11 @@ ncbytesset(NCbytes* bb, unsigned long index, char elem)
 int
 ncbytesappend(NCbytes* bb, char elem)
 {
+  char s[2];
   if(bb == NULL) return ncbytesfail();
-  /* We need space for the char + null */
-  ncbytessetalloc(bb,bb->length+2);
-  bb->content[bb->length] = (char)(elem & 0xFF);
-  bb->length++;
-  bb->content[bb->length] = '\0';
+  s[0] = elem;
+  s[1] = '\0';
+  ncbytesappendn(bb,s,1);
   return TRUE;
 }
 
@@ -123,9 +122,8 @@ ncbytesappend(NCbytes* bb, char elem)
 int
 ncbytescat(NCbytes* bb, const char* s)
 {
-  if(s == NULL) {
-    return 1;
-  }
+  if(bb == NULL) return ncbytesfail();
+  if(s == NULL) return 1;
   ncbytesappendn(bb,(void*)s,strlen(s)+1); /* include trailing null*/
   /* back up over the trailing null*/
   if(bb->length == 0) return ncbytesfail();
@@ -138,12 +136,9 @@ ncbytesappendn(NCbytes* bb, const void* elem, unsigned long n)
 {
   if(bb == NULL || elem == NULL) return ncbytesfail();
   if(n == 0) {n = strlen((char*)elem);}
-  while(!ncbytesavail(bb,n+1)) {
-    if(!ncbytessetalloc(bb,0)) return ncbytesfail();
-  }
+  ncbytessetalloc(bb,bb->length+n);
   memcpy((void*)&bb->content[bb->length],(void*)elem,n);
   bb->length += n;
-  bb->content[bb->length] = '\0';
   return TRUE;
 }
 
@@ -180,15 +175,15 @@ ncbytesextract(NCbytes* bb)
 }
 
 int
-ncbytessetcontents(NCbytes* bb, char* contents, unsigned long alloc)
+ncbytessetcontents(NCbytes* bb, void* contents, unsigned long alloc, unsigned long length)
 {
     if(bb == NULL) return ncbytesfail();
     ncbytesclear(bb);
-    if(!bb->nonextendible && bb->content != NULL) free(bb->content);
-    bb->content = contents;
-    bb->length = 0;
+    if(bb->extendible && bb->content != NULL) free(bb->content);
+    bb->content = (char*)contents;
+    bb->length = length;
     bb->alloc = alloc;
-    bb->nonextendible = 1;
+    bb->extendible = 0;
     return 1;
 }
 
@@ -208,7 +203,7 @@ ncbytesremove(NCbytes* bb, unsigned long pos)
     if(bb == NULL) return ncbytesfail();
     if(bb->length <= pos) return ncbytesfail();
     if(pos < (bb->length - 1)) {
-	int copylen = (bb->length - pos) - 1;
+	size_t copylen = (bb->length - pos) - 1;
         memmove(bb->content+pos,bb->content+pos+1,copylen);
     }
     bb->length--;
