@@ -5,7 +5,6 @@
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -46,18 +45,6 @@
 #include <unistd.h>
 #endif
 
-/*
-Define a simple #ifdef test for the version of H5FD_class_t we are using 
-*/
-
-#if H5_VERS_MAJOR == 1
-#if H5_VERS_MINOR < 10
-#define H5FDCLASS1 1
-#endif
-#else
-#error "Cannot determine version of H5FD_class_t"
-#endif
-
 #ifdef H5_HAVE_WIN32_API
 /* The following two defines must be before any windows headers are included */
 #define WIN32_LEAN_AND_MEAN    /* Exclude rarely-used stuff from Windows headers */
@@ -105,7 +92,7 @@ typedef struct H5FD_http_t {
     haddr_t     eof;            /* end of file; current file size   */
     haddr_t     pos;            /* current file I/O position        */
     unsigned    write_access;   /* Flag to indicate the file was opened with write access */
-    H5FD_http_file_op op;		/* last operation */
+    H5FD_http_file_op op;	/* last operation */
     NC_HTTP_STATE*  state;       /* Curl handle + extra */
     char*           url;        /* The URL (minus any fragment) for the dataset */ 
 } H5FD_http_t;
@@ -146,57 +133,62 @@ static herr_t H5FD_http_read(H5FD_t *lf, H5FD_mem_t type, hid_t fapl_id, haddr_t
                 size_t size, void *buf);
 static herr_t H5FD_http_write(H5FD_t *lf, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
                 size_t size, const void *buf);
+static herr_t H5FD_http_term(void);
 
 /* The H5FD_class_t structure has different versions */
-#ifdef H5FDCLASS1
-static haddr_t H5FD_http_get_eof(const H5FD_t *_file);
-static herr_t H5FD_http_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing);
-static herr_t H5FD_http_lock(H5FD_t *_file, unsigned char* old, unsigned lock_type, hbool_t last);
-static herr_t H5FD_http_unlock(H5FD_t *file, unsigned char *oid, hbool_t last);
-#else
-static herr_t H5FD_http_term(void);
 static haddr_t H5FD_http_get_eof(const H5FD_t *_file, H5FD_mem_t type);
 static herr_t H5FD_http_flush(H5FD_t *_file, hid_t dxpl_id, hbool_t closing);
 static herr_t H5FD_http_lock(H5FD_t *_file, hbool_t rw);
 static herr_t H5FD_http_unlock(H5FD_t *_file);
-#endif
 
 /* Beware, not same as H5FD_HTTP_g */
 static const H5FD_class_t H5FD_http_g = {
-    "http",                     /* name         */
-    MAXADDR,                    /* maxaddr      */
-    H5F_CLOSE_WEAK,             /* fc_degree    */
-#ifndef H5FDCLASS1
-    H5FD_http_term,             /* terminate    */
+#if H5FD_CLASS_VERSION > 0
+    H5FD_CLASS_VERSION,		/* struct version  */
+    H5_VFD_HTTP,		/* value           */
 #endif
-    NULL,                       /* sb_size      */
-    NULL,                       /* sb_encode    */
-    NULL,                       /* sb_decode    */
-    0,                          /* fapl_size    */
-    NULL,                       /* fapl_get     */
-    NULL,                       /* fapl_copy    */
-    NULL,                       /* fapl_free    */
-    0,                          /* dxpl_size    */
-    NULL,                       /* dxpl_copy    */
-    NULL,                       /* dxpl_free    */
-    H5FD_http_open,            /* open         */
-    H5FD_http_close,           /* close        */
-    H5FD_http_cmp,             /* cmp          */
-    H5FD_http_query,           /* query        */
-    NULL,                       /* get_type_map */
-    H5FD_http_alloc,           /* alloc        */
-    NULL,                       /* free         */
-    H5FD_http_get_eoa,         /* get_eoa      */
-    H5FD_http_set_eoa,         /* set_eoa      */
-    H5FD_http_get_eof,         /* get_eof      */
-    H5FD_http_get_handle,      /* get_handle   */
-    H5FD_http_read,            /* read         */
-    H5FD_http_write,           /* write        */
-    H5FD_http_flush,           /* flush        */
-    NULL, 		       /* truncate     */
-    H5FD_http_lock,            /* lock         */
-    H5FD_http_unlock,          /* unlock       */
-    H5FD_FLMAP_DICHOTOMY       /* fl_map       */
+    "http",			/* name         */
+    MAXADDR,			/* maxaddr      */
+    H5F_CLOSE_WEAK,		/* fc_degree    */
+    H5FD_http_term,		/* terminate    */
+    NULL,			/* sb_size      */
+    NULL,			/* sb_encode    */
+    NULL,			/* sb_decode    */
+    0,				/* fapl_size    */
+    NULL,			/* fapl_get     */
+    NULL,			/* fapl_copy    */
+    NULL,			/* fapl_free    */
+    0,				/* dxpl_size    */
+    NULL,			/* dxpl_copy    */
+    NULL,			/* dxpl_free    */
+    H5FD_http_open,		/* open         */
+    H5FD_http_close,		/* close        */
+    H5FD_http_cmp,		/* cmp          */
+    H5FD_http_query,		/* query        */
+    NULL,			/* get_type_map */
+    H5FD_http_alloc,		/* alloc        */
+    NULL,			/* free         */
+    H5FD_http_get_eoa,		/* get_eoa      */
+    H5FD_http_set_eoa,		/* set_eoa      */
+    H5FD_http_get_eof,		/* get_eof      */
+    H5FD_http_get_handle,	/* get_handle   */
+    H5FD_http_read,		/* read         */
+    H5FD_http_write,		/* write        */
+#if H5FD_CLASS_VERSION > 0
+    NULL,			/* read_vector     */
+    NULL,			/* write_vector    */
+    NULL,			/* read_selection  */
+    NULL,			/* write_selection */
+#endif
+    H5FD_http_flush,		/* flush        */
+    NULL,			/* truncate     */
+    H5FD_http_lock,		/* lock         */
+    H5FD_http_unlock,		/* unlock       */
+#if H5FD_CLASS_VERSION > 0
+    NULL,			/* del          */
+    NULL,			/* ctl	        */
+#endif
+    H5FD_FLMAP_DICHOTOMY	/* fl_map       */
 };
 
 
@@ -226,6 +218,31 @@ H5FD_http_init(void)
     return H5FD_HTTP_g;
 } /* end H5FD_http_init() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:  H5FD_http_finalize
+ *
+ * Purpose:  Free this driver by unregistering the driver with the
+ *    library.
+ *
+ * Returns:     Non-negative on success or negative on failure
+ *
+ * Programmer:  John Donoghue
+ *              Tuesday, December 12, 2023
+ *
+ *-------------------------------------------------------------------------
+ */
+EXTERNL hid_t
+H5FD_http_finalize(void)
+{
+    /* Reset VFL ID */
+    if (H5FD_HTTP_g && (H5Iis_valid(H5FD_HTTP_g) > 0))
+         H5FDunregister(H5FD_HTTP_g);
+    H5FD_HTTP_g = 0;
+
+    return H5FD_HTTP_g;
+} /* end H5FD_http_finalize() */
+
 
 /*---------------------------------------------------------------------------
  * Function:  H5FD_http_term
@@ -239,16 +256,11 @@ H5FD_http_init(void)
  *
  *---------------------------------------------------------------------------
  */
-#ifndef H5FDCLASS1
 static herr_t
 H5FD_http_term(void)
 {
-    /* Reset VFL ID */
-    H5FD_HTTP_g = 0;
-
     return 0;
 } /* end H5FD_http_term() */
-#endif
 
 
 /*-------------------------------------------------------------------------
@@ -334,10 +346,10 @@ H5FD_http_open( const char *name, unsigned flags, hid_t /*UNUSED*/ fapl_id,
     write_access = 0;
 
    /* Open file in read-only mode, to check for existence  and get length */
-    if((ncstat = nc_http_init(&state))) {
+    if((ncstat = nc_http_open(name,&state))) {
         H5Epush_ret(func, H5E_ERR_CLS, H5E_IO, H5E_CANTOPENFILE, "cannot access object", NULL);
     }
-    if((ncstat = nc_http_size(state,name,&len))) {
+    if((ncstat = nc_http_size(state,&len))) {
         H5Epush_ret(func, H5E_ERR_CLS, H5E_IO, H5E_CANTOPENFILE, "cannot access object", NULL);
     }
 
@@ -462,7 +474,7 @@ H5FD_http_query(const H5FD_t *_f, unsigned long /*OUT*/ *flags)
         *flags |= H5FD_FEAT_ACCUMULATE_METADATA;    /* OK to accumulate metadata for faster writes                      */
         *flags |= H5FD_FEAT_DATA_SIEVE;             /* OK to perform data sieving for faster raw data reads & writes    */
         *flags |= H5FD_FEAT_AGGREGATE_SMALLDATA;    /* OK to aggregate "small" raw data allocations                     */
-#ifndef H5FDCLASS1
+#if H5FD_CLASS_VERSION > 0
         *flags |= H5FD_FEAT_DEFAULT_VFD_COMPATIBLE; /* VFD creates a file which can be opened with the default VFD      */
 #endif
     }
@@ -595,19 +607,10 @@ H5FD_http_set_eoa(H5FD_t *_file, H5FD_mem_t /*UNUSED*/ type, haddr_t addr)
  */
 
 static haddr_t
-#ifdef H5FDCLASS1
-H5FD_http_get_eof(const H5FD_t *_file)
-#else
-H5FD_http_get_eof(const H5FD_t *_file, H5FD_mem_t /*UNUSED*/ type)
-#endif
+H5FD_http_get_eof(const H5FD_t *_file, H5FD_mem_t type)
 {
     const H5FD_http_t  *file = (const H5FD_http_t *)_file;
 
-#ifndef H5FDCLASS1
-    /* Quiet the compiler */
-    type = type;
-#endif
-
     /* Clear the error stack */
     H5Eclear2(H5E_DEFAULT);
 
@@ -716,7 +719,7 @@ H5FD_http_read(H5FD_t *_file, H5FD_mem_t /*UNUSED*/ type, hid_t /*UNUSED*/ dxpl_
 
     {
 	NCbytes* bbuf = ncbytesnew();
-        if((ncstat = nc_http_read(file->state,file->url,addr,size,bbuf))) {
+        if((ncstat = nc_http_read(file->state,addr,size,bbuf))) {
             file->op = H5FD_HTTP_OP_UNKNOWN;
             file->pos = HADDR_UNDEF;
 	    ncbytesfree(bbuf); bbuf = NULL;
@@ -795,18 +798,9 @@ H5FD_http_write(H5FD_t *_file, H5FD_mem_t /*UNUSED*/ type, hid_t /*UNUSED*/ dxpl
  *-------------------------------------------------------------------------
  */
 static herr_t
-#ifdef H5FDCLASS1
-H5FD_http_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
-#else
-H5FD_http_flush(H5FD_t *_file, hid_t /*UNUSED*/ dxpl_id, hbool_t closing)
-#endif
+H5FD_http_flush(H5FD_t *_file, hid_t dxpl_id, hbool_t closing)
 {
 
-#ifndef H5FDCLASS1
-    /* Quiet the compiler */
-    dxpl_id = dxpl_id;
-#endif
-
     /* Clear the error stack */
     H5Eclear2(H5E_DEFAULT);
 
@@ -830,23 +824,11 @@ H5FD_http_flush(H5FD_t *_file, hid_t /*UNUSED*/ dxpl_id, hbool_t closing)
  *-------------------------------------------------------------------------
  */
 static herr_t
-#ifdef H5FDCLASS1
-H5FD_http_lock(H5FD_t *_file, unsigned char* old, unsigned lock_type, hbool_t last)
-#else
 H5FD_http_lock(H5FD_t *_file, hbool_t rw)
-#endif
 {
     /* Clear the error stack */
     H5Eclear2(H5E_DEFAULT);
 
-#ifdef H5FDCLASS1
-    /* Quiet the compiler */
-    lock_type = lock_type;
-    last = last;
-#else
-    rw = rw;
-#endif
-
     return 0;
 } /* end H5FD_http_lock() */
 
@@ -866,21 +848,11 @@ H5FD_http_lock(H5FD_t *_file, hbool_t rw)
  *-------------------------------------------------------------------------
  */
 static herr_t
-#ifdef H5FDCLASS1
-H5FD_http_unlock(H5FD_t *file, /*UNUSED*/unsigned char *oid, /*UNUSED*/ hbool_t last)
-#else
-H5FD_http_unlock(H5FD_t *_file)
-#endif
+H5FD_http_unlock(H5FD_t *file)
 {
     /* Clear the error stack */
     H5Eclear2(H5E_DEFAULT);
 
-    /* Quiet the compiler */
-#ifdef H5FDCLASS1
-    oid = oid;
-    last = last;
-#endif
-
     return 0;
 } /* end H5FD_http_unlock() */
 
