@@ -10,6 +10,7 @@
  */
 
 #include "config.h"
+#include <stddef.h>
 #include "hdf5internal.h"
 #include "hdf5err.h"
 #include "hdf5debug.h"
@@ -18,7 +19,7 @@
 #include "ncauth.h"
 #include "ncmodel.h"
 #include "ncpathmgr.h"
-#include <stddef.h>
+#include "ncutil.h"
 
 #ifdef NETCDF_ENABLE_BYTERANGE
 #include "H5FDhttp.h"
@@ -443,8 +444,10 @@ create_phony_dims(NC_GRP_INFO_T *grp, hid_t hdf_datasetid, NC_VAR_INFO_T *var)
     }
     else
     {
-        /* Make sure it's scalar. */
-        assert(H5Sget_simple_extent_type(spaceid) == H5S_SCALAR);
+        /* Make sure it's scalar.  Null data space is possible in HDF5,
+         * but not allowed in netcdf data model. */
+        if (H5Sget_simple_extent_type(spaceid) != H5S_SCALAR)
+            BAIL(NC_EHDFERR);
     }
 
     /* Create a phony dimension for each dimension in the dataset,
@@ -907,9 +910,9 @@ nc4_open_file(const char *path, int mode, void* parameters, int ncid)
 		h5->uri = newuri;
 	        if((retval = NC_getactives3profile(h5->uri,&profile0)))
 		    BAIL(retval);
-   	        if((retval = NC_s3profilelookup(profile0,AWS_ACCESS_KEY_ID,&awsaccessid0)))
+   	        if((retval = NC_s3profilelookup(profile0,AWS_PROF_ACCESS_KEY_ID,&awsaccessid0)))
 		    BAIL(retval);		
-		if((retval = NC_s3profilelookup(profile0,AWS_SECRET_ACCESS_KEY,&awssecretkey0)))
+		if((retval = NC_s3profilelookup(profile0,AWS_PROF_SECRET_ACCESS_KEY,&awssecretkey0)))
 		    BAIL(retval);		
 		if(s3.region == NULL)
 		    s3.region = strdup(AWS_GLOBAL_DEFAULT_REGION);
