@@ -85,9 +85,9 @@ nc4_reopen_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
 
         if ((access_pid = H5Pcreate(H5P_DATASET_ACCESS)) < 0)
             return NC_EHDFERR;
-        if (H5Pset_chunk_cache(access_pid, var->chunk_cache_nelems,
-                               var->chunk_cache_size,
-                               var->chunk_cache_preemption) < 0)
+        if (H5Pset_chunk_cache(access_pid, var->chunkcache.nelems,
+                               var->chunkcache.size,
+                               var->chunkcache.preemption) < 0)
             return NC_EHDFERR;
         if (H5Dclose(hdf5_var->hdf_datasetid) < 0)
             return NC_EHDFERR;
@@ -121,11 +121,12 @@ give_var_secret_name(NC_VAR_INFO_T *var, const char *name)
      * clash. */
     if (strlen(name) + strlen(NON_COORD_PREPEND) > NC_MAX_NAME)
         return NC_EMAXNAME;
-    if (!(var->alt_name = malloc((strlen(NON_COORD_PREPEND) +
-                                   strlen(name) + 1) * sizeof(char))))
+    size_t alt_name_size = (strlen(NON_COORD_PREPEND) + strlen(name) + 1) *
+                           sizeof(char);
+    if (!(var->alt_name = malloc(alt_name_size)))
         return NC_ENOMEM;
 
-    sprintf(var->alt_name, "%s%s", NON_COORD_PREPEND, name);
+    snprintf(var->alt_name, alt_name_size, "%s%s", NON_COORD_PREPEND, name);
 
     return NC_NOERR;
 }
@@ -433,7 +434,7 @@ NC4_def_var(int ncid, const char *name, nc_type xtype, int ndims,
      * remember whether dimension scales have been attached to each
      * dimension. */
     if (!hdf5_var->dimscale && ndims)
-        if (!(hdf5_var->dimscale_attached = calloc(ndims, sizeof(nc_bool_t))))
+        if (!(hdf5_var->dimscale_attached = calloc((size_t)ndims, sizeof(nc_bool_t))))
             BAIL(NC_ENOMEM);
 
     /* Return the varid. */
@@ -465,6 +466,8 @@ exit:
  * @param no_fill Pointer to no_fill setting.
  * @param fill_value Pointer to fill value.
  * @param endianness Pointer to endianness setting.
+ * @param quantize_mode Pointer to quantization mode.
+ * @param nsd Pointer to number of significant digits.
  *
  * @returns ::NC_NOERR for success
  * @returns ::NC_EBADID Bad ncid.
@@ -484,7 +487,8 @@ static int
 nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
                  int *unused2, int *fletcher32, int *storage,
                  const size_t *chunksizes, int *no_fill,
-                 const void *fill_value, int *endianness)
+                 const void *fill_value, int *endianness,
+		 int *quantize_mode, int *nsd)
 {
     NC_GRP_INFO_T *grp;
     NC_FILE_INFO_T *h5;
@@ -504,7 +508,7 @@ nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
         return NC_EPERM;
 
     /* Find the var. */
-    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
         return NC_ENOTVAR;
     assert(var && var->hdr.id == varid);
 
@@ -530,31 +534,30 @@ nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
     }
 
     /* Shuffle filter? */
-    if (shuffle)
-    {
-        if(*shuffle) var->shuffle = *shuffle; /* Once set, cannot be unset */
-	if(var->shuffle)
-            var->storage = NC_CHUNKED;
+    if (shuffle && *shuffle) {
+	    retval = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_SHUFFLE,NULL,NULL);
+	    if(!retval || retval == NC_ENOFILTER) {
+	        if((retval = nc_def_var_filter(ncid,varid,H5Z_FILTER_SHUFFLE,0,NULL))) return retval;
+                var->storage = NC_CHUNKED;
+	    }
     }
 
     /* Fletcher32 checksum error protection? */
-    if (fletcher32)
-    {
-        if(*fletcher32) var->fletcher32 = *fletcher32; /* cannot be unset */
-	if(var->fletcher32)
+    if (fletcher32 && *fletcher32) {
+	retval = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_FLETCHER32,NULL,NULL);
+	if(!retval || retval == NC_ENOFILTER) {
+	    if((retval = nc_def_var_filter(ncid,varid,H5Z_FILTER_FLETCHER32,0,NULL))) return retval;
             var->storage = NC_CHUNKED;
+	    }
     }
 
 #ifdef USE_PARALLEL
-    /* If deflate, shuffle, or fletcher32 was turned on with
+    /* If filter is being applied with
      * parallel I/O writes, then switch to collective access. HDF5
      * requires collevtive access for filter use with parallel
      * I/O. */
-    if (shuffle || fletcher32)
-    {
-        if (h5->parallel && (nclistlength((NClist*)var->filters) > 0 || var->shuffle || var->fletcher32))
+    if (h5->parallel && (nclistlength((NClist*)var->filters) > 0))
             var->parallel_access = NC_COLLECTIVE;
-    }
 #endif /* USE_PARALLEL */
 
     /* Handle storage settings. */
@@ -565,7 +568,7 @@ nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
          * no filters in use for this data. */
         if (*storage != NC_CHUNKED)
         {
-            if (nclistlength(((NClist*)var->filters)) > 0 || var->fletcher32 || var->shuffle)
+            if (nclistlength(((NClist*)var->filters)) > 0)
                 return NC_EINVAL;
 	    for (d = 0; d < var->ndims; d++)
                 if (var->dim[d]->unlimited)
@@ -663,21 +666,27 @@ nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
     }
 
     /* Are we setting a fill value? */
-    if (fill_value && !var->no_fill)
+    if (fill_value && no_fill && !(*no_fill))
     {
         /* Copy the fill_value. */
         LOG((4, "Copying fill value into metadata for variable %s",
              var->hdr.name));
 
         /* If there's a _FillValue attribute, delete it. */
-        retval = NC4_HDF5_del_att(ncid, varid, _FillValue);
+        retval = NC4_HDF5_del_att(ncid, varid, NC_FillValue);
         if (retval && retval != NC_ENOTATT)
             return retval;
 
-        /* Create a _FillValue attribute. */
-        if ((retval = nc_put_att(ncid, varid, _FillValue, var->type_info->hdr.id,
+        /* Create a _FillValue attribute; will also fill in var->fill_value */
+        if ((retval = nc_put_att(ncid, varid, NC_FillValue, var->type_info->hdr.id,
                                  1, fill_value)))
             return retval;
+    } else if (var->fill_value && no_fill && (*no_fill)) { /* Turning off fill value? */
+        /* If there's a _FillValue attribute, delete it. */
+        retval = NC4_HDF5_del_att(ncid, varid, NC_FillValue);
+        if (retval && retval != NC_ENOTATT) return retval;
+	if((retval = NC_reclaim_data_all(h5->controller,var->type_info->hdr.id,var->fill_value,1))) return retval;
+	var->fill_value = NULL;
     }
 
     /* Is the user setting the endianness? */
@@ -706,6 +715,65 @@ nc_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	var->endianness = *endianness;
     }
 
+    /* Remember quantization settings. They will be used when data are
+     * written.
+     * Code block is identical to one in zvar.c---consider functionalizing */
+    if (quantize_mode)
+    {
+	/* Only four valid mode settings. */
+	if (*quantize_mode != NC_NOQUANTIZE &&
+	    *quantize_mode != NC_QUANTIZE_BITGROOM &&
+	    *quantize_mode != NC_QUANTIZE_GRANULARBR &&
+	    *quantize_mode != NC_QUANTIZE_BITROUND)
+	    return NC_EINVAL;
+
+	if (*quantize_mode == NC_QUANTIZE_BITGROOM ||
+	    *quantize_mode == NC_QUANTIZE_GRANULARBR ||
+	    *quantize_mode == NC_QUANTIZE_BITROUND)
+	{
+	    /* Only float and double types can have quantization. */
+	    if (var->type_info->hdr.id != NC_FLOAT &&
+		var->type_info->hdr.id != NC_DOUBLE)
+		return NC_EINVAL;
+	    
+	    
+	    /* All quantization codecs require number of significant digits */
+	    if (!nsd)
+		return NC_EINVAL;
+
+	    /* NSD must be in range. */
+	    if (*nsd <= 0)
+		return NC_EINVAL;
+
+	    if (*quantize_mode == NC_QUANTIZE_BITGROOM ||
+		*quantize_mode == NC_QUANTIZE_GRANULARBR)
+	      { 
+		if (var->type_info->hdr.id == NC_FLOAT &&
+		    *nsd > NC_QUANTIZE_MAX_FLOAT_NSD)
+		  return NC_EINVAL;
+		if (var->type_info->hdr.id == NC_DOUBLE &&
+		    *nsd > NC_QUANTIZE_MAX_DOUBLE_NSD)
+		  return NC_EINVAL;
+	      }
+	    else if (*quantize_mode == NC_QUANTIZE_BITROUND)
+	      {
+		if (var->type_info->hdr.id == NC_FLOAT &&
+		    *nsd > NC_QUANTIZE_MAX_FLOAT_NSB)
+		  return NC_EINVAL;
+		if (var->type_info->hdr.id == NC_DOUBLE &&
+		    *nsd > NC_QUANTIZE_MAX_DOUBLE_NSB)
+		  return NC_EINVAL;
+	      }
+	    var->nsd = *nsd;
+	}
+	
+	var->quantize_mode = *quantize_mode;
+
+	/* If quantization is turned off, then set nsd to 0. */
+	if (*quantize_mode == NC_NOQUANTIZE)
+	    var->nsd = 0;
+    }
+
     return NC_NOERR;
 }
 
@@ -734,16 +802,150 @@ int
 NC4_def_var_deflate(int ncid, int varid, int shuffle, int deflate,
                     int deflate_level)
 {
-    int stat = NC_NOERR;
+    int stat;
     unsigned int level = (unsigned int)deflate_level;
-    /* Set shuffle first */
-    if((stat = nc_def_var_extra(ncid, varid, &shuffle, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL))) goto done;
-    if(deflate) {
-        if((stat = nc_def_var_filter(ncid, varid, H5Z_FILTER_DEFLATE,1,&level))) goto done;
-    } /* else ignore */
 
-done:
-    return stat;
+    /* Set shuffle first */
+    if ((stat = nc_def_var_extra(ncid, varid, &shuffle, NULL, NULL, NULL,
+                                 NULL, NULL, NULL, NULL, NULL, NULL, NULL)))
+        return stat;
+
+    /* Don't turn on deflate if deflate_level = 0. It's a valid zlib
+     * setting, but results in a write slowdown, and a file that is
+     * larger than the uncompressed file would be. So when
+     * deflate_level is 0, don't use compression. */
+    if (deflate && deflate_level)
+        if ((stat = nc_def_var_filter(ncid, varid, H5Z_FILTER_DEFLATE, 1, &level)))
+            return stat;
+
+    return NC_NOERR;
+}
+
+/**
+ * @internal Set quantization settings on a variable. This is
+ * called by nc_def_var_quantize().
+ *
+ * Quantization allows the user to specify a number of significant
+ * digits for variables of type ::NC_FLOAT or ::NC_DOUBLE. (Attempting
+ * to set quantize for other types will result in an ::NC_EINVAL
+ * error.)
+ *
+ * When quantize is turned on, and the number of significant digits
+ * (NSD) has been specified, then the netCDF library will quantize according
+ * to the selected algorithm. BitGroom interprets NSD as decimal digits
+ * will apply all zeros or all ones (alternating) to bits which are not 
+ * needed to specify the value to the number of significant decimal digits. 
+ * BitGroom retain the same number of bits for all values of a variable. 
+ * BitRound (BR) interprets NSD as binary digits (i.e., bits) and keeps the
+ * the user-specified number of significant bits then rounds the result
+ * to the nearest representable number according to IEEE rounding rules.
+ * BG and BR both retain a uniform number of significant bits for all 
+ * values of a variable. Granular BitRound interprest NSD as decimal
+ * digits. GranularBR determines the number of bits to necessary to 
+ * retain the user-specified number of significant digits individually
+ * for every value of the variable. GranularBR then applies the BR
+ * quantization algorithm on a granular, value-by-value, rather than
+ * uniformly for the entire variable. GranularBR quantizes more bits
+ * than BG, and is thus more compressive and less accurate than BG.
+ * BR knows bits and makes no guarantees about decimal precision.
+ * All quantization algorithms change the values of the data, and make 
+ * it more compressible.
+ *
+ * Quantizing the data does not reduce the size of the data on disk,
+ * but combining quantize with compression will allow for better
+ * compression. Since the data values are changed, the use of quantize
+ * and compression such as DEFLATE constitute lossy compression.
+ *
+ * Producers of large datasets may find that using quantize with
+ * compression will result in significant improvement in the final data
+ * size.
+ *
+ * Variables which use quantize will have added an attribute with name
+ * ::NC_QUANTIZE_BITGROOM_ATT_NAME, ::NC_QUANTIZE_GRANULARBR_ATT_NAME,
+ * or ::NC_QUANTIZE_BITROUND_ATT_NAME that contains the number of
+ * significant digits. Users should not delete or change this attribute.
+ * This is the only record that quantize has been applied to the data.
+ *
+ * As with the deflate settings, quantize settings may only be
+ * modified before the first call to nc_enddef(). Once nc_enddef() is
+ * called for the file, quantize settings for any variable in the file
+ * may not be changed.
+ *
+ * Use of quantization is fully backwards compatible with existing
+ * versions and packages that can read compressed netCDF data. A
+ * variable which has been quantized is readable to older versions of
+ * the netCDF libraries, and to netCDF-Java.
+ *
+ * @param ncid File ID.
+ * @param varid Variable ID. NC_GLOBAL may not be used.
+ * @param quantize_mode Quantization mode. May be ::NC_NOQUANTIZE or
+ * ::NC_QUANTIZE_BITGROOM, ::NC_QUANTIZE_BITROUND or ::NC_QUANTIZE_GRANULARBR.
+ * @param nsd Number of significant digits (either decimal or binary). 
+ * May be any integer from 1 to ::NC_QUANTIZE_MAX_FLOAT_NSD (for variables 
+ * of type ::NC_FLOAT) or ::NC_QUANTIZE_MAX_DOUBLE_NSD (for variables 
+ * of type ::NC_DOUBLE) for mode ::NC_QUANTIZE_BITGROOM and mode
+ * ::NC_QUANTIZE_GRANULARBR. May be any integer from 1 to 
+ * ::NC_QUANTIZE_MAX_FLOAT_NSB (for variables of type ::NC_FLOAT) or 
+ * ::NC_QUANTIZE_MAX_DOUBLE_NSB (for variables of type ::NC_DOUBLE) 
+ * for mode ::NC_QUANTIZE_BITROUND.
+ *
+ * @returns ::NC_NOERR No error.
+ * @returns ::NC_EBADID Bad ncid.
+ * @returns ::NC_ENOTVAR Invalid variable ID.
+ * @returns ::NC_ENOTNC4 Attempting netcdf-4 operation on file that is
+ * not netCDF-4/HDF5.
+ * @returns ::NC_ELATEDEF Too late to change settings for this variable.
+ * @returns ::NC_ENOTINDEFINE Not in define mode.
+ * @returns ::NC_EINVAL Invalid input
+ * @author Ed Hartnett, Dennis Heimbigner
+ */
+int
+NC4_def_var_quantize(int ncid, int varid, int quantize_mode, int nsd)
+{
+    return nc_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
+			    NULL, NULL, NULL, NULL, NULL,
+			    &quantize_mode, &nsd);
+}
+
+/**
+ * @internal Get quantize information about a variable. Pass NULL for
+ * whatever you don't care about.
+ *
+ * @param ncid File ID.
+ * @param varid Variable ID.
+ * @param quantize_modep Gets quantize mode.
+ * @param nsdp Gets Number of Significant Digits if quantize is in use.
+ *
+ * @returns ::NC_NOERR No error.
+ * @returns ::NC_EBADID Bad ncid.
+ * @returns ::NC_ENOTVAR Bad varid.
+ * @returns ::NC_EINVAL Invalid input.
+ * @author Ed Hartnett
+ */
+int
+NC4_inq_var_quantize(int ncid, int varid, int *quantize_modep,
+		     int *nsdp)
+{
+    NC_VAR_INFO_T *var;
+    int retval;
+
+    LOG((2, "%s: ncid 0x%x varid %d", __func__, ncid, varid));
+
+    /* Find info for this file and group, and set pointer to each. */
+    /* Get pointer to the var. */
+    if ((retval = nc4_hdf5_find_grp_h5_var(ncid, varid, NULL, NULL, &var)))
+        return retval;
+    if (!var)
+        return NC_ENOTVAR;	
+    assert(var->hdr.id == varid);
+
+    /* Copy the data to the user's data buffers. */
+    if (quantize_modep)
+        *quantize_modep = var->quantize_mode;
+    if (nsdp)
+        *nsdp = var->nsd;
+
+    return 0;
 }
 
 #if 0
@@ -803,7 +1005,7 @@ int
 NC4_def_var_fletcher32(int ncid, int varid, int fletcher32)
 {
     return nc_def_var_extra(ncid, varid, NULL, NULL, NULL, &fletcher32,
-                            NULL, NULL, NULL, NULL, NULL);
+                            NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 /**
@@ -832,12 +1034,13 @@ int
 NC4_def_var_chunking(int ncid, int varid, int storage, const size_t *chunksizesp)
 {
     return nc_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
-                            &storage, chunksizesp, NULL, NULL, NULL);
+                            &storage, chunksizesp, NULL, NULL, NULL, NULL, NULL);
 }
 
 /**
  * @internal Define chunking stuff for a var. This is called by
  * the fortran API.
+ * Note: see libsrc4/nc4cache.c for definition when HDF5 is disabled
  *
  * @param ncid File ID.
  * @param varid Variable ID.
@@ -874,10 +1077,10 @@ nc_def_var_chunking_ints(int ncid, int varid, int storage, int *chunksizesp)
 
     /* Copy to size_t array. */
     for (i = 0; i < var->ndims; i++)
-        cs[i] = chunksizesp[i];
+        cs[i] = (size_t)chunksizesp[i];
 
     retval = nc_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
-                              &storage, cs, NULL, NULL, NULL);
+                              &storage, cs, NULL, NULL, NULL, NULL, NULL);
 
     if (var->ndims)
         free(cs);
@@ -911,7 +1114,7 @@ int
 NC4_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value)
 {
     return nc_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL, NULL,
-                            NULL, &no_fill, fill_value, NULL);
+                            NULL, &no_fill, fill_value, NULL, NULL, NULL);
 }
 
 /**
@@ -940,7 +1143,7 @@ int
 NC4_def_var_endian(int ncid, int varid, int endianness)
 {
     return nc_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL, NULL,
-                            NULL, NULL, NULL, &endianness);
+                            NULL, NULL, NULL, &endianness, NULL, NULL);
 }
 
 /**
@@ -1014,7 +1217,7 @@ NC4_rename_var(int ncid, int varid, const char *name)
         return retval;
 
     /* Get the variable wrt varid */
-    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
         return NC_ENOTVAR;
 
     /* Check if new name is in use; note that renaming to same name is
@@ -1054,7 +1257,6 @@ NC4_rename_var(int ncid, int varid, const char *name)
        there. */
     if (var->created)
     {
-        int v;
         char *hdf5_name; /* Dataset will be renamed to this. */
         hdf5_name = use_secret_name ? var->alt_name: (char *)name;
 
@@ -1088,7 +1290,7 @@ NC4_rename_var(int ncid, int varid, const char *name)
          * and we have just changed that for this var. We must do the
          * same for all vars with a > varid, so that the creation order
          * will continue to be correct. */
-        for (v = var->hdr.id + 1; v < ncindexsize(grp->vars); v++)
+        for (size_t v = (size_t)var->hdr.id + 1; v < ncindexsize(grp->vars); v++)
         {
             NC_VAR_INFO_T *my_var;
             my_var = (NC_VAR_INFO_T *)ncindexith(grp->vars, v);
@@ -1163,8 +1365,10 @@ NC4_rename_var(int ncid, int varid, const char *name)
  *
  * @param ncid File ID.
  * @param varid Variable ID.
- * @param startp Array of start indices.
- * @param countp Array of counts.
+ * @param startp Array of start indices. This array must be same size
+ * as variable's number of dimensions.
+ * @param countp Array of counts. This array must be same size as
+ * variable's number of dimensions.
  * @param op pointer that gets the data.
  * @param memtype The type of these data in memory.
  *
@@ -1185,11 +1389,12 @@ NC4_put_vara(int ncid, int varid, const size_t *startp,
  *
  * @param ncid File ID.
  * @param varid Variable ID.
- * @param startp Array of start indices.
+ * @param startp Array of start indices. This array must be same size
+ * as variable's number of dimensions.
  * @param countp Array of counts.
  * @param ip pointer that gets the data.
  * @param memtype The type of these data after it is read into memory.
-
+ *
  * @returns ::NC_NOERR for success
  * @author Ed Hartnett, Dennis Heimbigner
  */
@@ -1310,11 +1515,13 @@ set_par_access(NC_FILE_INFO_T *h5, NC_VAR_INFO_T *var, hid_t xfer_plistid)
  * @param ncid File ID.
  * @param varid Variable ID.
  * @param startp Array of start indices. Must always be provided by
- * caller for non-scalar vars.
+ * caller for non-scalar vars. This array must be same size
+ * as variable's number of dimensions.
  * @param countp Array of counts. Will default to counts of full
- * dimension size if NULL.
+ * dimension size if NULL. This array must be same size
+ * as variable's number of dimensions.
  * @param stridep Array of strides. Will default to strides of 1 if
- * NULL.
+ * NULL. This array must be same size as variable's number of dimensions.
  * @param data The data to be written.
  * @param mem_nc_type The type of the data in memory.
  *
@@ -1339,11 +1546,12 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     NC_VAR_INFO_T *var;
     NC_DIM_INFO_T *dim;
     NC_HDF5_VAR_INFO_T *hdf5_var;
+    herr_t herr;
     hid_t file_spaceid = 0, mem_spaceid = 0, xfer_plistid = 0;
     long long unsigned xtend_size[NC_MAX_VAR_DIMS];
     hsize_t fdims[NC_MAX_VAR_DIMS], fmaxdims[NC_MAX_VAR_DIMS];
     hsize_t start[NC_MAX_VAR_DIMS], count[NC_MAX_VAR_DIMS];
-    hsize_t stride[NC_MAX_VAR_DIMS];
+    hsize_t stride[NC_MAX_VAR_DIMS], ones[NC_MAX_VAR_DIMS];
     int need_to_extend = 0;
 #ifdef USE_PARALLEL4
     int extend_possible = 0;
@@ -1393,7 +1601,8 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
 
         start[i] = startp[i];
         count[i] = countp ? countp[i] : var->dim[i]->len;
-        stride[i] = stridep ? stridep[i] : 1;
+        stride[i] = stridep ? (hsize_t)stridep[i] : 1;
+        ones[i] = 1;
 	LOG((4, "start[%d] %ld count[%d] %ld stride[%d] %ld", i, start[i], i, count[i], i, stride[i]));
 
         /* Check to see if any counts are zero. */
@@ -1444,20 +1653,27 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     }
     else
     {
-        if (H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET, start, stride,
-                                count, NULL) < 0)
+        if (stridep == NULL)
+            herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET, start,
+                                       NULL, ones, count);
+        else
+            herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET, start,
+                                       stride, count, NULL);
+        if (herr < 0)
             BAIL(NC_EHDFERR);
 
         /* Create a space for the memory, just big enough to hold the slab
            we want. */
-        if ((mem_spaceid = H5Screate_simple(var->ndims, count, NULL)) < 0)
+        if ((mem_spaceid = H5Screate_simple((int)var->ndims, count, NULL)) < 0)
             BAIL(NC_EHDFERR);
     }
 
     /* Are we going to convert any data? (No converting of compound or
-     * opaque types.) */
-    if (mem_nc_type != var->type_info->hdr.id &&
-        mem_nc_type != NC_COMPOUND && mem_nc_type != NC_OPAQUE)
+     * opaque types.) We also need to call this code if we are doing
+     * quantization. */
+    if ((mem_nc_type != var->type_info->hdr.id &&
+	 mem_nc_type != NC_COMPOUND && mem_nc_type != NC_OPAQUE) ||
+	var->quantize_mode)
     {
         size_t file_type_size;
 
@@ -1494,9 +1710,9 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
         BAIL(retval);
 #endif
 
-    /* Read this hyperslab from memory. Does the dataset have to be
-       extended? If it's already extended to the required size, it will
-       do no harm to reextend it to that size. */
+    /* Does the dataset have to be extended? If it's already extended
+       to the required size, it will do no harm to reextend it to that
+       size. */
     if (var->ndims)
     {
         for (d2 = 0; d2 < var->ndims; d2++)
@@ -1515,15 +1731,10 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
                 {
                     xtend_size[d2] = (long long unsigned)(endindex + 1);
                     need_to_extend++;
+                    dim->extended = NC_TRUE;
                 }
                 else
                     xtend_size[d2] = (long long unsigned)fdims[d2];
-
-                if (!zero_count && endindex >= dim->len)
-                {
-                    dim->len = endindex + 1;
-                    dim->extended = NC_TRUE;
-                }
             }
             else
             {
@@ -1573,8 +1784,14 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
                 BAIL2(NC_EHDFERR);
             if ((file_spaceid = H5Dget_space(hdf5_var->hdf_datasetid)) < 0)
                 BAIL(NC_EHDFERR);
-            if (H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
-                                    start, stride, count, NULL) < 0)
+
+            if (stridep == NULL)
+                herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
+                                           start, NULL, ones, count);
+            else
+                herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
+                                           start, stride, count, NULL);
+            if (herr < 0)
                 BAIL(NC_EHDFERR);
         }
     }
@@ -1584,7 +1801,8 @@ NC4_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     {
         if ((retval = nc4_convert_type(data, bufr, mem_nc_type, var->type_info->hdr.id,
                                        len, &range_error, var->fill_value,
-                                       (h5->cmode & NC_CLASSIC_MODEL))))
+                                       (h5->cmode & NC_CLASSIC_MODEL), var->quantize_mode,
+				       var->nsd)))
             BAIL(retval);
     }
 
@@ -1635,11 +1853,13 @@ exit:
  * @param ncid File ID.
  * @param varid Variable ID.
  * @param startp Array of start indices. Must be provided for
- * non-scalar vars.
+ * non-scalar vars. This array must be same size as variable's number
+ * of dimensions.
  * @param countp Array of counts. Will default to counts of extent of
- * dimension if NULL.
+ * dimension if NULL. This array must be same size as variable's
+ * number of dimensions.
  * @param stridep Array of strides. Will default to strides of 1 if
- * NULL.
+ * NULL. This array must be same size as variable's number of dimensions.
  * @param data The data to be written.
  * @param mem_nc_type The type of the data in memory. (Convert to this
  * type from file type.)
@@ -1672,7 +1892,7 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     hsize_t count[NC_MAX_VAR_DIMS];
     hsize_t fdims[NC_MAX_VAR_DIMS], fmaxdims[NC_MAX_VAR_DIMS];
     hsize_t start[NC_MAX_VAR_DIMS];
-    hsize_t stride[NC_MAX_VAR_DIMS];
+    hsize_t stride[NC_MAX_VAR_DIMS], ones[NC_MAX_VAR_DIMS];
     void *fillvalue = NULL;
     int no_read = 0, provide_fill = 0;
     hssize_t fill_value_size[NC_MAX_VAR_DIMS];
@@ -1680,6 +1900,9 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     void *bufr = NULL;
     int need_to_convert = 0;
     size_t len = 1;
+    int fixedlengthstring = 0;
+    hsize_t fstring_len = 0;
+    size_t fstring_count = 1;
 
     /* Find info for this file, group, and var. */
     if ((retval = nc4_hdf5_find_grp_h5_var(ncid, varid, &h5, &grp, &var)))
@@ -1719,7 +1942,8 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
 
         start[i] = startp[i];
         count[i] = countp[i];
-        stride[i] = stridep ? stridep[i] : 1;
+        stride[i] = stridep ? (hsize_t)stridep[i] : 1;
+        ones[i] = 1;
 
         /* if any of the count values are zero don't actually read. */
         if (count[i] == 0)
@@ -1849,12 +2073,19 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
         }
         else
         {
-            if (H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
-                                    start, stride, count, NULL) < 0)
+            herr_t herr;
+            if (stridep == NULL)
+                herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
+                                           start, NULL, ones, count);
+            else
+                herr = H5Sselect_hyperslab(file_spaceid, H5S_SELECT_SET,
+                                           start, stride, count, NULL);
+            if (herr < 0)
                 BAIL(NC_EHDFERR);
+
             /* Create a space for the memory, just big enough to hold the slab
                we want. */
-            if ((mem_spaceid = H5Screate_simple(var->ndims, count, NULL)) < 0)
+            if ((mem_spaceid = H5Screate_simple((int)var->ndims, count, NULL)) < 0)
                 BAIL(NC_EHDFERR);
         }
 
@@ -1868,14 +2099,21 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
             H5Tget_size(hdf5_type->hdf_typeid) > 1 &&
             !H5Tis_variable_str(hdf5_type->hdf_typeid))
         {
-            hsize_t fstring_len;
+	    size_t k;
 
             if ((fstring_len = H5Tget_size(hdf5_type->hdf_typeid)) == 0)
                 BAIL(NC_EHDFERR);
-            if (!(*(char **)data = malloc(1 + fstring_len)))
-                BAIL(NC_ENOMEM);
-            bufr = *(char **)data;
-        }
+	    /* Compute the total number of strings to read */
+            if (var->ndims) {
+                for (k = 0; k < var->ndims; k++) {
+                    fstring_count *= countp[k];
+		}
+	    }
+	    /* Allocate space for the all the strings */
+            if (!(bufr = malloc(fstring_len*fstring_count)))
+                    BAIL(NC_ENOMEM);
+	    fixedlengthstring = 1;
+	}
 
         /* Create the data transfer property list. */
         if ((xfer_plistid = H5Pcreate(H5P_DATASET_XFER)) < 0)
@@ -1928,6 +2166,24 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
         }
 #endif /* USE_PARALLEL4 */
     }
+
+    /* If we read a sequence of fixed length strings, then we need to convert to char* in memory */
+    if(fixedlengthstring) {
+	size_t k;
+	char** strvec = (char**)data;
+	char* p;
+	for(k=0;k < fstring_count;k++) {
+	    char* eol;
+	    if((p = (char*)malloc(1+fstring_len))==NULL) BAIL(NC_ENOMEM);
+	    memcpy(p,((char*)bufr)+(k*fstring_len),fstring_len);
+	    eol = p + fstring_len;
+	    *eol = '\0';
+	    strvec[k] = p; p = NULL;
+	}
+	free(bufr);
+	bufr = NULL;
+    }
+
     /* Now we need to fake up any further data that was asked for,
        using the fill values instead. First skip past the data we
        just read, if any. */
@@ -1956,27 +2212,11 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
         for (i = 0; i < fill_len; i++)
         {
 
-            if (var->type_info->nc_type_class == NC_STRING)
-            {
-                if (*(char **)fillvalue)
-                {
-                    if (!(*(char **)filldata = strdup(*(char **)fillvalue)))
-                        BAIL(NC_ENOMEM);
-                }
-                else
-                    *(char **)filldata = NULL;
-            }
-            else if (var->type_info->nc_type_class == NC_VLEN)
-            {
-                if (fillvalue)
-                {
-                    memcpy(filldata,fillvalue,file_type_size);
-                } else {
-                    *(char **)filldata = NULL;
-                }
-            }
-            else
-                memcpy(filldata, fillvalue, file_type_size);
+	    {
+		/* Copy one instance of the fill_value */
+		if((retval = NC_copy_data(h5->controller,var->type_info->hdr.id,fillvalue,1,filldata)))
+		    BAIL(retval);
+	    }
             filldata = (char *)filldata + file_type_size;
 	}        
     }
@@ -1985,8 +2225,8 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     if (need_to_convert)
     {
         if ((retval = nc4_convert_type(bufr, data, var->type_info->hdr.id, mem_nc_type,
-                                           len, &range_error, var->fill_value,
-                                           (h5->cmode & NC_CLASSIC_MODEL))))
+				       len, &range_error, var->fill_value,
+				       (h5->cmode & NC_CLASSIC_MODEL), var->quantize_mode, var->nsd)))
             BAIL(retval);
 
         /* For strict netcdf-3 rules, ignore erange errors between UBYTE
@@ -1999,6 +2239,7 @@ NC4_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     }
     
 exit:
+    if(fixedlengthstring && bufr) free(bufr);
     if (file_spaceid > 0)
         if (H5Sclose(file_spaceid) < 0)
             BAIL2(NC_EHDFERR);
@@ -2053,6 +2294,8 @@ exit:
  * @param nparamsp Pointer to memory to store filter parameter count.
  * @param params Pointer to vector of unsigned integers into which
  * to store filter parameters.
+ * @param quantize_modep Gets quantization mode.
+ * @param nsdp Gets number of significant digits, if quantization is in use.
  *
  * @returns ::NC_NOERR No error.
  * @returns ::NC_EBADID Bad ncid.
@@ -2129,14 +2372,14 @@ NC4_HDF5_set_var_chunk_cache(int ncid, int varid, size_t size, size_t nelems,
     assert(grp && h5);
 
     /* Find the var. */
-    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
         return NC_ENOTVAR;
     assert(var && var->hdr.id == varid);
 
     /* Set the values. */
-    var->chunk_cache_size = size;
-    var->chunk_cache_nelems = nelems;
-    var->chunk_cache_preemption = preemption;
+    var->chunkcache.size = size;
+    var->chunkcache.nelems = nelems;
+    var->chunkcache.preemption = preemption;
 
     /* Reopen the dataset to bring new settings into effect. */
     if ((retval = nc4_reopen_dataset(grp, var)))
@@ -2147,6 +2390,7 @@ NC4_HDF5_set_var_chunk_cache(int ncid, int varid, size_t size, size_t nelems,
 /**
  * @internal A wrapper for NC4_set_var_chunk_cache(), we need this
  * version for fortran. Negative values leave settings as they are.
+ * Note: see libsrc4/nc4cache.c for definition when HDF5 is disabled
  *
  * @param ncid File ID.
  * @param varid Variable ID.
@@ -2172,10 +2416,10 @@ nc_set_var_chunk_cache_ints(int ncid, int varid, int size, int nelems,
         real_size = ((size_t) size) * MEGABYTE;
 
     if (nelems >= 0)
-        real_nelems = nelems;
+        real_nelems = (size_t)nelems;
 
     if (preemption >= 0)
-        real_preemption = preemption / 100.;
+        real_preemption = (float)(preemption / 100.);
 
     return NC4_HDF5_set_var_chunk_cache(ncid, varid, real_size, real_nelems,
                                         real_preemption);
