@@ -22,6 +22,7 @@
 #include "hdf5err.h" /* For BAIL2 */
 #include "hdf5debug.h"
 #include <math.h>
+#include <stddef.h>
 
 #ifdef HAVE_INTTYPES_H
 #define __STDC_FORMAT_MACROS
@@ -42,13 +43,12 @@ static int
 flag_atts_dirty(NCindex *attlist) {
 
     NC_ATT_INFO_T *att = NULL;
-    int i;
 
     if(attlist == NULL) {
         return NC_NOERR;
     }
 
-    for(i=0;i<ncindexsize(attlist);i++) {
+    for(size_t i=0;i<ncindexsize(attlist);i++) {
         att = (NC_ATT_INFO_T*)ncindexith(attlist,i);
         if(att == NULL) continue;
         att->dirty = NC_TRUE;
@@ -70,7 +70,7 @@ flag_atts_dirty(NCindex *attlist) {
  * @param dimscaleid HDF5 dimension scale ID.
  *
  * @returns NC_NOERR No error.
- * @returns NC_EHDFERR HDF5 returned an error.
+ * @returns NC_EDIMSCALE HDF5 returned an error when trying to reattach a dimension scale.
  * @author Ed Hartnett
  */
 int
@@ -78,7 +78,7 @@ rec_reattach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
 {
     NC_VAR_INFO_T *var;
     NC_GRP_INFO_T *child_grp;
-    int d, i;
+    size_t i;
     int retval;
 
     assert(grp && grp->hdr.name && dimid >= 0 && dimscaleid >= 0);
@@ -100,11 +100,10 @@ rec_reattach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
 
         var = (NC_VAR_INFO_T*)ncindexith(grp->vars,i);
         assert(var && var->format_var_info);
-        hdf5_var = (NC_HDF5_VAR_INFO_T *)var->format_var_info;
 
 	hdf5_var = (NC_HDF5_VAR_INFO_T*)var->format_var_info;	
 	assert(hdf5_var != NULL);
-        for (d = 0; d < var->ndims; d++)
+        for (unsigned int d = 0; d < var->ndims; d++)
         {
             if (var->dimids[d] == dimid && !hdf5_var->dimscale)
             {
@@ -114,7 +113,7 @@ rec_reattach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
                 {
                     if (H5DSattach_scale(hdf5_var->hdf_datasetid,
                                          dimscaleid, d) < 0)
-                        return NC_EHDFERR;
+                        return NC_EDIMSCALE;
                     hdf5_var->dimscale_attached[d] = NC_TRUE;
                 }
             }
@@ -136,7 +135,7 @@ rec_reattach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
  * @param dimscaleid HDF5 dimension scale ID.
  *
  * @returns NC_NOERR No error.
- * @returns NC_EHDFERR HDF5 returned an error.
+ * @returns NC_EDIMSCALE HDF5 returned an error when trying to detach a dimension scale.
  * @author Ed Hartnett
  */
 int
@@ -144,7 +143,7 @@ rec_detach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
 {
     NC_VAR_INFO_T *var;
     NC_GRP_INFO_T *child_grp;
-    int d, i;
+    size_t i;
     int retval;
 
     assert(grp && grp->hdr.name && dimid >= 0 && dimscaleid >= 0);
@@ -166,7 +165,7 @@ rec_detach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
         assert(var && var->format_var_info);
         hdf5_var = (NC_HDF5_VAR_INFO_T *)var->format_var_info;
 
-        for (d = 0; d < var->ndims; d++)
+        for (unsigned int d = 0; d < var->ndims; d++)
         {
             if (var->dimids[d] == dimid && !hdf5_var->dimscale)
             {
@@ -178,7 +177,7 @@ rec_detach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid)
                     {
                         if (H5DSdetach_scale(hdf5_var->hdf_datasetid,
                                              dimscaleid, d) < 0)
-                            return NC_EHDFERR;
+                            return NC_EDIMSCALE;
                         hdf5_var->dimscale_attached[d] = NC_FALSE;
                     }
                 }
@@ -208,7 +207,7 @@ nc4_open_var_grp2(NC_GRP_INFO_T *grp, int varid, hid_t *dataset)
     assert(grp && grp->format_grp_info && dataset);
 
     /* Find the requested varid. */
-    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
         return NC_ENOTVAR;
     assert(var && var->hdr.id == varid && var->format_var_info);
     hdf5_var = (NC_HDF5_VAR_INFO_T *)var->format_var_info;
@@ -461,7 +460,7 @@ put_att_grpa(NC_GRP_INFO_T *grp, int varid, NC_ATT_INFO_T *att)
 
     /* Get the length ready, and find the HDF type we'll be
      * writing. */
-    dims[0] = att->len;
+    dims[0] = (hsize_t)att->len;
     if ((retval = nc4_get_hdf_typeid(grp->nc4_info, att->nc_typeid,
                                      &file_typeid, 0)))
         BAIL(retval);
@@ -471,12 +470,8 @@ put_att_grpa(NC_GRP_INFO_T *grp, int varid, NC_ATT_INFO_T *att)
      * some phoney data (which won't be written anyway.)*/
     if (!dims[0])
         data = &phoney_data;
-    else if (att->data)
-        data = att->data;
-    else if (att->stdata)
-        data = att->stdata;
     else
-        data = att->vldata;
+        data = att->data;
 
     /* NC_CHAR types require some extra work. The space ID is set to
      * scalar, and the type is told how long the string is. If it's
@@ -516,33 +511,30 @@ put_att_grpa(NC_GRP_INFO_T *grp, int varid, NC_ATT_INFO_T *att)
         }
     }
 
-    /* Does the att exists already? */
+    /* Does the att exist already? */
     if ((attr_exists = H5Aexists(locid, att->hdr.name)) < 0)
         BAIL(NC_EHDFERR);
     if (attr_exists)
     {
-        hssize_t npoints;
-
-        /* Open the attribute. */
+        /* Open the existing attribute. */
         if ((existing_attid = H5Aopen(locid, att->hdr.name, H5P_DEFAULT)) < 0)
             BAIL(NC_EATTMETA);
 
-        /* Find the type of the existing attribute. */
+        /* Get the HDF5 datatype of the existing attribute. */
         if ((existing_att_typeid = H5Aget_type(existing_attid)) < 0)
             BAIL(NC_EATTMETA);
 
-        /* How big is the attribute? */
+        /* Get the HDF5 dataspace of the existing attribute. */
         if ((existing_spaceid = H5Aget_space(existing_attid)) < 0)
             BAIL(NC_EATTMETA);
-        if ((npoints = H5Sget_simple_extent_npoints(existing_spaceid)) < 0)
-            BAIL(NC_EATTMETA);
 
-        /* For text attributes the size is specified in the datatype
-           and it is enough to compare types using H5Tequal(). */
+        /* Is new attribute the same datatype and size as previous?
+         * For text attributes the size is embedded in the datatype, not the dataspace.
+         * H5Sextent_equal will also do the right thing with NULL dataspace cases. */
         if (!H5Tequal(file_typeid, existing_att_typeid) ||
-            (att->nc_typeid != NC_CHAR && npoints != att->len))
+            (!H5Sextent_equal(spaceid, existing_spaceid)))
         {
-            /* The attribute exists but we cannot re-use it. */
+            /* The attribute exists but we cannot reuse it. */
 
             /* Delete the attribute. */
             if (H5Adelete(locid, att->hdr.name) < 0)
@@ -560,9 +552,9 @@ put_att_grpa(NC_GRP_INFO_T *grp, int varid, NC_ATT_INFO_T *att)
         }
         else
         {
-            /* The attribute exists and we can re-use it. */
+            /* The attribute exists and we can reuse it. */
 
-            /* Write the values, re-using the existing attribute. */
+            /* Write the values, reusing the existing attribute. */
             if (H5Awrite(existing_attid, file_typeid, data) < 0)
                 BAIL(NC_EATTMETA);
         }
@@ -613,9 +605,8 @@ write_attlist(NCindex *attlist, int varid, NC_GRP_INFO_T *grp)
 {
     NC_ATT_INFO_T *att;
     int retval;
-    int i;
 
-    for(i = 0; i < ncindexsize(attlist); i++)
+    for(size_t i = 0; i < ncindexsize(attlist); i++)
     {
         att = (NC_ATT_INFO_T *)ncindexith(attlist, i);
         assert(att);
@@ -679,6 +670,67 @@ exit:
     return retval;
 }
 
+/**
+ * @internal When nc_def_var_quantize() is used, a new attribute is
+ * added to the var, containing the quantize information.
+ *
+ * @param var Pointer to var info struct.
+ *
+ * @returns NC_NOERR No error.
+ * @returns NC_EHDFERR HDF5 returned an error.
+ * @author Ed Hartnett
+ */
+static int
+write_quantize_att(NC_VAR_INFO_T *var)
+{
+    NC_HDF5_VAR_INFO_T *hdf5_var;
+    hsize_t len = 1;
+    hid_t c_spaceid = -1, c_attid = -1;
+    char att_name[NC_MAX_NAME + 1];
+    int retval = NC_NOERR;
+
+    assert(var && var->format_var_info);
+
+    /* Get HDF5-specific var info. */
+    hdf5_var = (NC_HDF5_VAR_INFO_T *)var->format_var_info;
+
+    /* Different quantize algorithms get different attribute names. */
+    switch (var->quantize_mode)
+    {
+	case NC_QUANTIZE_BITGROOM:
+	    snprintf(att_name, sizeof(att_name), "%s", NC_QUANTIZE_BITGROOM_ATT_NAME);
+	    break;
+	case NC_QUANTIZE_GRANULARBR:
+	    snprintf(att_name, sizeof(att_name), "%s", NC_QUANTIZE_GRANULARBR_ATT_NAME);
+	    break;
+	case NC_QUANTIZE_BITROUND:
+	    snprintf(att_name, sizeof(att_name), "%s", NC_QUANTIZE_BITROUND_ATT_NAME);
+	   break;
+        default:
+	    return NC_EINVAL;
+    }
+
+    /* Set up space for attribute. */
+    if ((c_spaceid = H5Screate_simple(1, &len, &len)) < 0)
+        BAIL(NC_EHDFERR);
+
+    /* Create the attribute. */
+    if ((c_attid = H5Acreate1(hdf5_var->hdf_datasetid, att_name,
+                             H5T_NATIVE_INT, c_spaceid, H5P_DEFAULT)) < 0)
+        BAIL(NC_EHDFERR);
+
+    /* Write our attribute. */
+    if (H5Awrite(c_attid, H5T_NATIVE_INT, &var->nsd) < 0)
+        BAIL(NC_EHDFERR);
+
+exit:
+    if (c_spaceid >= 0 && H5Sclose(c_spaceid) < 0)
+        BAIL2(NC_EHDFERR);
+    if (c_attid >= 0 && H5Aclose(c_attid) < 0)
+        BAIL2(NC_EHDFERR);
+    return retval;
+}
+
 /**
  * @internal Write a special attribute for the netCDF-4 dimension ID.
  *
@@ -823,18 +875,6 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
         }
     }
 
-    /* If the user wants to fletcher error correction, set that up now. */
-    /* Since it is a checksum of sorts, flatcher is always applied first */
-    if (var->fletcher32)
-        if (H5Pset_fletcher32(plistid) < 0)
-            BAIL(NC_EHDFERR);
-
-    /* If the user wants to shuffle the data, set that up now. */
-    if (var->shuffle) {
-        if (H5Pset_shuffle(plistid) < 0)
-            BAIL(NC_EHDFERR);
-    }
-
     /* If the user wants to compress the data, using either zlib
      * (a.k.a deflate) or szip, or another filter, set that up now.
      * Szip and zip can be turned on
@@ -842,39 +882,36 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
      * nc_def_var_filter(). If the user
      * has specified a filter, it will be applied here. */
     if(var->filters != NULL) {
-	int j;
+	size_t j;
 	NClist* filters = (NClist*)var->filters;
 	for(j=0;j<nclistlength(filters);j++) {
 	    struct NC_HDF5_Filter* fi = (struct NC_HDF5_Filter*)nclistget(filters,j);
-	    {
-                if(fi->filterid == H5Z_FILTER_DEFLATE) {/* Handle zip case here */
-                    unsigned level;
-                    if(fi->nparams != 1)
-                        BAIL(NC_EFILTER);
-                    level = (int)fi->params[0];
-                    if(H5Pset_deflate(plistid, level) < 0)
-                        BAIL(NC_EFILTER);
-  	        } else if(fi->filterid == H5Z_FILTER_SZIP) {/* Handle szip case here */
-                    int options_mask;
-                    int bits_per_pixel;
-                    if(fi->nparams != 2)
-                        BAIL(NC_EFILTER);
-                    options_mask = (int)fi->params[0];
-                    bits_per_pixel = (int)fi->params[1];
-                    if(H5Pset_szip(plistid, options_mask, bits_per_pixel) < 0)
-                        BAIL(NC_EFILTER);
-                } else {
-                    herr_t code = H5Pset_filter(plistid, fi->filterid,
-#if 0
-		    				H5Z_FLAG_MANDATORY,
-#else
-		    				H5Z_FLAG_OPTIONAL,
-#endif
-						fi->nparams, fi->params);
-                    if(code < 0)
-                        BAIL(NC_EFILTER);
-		}
-            }
+	    if(fi->filterid == H5Z_FILTER_FLETCHER32) {
+	        if(H5Pset_fletcher32(plistid) < 0)
+                    BAIL(NC_EHDFERR);
+	    } else if(fi->filterid == H5Z_FILTER_SHUFFLE) {
+	        if(H5Pset_shuffle(plistid) < 0)
+                    BAIL(NC_EHDFERR);
+            } else if(fi->filterid == H5Z_FILTER_DEFLATE) {/* Handle zip case here */
+                if(fi->nparams != 1)
+                    BAIL(NC_EFILTER);
+                unsigned int level = fi->params[0];
+                if(H5Pset_deflate(plistid, level) < 0)
+                    BAIL(NC_EFILTER);
+            } else if(fi->filterid == H5Z_FILTER_SZIP) {/* Handle szip case here */
+                if(fi->nparams != 2)
+                    BAIL(NC_EFILTER);
+                unsigned int options_mask = fi->params[0];
+                unsigned int bits_per_pixel = fi->params[1];
+                if(H5Pset_szip(plistid, options_mask, bits_per_pixel) < 0)
+                    BAIL(NC_EFILTER);
+            } else {
+                herr_t code = H5Pset_filter(plistid, fi->filterid,
+                                            H5Z_FLAG_OPTIONAL, /* always make optional so filters on vlens are ignored */
+                                           fi->nparams, fi->params);
+		if(code < 0)
+                    BAIL(NC_EFILTER);
+	    }
         }
     }
 
@@ -897,7 +934,7 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
         /* If there are no unlimited dims, and no filters, and the user
          * has not specified chunksizes, use contiguous variable for
          * better performance. */
-        if (!var->shuffle && !var->fletcher32 && nclistlength((NClist*)var->filters) == 0 &&
+        if (nclistlength((NClist*)var->filters) == 0 &&
             (var->chunksizes == NULL || !var->chunksizes[0]) && !unlimdim)
 	    var->storage = NC_CONTIGUOUS;
 
@@ -925,8 +962,8 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
                     if (dim->unlimited)
                         chunksize[d] = 1;
                     else
-                        chunksize[d] = pow((double)DEFAULT_CHUNK_SIZE/type_size,
-                                           1/(double)(var->ndims - unlimdim));
+                        chunksize[d] = (hsize_t)pow(DEFAULT_CHUNK_SIZE/(double)type_size,
+                                                    1/(double)((int)var->ndims - unlimdim));
 
                     /* If the chunksize is greater than the dim
                      * length, make it the dim length. */
@@ -940,7 +977,7 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
         }
 
         /* Create the dataspace. */
-        if ((spaceid = H5Screate_simple(var->ndims, dimsize, maxdimsize)) < 0)
+        if ((spaceid = H5Screate_simple((int)var->ndims, dimsize, maxdimsize)) < 0)
             BAIL(NC_EHDFERR);
     }
     else
@@ -964,19 +1001,21 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
     }
     else if (var->ndims)
     {
-        if (H5Pset_chunk(plistid, var->ndims, chunksize) < 0)
+        if (H5Pset_chunk(plistid, (int)var->ndims, chunksize) < 0)
             BAIL(NC_EHDFERR);
     }
 
     /* Turn on creation order tracking. */
-    if (H5Pset_attr_creation_order(plistid, H5P_CRT_ORDER_TRACKED|
-                                   H5P_CRT_ORDER_INDEXED) < 0)
+    if (!grp->nc4_info->no_attr_create_order) {
+      if (H5Pset_attr_creation_order(plistid, H5P_CRT_ORDER_TRACKED|
+				     H5P_CRT_ORDER_INDEXED) < 0)
         BAIL(NC_EHDFERR);
+    }
 
     /* Set per-var chunk cache, for chunked datasets. */
-    if (var->storage == NC_CHUNKED && var->chunk_cache_size)
-        if (H5Pset_chunk_cache(access_plistid, var->chunk_cache_nelems,
-                               var->chunk_cache_size, var->chunk_cache_preemption) < 0)
+    if (var->storage == NC_CHUNKED && var->chunkcache.size)
+        if (H5Pset_chunk_cache(access_plistid, var->chunkcache.nelems,
+                               var->chunkcache.size, var->chunkcache.preemption) < 0)
             BAIL(NC_EHDFERR);
 
     /* At long last, create the dataset. */
@@ -990,11 +1029,11 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
     var->is_new_var = NC_FALSE;
 
     /* Always write the hidden coordinates attribute, which lists the
-     * dimids of this var. When present, this speeds opens. When no
+     * dimids of this var. When present, this speeds opens. When not
      * present, dimscale matching is used. */
     if (var->ndims)
         if ((retval = write_coord_dimids(var)))
-           BAIL(retval);
+            BAIL(retval);
 
     /* If this is a dimscale, mark it as such in the HDF5 file. Also
      * find the dimension info and store the dataset id of the dimscale
@@ -1016,9 +1055,19 @@ var_create_dataset(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var, nc_bool_t write_dimid
                 BAIL(retval);
     }
 
+    /* If quantization is in use, write an attribute indicating it, a
+     * single integer which is the number of significant digits 
+     * (NSD, for BitGroom and Granular BitRound) or number of significant bits
+     * (NSB, for BitRound). */
+    if (var->quantize_mode)
+	if ((retval = write_quantize_att(var)))
+	    BAIL(retval);
+
     /* Write attributes for this var. */
     if ((retval = write_attlist(var->att, var->hdr.id, grp)))
         BAIL(retval);
+
+    /* The file is now up-to-date with all settings for this var. */
     var->attr_dirty = NC_FALSE;
 
 exit:
@@ -1084,12 +1133,12 @@ nc4_adjust_var_cache(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
     /* If the chunk cache is too small, and the user has not changed
      * the default value of the chunk cache size, then increase the
      * size of the cache. */
-    if (var->chunk_cache_size == CHUNK_CACHE_SIZE)
-        if (chunk_size_bytes > var->chunk_cache_size)
+    if (var->chunkcache.size == CHUNK_CACHE_SIZE)
+        if (chunk_size_bytes > var->chunkcache.size)
         {
-            var->chunk_cache_size = chunk_size_bytes * DEFAULT_CHUNKS_IN_CACHE;
-            if (var->chunk_cache_size > MAX_DEFAULT_CACHE_SIZE)
-                var->chunk_cache_size = MAX_DEFAULT_CACHE_SIZE;
+            var->chunkcache.size = chunk_size_bytes * DEFAULT_CHUNKS_IN_CACHE;
+            if (var->chunkcache.size > DEFAULT_CHUNK_CACHE_SIZE)
+                var->chunkcache.size = DEFAULT_CHUNK_CACHE_SIZE;
             if ((retval = nc4_reopen_dataset(grp, var)))
                 return retval;
         }
@@ -1135,7 +1184,7 @@ commit_type(NC_GRP_INFO_T *grp, NC_TYPE_INFO_T *type)
     {
         NC_FIELD_INFO_T *field;
         hid_t hdf_base_typeid, hdf_typeid;
-        int i;
+        size_t i;
 
         if ((hdf5_type->hdf_typeid = H5Tcreate(H5T_COMPOUND, type->size)) < 0)
             return NC_EHDFERR;
@@ -1157,7 +1206,7 @@ commit_type(NC_GRP_INFO_T *grp, NC_TYPE_INFO_T *type)
                 hsize_t dims[NC_MAX_VAR_DIMS];
 
                 for (d = 0; d < field->ndims; d++)
-                    dims[d] = field->dim_size[d];
+                    dims[d] = (hsize_t)field->dim_size[d];
                 if ((hdf_typeid = H5Tarray_create1(hdf_base_typeid, field->ndims,
 						   dims, NULL)) < 0)
                 {
@@ -1199,7 +1248,7 @@ commit_type(NC_GRP_INFO_T *grp, NC_TYPE_INFO_T *type)
     else if (type->nc_type_class == NC_ENUM)
     {
         NC_ENUM_MEMBER_INFO_T *enum_m;
-        int i;
+        size_t i;
 
         if (nclistlength(type->u.e.enum_member) == 0)
             return NC_EINVAL;
@@ -1327,8 +1376,10 @@ create_group(NC_GRP_INFO_T *grp)
         BAIL(NC_EHDFERR);
 
     /* Tell HDF5 to keep track of attributes in creation order. */
-    if (H5Pset_attr_creation_order(gcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0)
+    if (!grp->nc4_info->no_attr_create_order) {
+      if (H5Pset_attr_creation_order(gcpl_id, H5P_CRT_ORDER_TRACKED|H5P_CRT_ORDER_INDEXED) < 0)
         BAIL(NC_EHDFERR);
+    }
 
     /* Create the group. */
     if ((hdf5_grp->hdf_grpid = H5Gcreate2(parent_hdf5_grp->hdf_grpid, grp->hdr.name,
@@ -1353,6 +1404,7 @@ exit:
  * @param grp Pointer to group info struct.
  *
  * @return ::NC_NOERR No error.
+ * @returns NC_EDIMSCALE HDF5 returned an error when trying to attach a dimension scale.
  * @author Ed Hartnett
  */
 static int
@@ -1360,10 +1412,9 @@ attach_dimscales(NC_GRP_INFO_T *grp)
 {
     NC_VAR_INFO_T *var;
     NC_HDF5_VAR_INFO_T *hdf5_var;
-    int d, v;
 
     /* Attach dimension scales. */
-    for (v = 0; v < ncindexsize(grp->vars); v++)
+    for (size_t v = 0; v < ncindexsize(grp->vars); v++)
     {
         /* Get pointer to var and HDF5-specific var info. */
         var = (NC_VAR_INFO_T *)ncindexith(grp->vars, v);
@@ -1376,7 +1427,7 @@ attach_dimscales(NC_GRP_INFO_T *grp)
             continue;
 
         /* Find the scale for each dimension, if any, and attach it. */
-        for (d = 0; d < var->ndims; d++)
+        for (unsigned int d = 0; d < var->ndims; d++)
         {
             /* Is there a dimscale for this dimension? */
             if (hdf5_var->dimscale_attached)
@@ -1395,11 +1446,13 @@ attach_dimscales(NC_GRP_INFO_T *grp)
                         dsid = ((NC_HDF5_VAR_INFO_T *)(var->dim[d]->coord_var->format_var_info))->hdf_datasetid;
                     else
                         dsid = ((NC_HDF5_DIM_INFO_T *)var->dim[d]->format_dim_info)->hdf_dimscaleid;
-                    assert(dsid > 0);
+
+                    if (dsid <= 0)
+                        return NC_EDIMSCALE;
 
                     /* Attach the scale. */
                     if (H5DSattach_scale(hdf5_var->hdf_datasetid, dsid, d) < 0)
-                        return NC_EHDFERR;
+                        return NC_EDIMSCALE;
                     hdf5_var->dimscale_attached[d] = NC_TRUE;
                 }
             }
@@ -1600,8 +1653,6 @@ write_var(NC_VAR_INFO_T *var, NC_GRP_INFO_T *grp, nc_bool_t write_dimid)
      * and delete dimscale attributes from the var. */
     if (var->was_coord_var && hdf5_var->dimscale_attached)
     {
-        int d;
-
         /* If the variable already exists in the file, Remove any dimension scale
          * attributes from it, if they exist. */
         if (var->created)
@@ -1609,7 +1660,7 @@ write_var(NC_VAR_INFO_T *var, NC_GRP_INFO_T *grp, nc_bool_t write_dimid)
                 return retval;
 
         /* If this is a regular var, detach all its dim scales. */
-        for (d = 0; d < var->ndims; d++)
+        for (unsigned int d = 0; d < var->ndims; d++)
         {
             if (hdf5_var->dimscale_attached[d])
             {
@@ -1699,8 +1750,6 @@ write_var(NC_VAR_INFO_T *var, NC_GRP_INFO_T *grp, nc_bool_t write_dimid)
  * coordinate variable. This is a special 1-D dataset.
  *
  * @param dim Pointer to dim info struct.
- * @param grp Pointer to group info struct.
- * @param write_dimid
  *
  * @returns ::NC_NOERR No error.
  * @returns ::NC_EPERM Read-only file.
@@ -1752,10 +1801,11 @@ nc4_create_dim_wo_var(NC_DIM_INFO_T *dim)
         BAIL(NC_EHDFERR);
 
     /* Turn on creation-order tracking. */
-    if (H5Pset_attr_creation_order(create_propid, H5P_CRT_ORDER_TRACKED|
-                                   H5P_CRT_ORDER_INDEXED) < 0)
+    if (!dim->container->nc4_info->no_attr_create_order) {
+      if (H5Pset_attr_creation_order(create_propid, H5P_CRT_ORDER_TRACKED|
+				     H5P_CRT_ORDER_INDEXED) < 0)
         BAIL(NC_EHDFERR);
-
+    }
     /* Create the dataset that will be the dimension scale. */
     LOG((4, "%s: about to H5Dcreate1 a dimscale dataset %s", __func__,
          dim->hdr.name));
@@ -1768,7 +1818,7 @@ nc4_create_dim_wo_var(NC_DIM_INFO_T *dim)
     /* Indicate that this is a scale. Also indicate that not
      * be shown to the user as a variable. It is hidden. It is
      * a DIM WITHOUT A VARIABLE! */
-    sprintf(dimscale_wo_var, "%s%10d", DIM_WITHOUT_VARIABLE, (int)dim->len);
+    snprintf(dimscale_wo_var, sizeof(dimscale_wo_var), "%s%10d", DIM_WITHOUT_VARIABLE, (int)dim->len);
     if (H5DSset_scale(hdf5_dim->hdf_dimscaleid, dimscale_wo_var) < 0)
         BAIL(NC_EHDFERR);
 
@@ -1865,7 +1915,7 @@ exit:
 
 /**
  * @internal Recursively write all the metadata in a group. Groups and
- * types have all already been written. Propagate bad cooordinate
+ * types have all already been written. Propagate bad coordinate
  * order to subgroups, if detected.
  *
  * @param grp Pointer to group info struct.
@@ -1882,10 +1932,9 @@ nc4_rec_write_metadata(NC_GRP_INFO_T *grp, nc_bool_t bad_coord_order)
     NC_VAR_INFO_T *var = NULL;
     NC_GRP_INFO_T *child_grp = NULL;
     int coord_varid = -1;
-    int var_index = 0;
-    int dim_index = 0;
+    size_t var_index = 0;
+    size_t dim_index = 0;
     int retval;
-    int i;
 
     assert(grp && grp->hdr.name &&
            ((NC_HDF5_GRP_INFO_T *)(grp->format_grp_info))->hdf_grpid);
@@ -1937,12 +1986,14 @@ nc4_rec_write_metadata(NC_GRP_INFO_T *grp, nc_bool_t bad_coord_order)
         }
     } /* end while */
 
-    /* Attach dimscales to vars in this group. */
-    if ((retval = attach_dimscales(grp)))
-        return retval;
+    /* Attach dimscales to vars in this group. Unless directed not to. */
+    if (!grp->nc4_info->no_dimscale_attach) {
+        if ((retval = attach_dimscales(grp)))
+            return retval;
+    }
 
     /* If there are any child groups, write their metadata. */
-    for (i = 0; i < ncindexsize(grp->children); i++)
+    for (size_t i = 0; i < ncindexsize(grp->children); i++)
     {
         child_grp = (NC_GRP_INFO_T *)ncindexith(grp->children, i);
         assert(child_grp);
@@ -1968,7 +2019,7 @@ nc4_rec_write_groups_types(NC_GRP_INFO_T *grp)
     NC_HDF5_GRP_INFO_T *hdf5_grp;
     NC_TYPE_INFO_T *type;
     int retval;
-    int i;
+    size_t i;
 
     assert(grp && grp->hdr.name && grp->format_grp_info);
     LOG((3, "%s: grp->hdr.name %s", __func__, grp->hdr.name));
@@ -2024,7 +2075,7 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
     NC_VAR_INFO_T *var;
     NC_DIM_INFO_T *dim;
     int retval = NC_NOERR;
-    int i;
+    size_t i;
 
     assert(grp && grp->hdr.name);
     LOG((4, "%s: grp->hdr.name %s", __func__, grp->hdr.name));
@@ -2043,7 +2094,6 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
     for (i = 0; i < ncindexsize(grp->vars); i++)
     {
         NC_HDF5_VAR_INFO_T *hdf5_var;
-        int ndims;
         int d;
 
         /* Get pointer to var and to the HDF5-specific var info. */
@@ -2054,7 +2104,7 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
         /* Check all vars and see if dim[i] != NULL if dimids[i] valid. */
         /* This loop is very odd. Under normal circumstances, var->dimid[d] is zero
            (from the initial calloc) which is a legitimate dimid. The code does not
-           distinquish this case from the dimscale case where the id might actually
+           distinguish this case from the dimscale case where the id might actually
            be defined.
            The original nc4_find_dim searched up the group tree looking for the given
            dimid in one of the dim lists associated with each ancestor group.
@@ -2066,8 +2116,8 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
            The solution I choose is to modify nc4_var_list_add to initialize dimids to
            illegal values (-1). This is another example of the problems with dimscales.
         */
-        ndims = var->ndims;
-        for (d = 0; d < ndims; d++)
+        const size_t ndims = var->ndims;
+        for (size_t d = 0; d < ndims; d++)
         {
             if (var->dim[d] == NULL) {
                 nc4_find_dim(grp, var->dimids[d], &var->dim[d], NULL);
@@ -2078,13 +2128,10 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
         /* Skip dimension scale variables */
         if (!hdf5_var->dimscale)
         {
-            int d;
-            int j;
-
             /* Are there dimscales for this variable? */
             if (hdf5_var->dimscale_hdf5_objids)
             {
-                for (d = 0; d < var->ndims; d++)
+                for (size_t d = 0; d < var->ndims; d++)
                 {
                     nc_bool_t finished = NC_FALSE;
                     LOG((5, "%s: var %s has dimscale info...", __func__, var->hdr.name));
@@ -2093,7 +2140,7 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
                     for (g = grp; g && !finished; g = g->parent)
                     {
                         /* Check all dims in this group. */
-                        for (j = 0; j < ncindexsize(g->dim); j++)
+                        for (size_t j = 0; j < ncindexsize(g->dim); j++)
                         {
                             /* Get the HDF5 specific dim info. */
                             NC_HDF5_DIM_INFO_T *hdf5_dim;
@@ -2182,22 +2229,21 @@ nc4_rec_match_dimscales(NC_GRP_INFO_T *grp)
                  * size. */
                 for (d = 0; d < var->ndims; d++)
                 {
-                    int k;
-                    int match;
+                    nc_bool_t match = NC_FALSE;
                     /* Is there already a phony dimension of the correct size? */
-                    for(match=-1,k=0;k<ncindexsize(grp->dim);k++) {
+                    for(size_t k=0;k<ncindexsize(grp->dim);k++) {
                         if((dim = (NC_DIM_INFO_T*)ncindexith(grp->dim,k)) == NULL) continue;
                         if ((dim->len == h5dimlen[d]) &&
                             ((h5dimlenmax[d] == H5S_UNLIMITED && dim->unlimited) ||
                              (h5dimlenmax[d] != H5S_UNLIMITED && !dim->unlimited)))
-                        {match = k; break;}
+                        {match = NC_TRUE; break;}
                     }
 
                     /* Didn't find a phony dim? Then create one. */
-                    if (match < 0)
+                    if (match == NC_FALSE)
                     {
                         char phony_dim_name[NC_MAX_NAME + 1];
-                        sprintf(phony_dim_name, "phony_dim_%d", grp->nc4_info->next_dimid);
+                        snprintf(phony_dim_name, sizeof(phony_dim_name), "phony_dim_%d", grp->nc4_info->next_dimid);
                         LOG((3, "%s: creating phony dim for var %s", __func__, var->hdr.name));
                         if ((retval = nc4_dim_list_add(grp, phony_dim_name, h5dimlen[d], -1, &dim)))
                         {
@@ -2289,7 +2335,6 @@ reportopenobjectsT(int uselog, hid_t fid, int ntypes, unsigned int* otypes)
 {
     int t,i;
     ssize_t ocount;
-    size_t maxobjs = -1;
     hid_t* idlist = NULL;
 
     /* Always report somehow */
@@ -2299,7 +2344,7 @@ reportopenobjectsT(int uselog, hid_t fid, int ntypes, unsigned int* otypes)
     else
 #endif
         fprintf(stdout,"\nReport: open objects on %lld\n",(long long)fid);
-    maxobjs = H5Fget_obj_count(fid,H5F_OBJ_ALL);
+    size_t maxobjs = (size_t)H5Fget_obj_count(fid,H5F_OBJ_ALL);
     if(idlist != NULL) free(idlist);
     idlist = (hid_t*)malloc(sizeof(hid_t)*maxobjs);
     for(t=0;t<ntypes;t++) {
@@ -2422,7 +2467,7 @@ done:
     return stat;
 }
 
-static int NC4_strict_att_exists(NC_FILE_INFO_T*);
+static int NC4_root_att_exists(NC_FILE_INFO_T*, const char* aname);
 static int NC4_walk(hid_t, int*);
 
 /**
@@ -2459,9 +2504,13 @@ NC4_isnetcdf4(struct NC_FILE_INFO* h5)
     int count;
 
     /* Look for NC3_STRICT_ATT_NAME */
-    exists = NC4_strict_att_exists(h5);
+    exists = NC4_root_att_exists(h5,NC3_STRICT_ATT_NAME);
     if(exists)
-        goto done;
+        {isnc4 = 1; goto done;}
+    /* Look for _NCProperties */
+    exists = NC4_root_att_exists(h5,NCPROPS);
+    if(exists)
+        {isnc4 = 1; goto done;}
     /* attribute did not exist */
     /* => last resort: walk the HDF5 file looking for markers */
     count = 0;
@@ -2477,15 +2526,16 @@ done:
 }
 
 /**
- * @internal See if the NC3 strict attribute exists.
+ * @internal See if the named root attribute exists.
  *
  * @param h5 Pointer to HDF5 file info struct.
+ * @param aname attribute name for which to look.
  *
- * @returns 1 if error || exists; 0 otherwise
+ * @returns -1 if error || 1 if exists || 0 otherwise
  * @author Dennis Heimbigner.
  */
 static int
-NC4_strict_att_exists(NC_FILE_INFO_T *h5)
+NC4_root_att_exists(NC_FILE_INFO_T *h5, const char* aname)
 {
     hid_t grpid = -1;
     htri_t attr_exists;
@@ -2494,8 +2544,8 @@ NC4_strict_att_exists(NC_FILE_INFO_T *h5)
     grpid = ((NC_HDF5_GRP_INFO_T *)(h5->root_grp->format_grp_info))->hdf_grpid;
 
     /* See if the NC3_STRICT_ATT_NAME attribute exists */
-    if ((attr_exists = H5Aexists(grpid, NC3_STRICT_ATT_NAME)) < 0)
-        return 1;
+    if ((attr_exists = H5Aexists(grpid, aname))<0)
+        return -1;
     return (attr_exists?1:0);
 }
 
@@ -2512,7 +2562,7 @@ static int
 NC4_walk(hid_t gid, int* countp)
 {
     int ncstat = NC_NOERR;
-    int i,j,na;
+    int j,na;
     ssize_t len;
     hsize_t nobj;
     herr_t err;
@@ -2524,12 +2574,12 @@ NC4_walk(hid_t gid, int* countp)
     err = H5Gget_num_objs(gid, &nobj);
     if(err < 0) return err;
 
-    for(i = 0; i < nobj; i++) {
+    for(hsize_t i = 0; i < nobj; i++) {
         /* Get name & kind of object in the group */
-        len = H5Gget_objname_by_idx(gid,(hsize_t)i,name,(size_t)NC_HDF5_MAX_NAME);
-        if(len < 0) return len;
+        len = H5Gget_objname_by_idx(gid,i,name,(size_t)NC_HDF5_MAX_NAME);
+        if(len < 0) return (int)len;
 
-        otype =  H5Gget_objtype_by_idx(gid,(size_t)i);
+        otype =  H5Gget_objtype_by_idx(gid, i);
         switch(otype) {
         case H5G_GROUP:
 	    grpid = H5Gopen1(gid,name);
@@ -2547,8 +2597,7 @@ NC4_walk(hid_t gid, int* countp)
                 if(aid >= 0) {
                     const NC_reservedatt* ra;
                     ssize_t len = H5Aget_name(aid, NC_HDF5_MAX_NAME, name);
-                    if(len < 0) return len;
-                    /* Is this a netcdf-4 marker attribute */
+                    if(len < 0) return (int)len;
                     /* Is this a netcdf-4 marker attribute */
                     ra = NC_findreserved(name);
                     if(ra != NULL)
