@@ -51,7 +51,7 @@ ncz_getattlist(NC_GRP_INFO_T *grp, int varid, NC_VAR_INFO_T **varp, NCindex **at
     {
         NC_VAR_INFO_T *var;
 
-        if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+        if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
             return NC_ENOTVAR;
         assert(var->hdr.id == varid);
 
@@ -102,7 +102,7 @@ ncz_get_att_special(NC_FILE_INFO_T* h5, NC_VAR_INFO_T* var, const char* name,
 
     /* Handle the per-var case(s) first */
     if(var != NULL) {
-#ifdef ENABLE_NCZARR_FILTERS
+#ifdef NETCDF_ENABLE_NCZARR_FILTERS
         if(strcmp(name,NC_ATT_CODECS)==0) {	
             NClist* filters = (NClist*)var->filters;
 
@@ -120,7 +120,7 @@ ncz_get_att_special(NC_FILE_INFO_T* h5, NC_VAR_INFO_T* var, const char* name,
 
     /* The global reserved attributes */
     if(strcmp(name,NCPROPS)==0) {
-        int len;
+        size_t len;
         if(h5->provenance.ncproperties == NULL)
             {stat = NC_ENOTATT; goto done;}
         if(mem_type == NC_NAT) mem_type = NC_CHAR;
@@ -138,7 +138,7 @@ ncz_get_att_special(NC_FILE_INFO_T* h5, NC_VAR_INFO_T* var, const char* name,
         if(strcmp(name,SUPERBLOCKATT)==0)
             iv = (unsigned long long)h5->provenance.superblockversion;
         else /* strcmp(name,ISNETCDF4ATT)==0 */
-            iv = NCZ_isnetcdf4(h5);
+            iv = (unsigned long long)NCZ_isnetcdf4(h5);
         if(mem_type == NC_NAT) mem_type = NC_INT;
         if(data)
             switch (mem_type) {
@@ -279,8 +279,8 @@ NCZ_del_att(int ncid, int varid, const char *name)
     NC_FILE_INFO_T *h5;
     NC_ATT_INFO_T *att;
     NCindex* attlist = NULL;
-    int i;
-    size_t deletedid;
+    size_t i;
+    int deletedid;
     int retval;
 
     /* Name must be provided. */
@@ -326,7 +326,7 @@ NCZ_del_att(int ncid, int varid, const char *name)
 
     /* Reclaim the content of the attribute */
     if(att->data) {
-	if((retval = nc_reclaim_data_all(ncid,att->nc_typeid,att->data,att->len))) return retval;
+	if((retval = NC_reclaim_data_all(h5->controller,att->nc_typeid,att->data,att->len))) return retval;
     }
     att->data = NULL;
     att->len = 0;
@@ -516,7 +516,7 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
         /* For an existing att, if we're not in define mode, the len
            must not be greater than the existing len for classic model. */
         if (!(h5->flags & NC_INDEF) &&
-            len * nc4typelen(file_type) > (size_t)att->len * nc4typelen(att->nc_typeid))
+            len * (size_t)nc4typelen(file_type) > (size_t)att->len * (size_t)nc4typelen(att->nc_typeid))
         {
             if (h5->cmode & NC_CLASSIC_MODEL)
                 return NC_ENOTINDEFINE;
@@ -581,7 +581,7 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
      * copy the value to the fill_value pointer of the NC_VAR_INFO_T
      * struct for this var. (But ignore a global _FillValue
      * attribute). Also kill the cache fillchunk as no longer valid */
-    if (!strcmp(att->hdr.name, _FillValue) && varid != NC_GLOBAL)
+    if (!strcmp(att->hdr.name, NC_FillValue) && varid != NC_GLOBAL)
     {
         /* Fill value must have exactly one value */
         if (len != 1)
@@ -591,7 +591,7 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
         if (var->written_to)
             return NC_ELATEFILL;
 
-        /* Get the length of the veriable data type. */
+        /* Get the length of the variable data type. */
         if ((retval = nc4_get_typelen_mem(grp->nc4_info, var->type_info->hdr.id,
                                           &type_size)))
             return retval;
@@ -628,7 +628,7 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
 	    } else { /* no conversion */
 		/* Still need a copy of the input data */
 		copy = NULL;
-	        if((retval = nc_copy_data_all(h5->controller->ext_ncid, mem_type, data, 1, &copy)))
+	        if((retval = NC_copy_data_all(h5->controller, mem_type, data, 1, &copy)))
 		    BAIL(retval);
 	    }
 	    var->fill_value = copy;
@@ -665,7 +665,7 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
 					       NC_NOQUANTIZE, 0)))
                     BAIL(retval);
 	    } else if(mem_type == file_type) { /* General case: no conversion */
-	        if((retval = nc_copy_data(h5->controller->ext_ncid,file_type,data,len,copy)))
+	        if((retval = NC_copy_data(h5->controller,file_type,data,len,copy)))
 		    BAIL(retval);
 	    } else
 	    	BAIL(NC_EURL);
@@ -700,30 +700,30 @@ ncz_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
     /* Reclaim saved data */
     if(attsave.data != NULL) {
         assert(attsave.len > 0);
-        (void)nc_reclaim_data_all(h5->controller->ext_ncid,attsave.type,attsave.data,attsave.len);
+        (void)NC_reclaim_data_all(h5->controller,attsave.type,attsave.data,attsave.len);
 	attsave.len = 0; attsave.data = NULL;
     }
     if(fillsave.data != NULL) {
         assert(fillsave.len > 0);
-        (void)nc_reclaim_data_all(h5->controller->ext_ncid,fillsave.type,fillsave.data,fillsave.len);
+        (void)NC_reclaim_data_all(h5->controller,fillsave.type,fillsave.data,fillsave.len);
 	fillsave.len = 0; fillsave.data = NULL;
     }
 
 exit:
     if(copy)
-        (void)nc_reclaim_data_all(h5->controller->ext_ncid,file_type,copy,len);
+        (void)NC_reclaim_data_all(h5->controller,file_type,copy,len);
     if(retval) {
 	/* Rollback */
         if(attsave.data != NULL) {
             assert(attsave.len > 0);
 	    if(att->data)
-                (void)nc_reclaim_data_all(h5->controller->ext_ncid,attsave.type,att->data,att->len);
+                (void)NC_reclaim_data_all(h5->controller,attsave.type,att->data,att->len);
 	    att->len = attsave.len; att->data = attsave.data;
         }
         if(fillsave.data != NULL) {
             assert(fillsave.len > 0);
 	    if(att->data)
-	    (void)nc_reclaim_data_all(h5->controller->ext_ncid,fillsave.type,var->fill_value,1);
+	    (void)NC_reclaim_data_all(h5->controller,fillsave.type,var->fill_value,1);
 	    var->fill_value = fillsave.data;
         }
     }    
@@ -862,7 +862,7 @@ NCZ_inq_attid(int ncid, int varid, const char *name, int *attnump)
  * @param ncid File and group ID.
  * @param varid Variable ID.
  * @param attnum The index number of the attribute.
- * @param name Pointer that gets name of attrribute.
+ * @param name Pointer that gets name of attribute.
  *
  * @return ::NC_NOERR No error.
  * @return ::NC_EBADID Bad ncid.
@@ -980,7 +980,7 @@ int
 ncz_create_fillvalue(NC_VAR_INFO_T* var)
 {
     int stat = NC_NOERR;
-    int i;
+    size_t i;
     NC_ATT_INFO_T* fv = NULL;
 
     /* Have the var's attributes been read? */
@@ -991,12 +991,12 @@ ncz_create_fillvalue(NC_VAR_INFO_T* var)
         /* Make sure _FillValue does not exist */
 	for(i=0;i<ncindexsize(var->att);i++) {
 	    fv = (NC_ATT_INFO_T*)ncindexith(var->att,i);
-	    if(strcmp(fv->hdr.name,_FillValue)==0) break;
+	    if(strcmp(fv->hdr.name,NC_FillValue)==0) break;
 	    fv = NULL;
         }
 	if(fv == NULL) {
 	    /* Create it */
-	    if((stat = ncz_makeattr((NC_OBJ*)var,var->att,_FillValue,var->type_info->hdr.id,1,var->fill_value,&fv)))
+	    if((stat = ncz_makeattr((NC_OBJ*)var,var->att,NC_FillValue,var->type_info->hdr.id,1,var->fill_value,&fv)))
 	    goto done;
 	}
     }
@@ -1014,20 +1014,14 @@ ncz_makeattr(NC_OBJ* container, NCindex* attlist, const char* name, nc_type type
     NCZ_ATT_INFO_T* zatt = NULL;
     void* clone = NULL;
     size_t typesize, clonesize;
-    int ncid;
-    NC* nc = NULL;
     NC_GRP_INFO_T* grp = (container->sort == NCGRP ? (NC_GRP_INFO_T*)container
                                                    : ((NC_VAR_INFO_T*)container)->container);
 
-    nc = grp->nc4_info->controller;
-    ncid = nc->ext_ncid | grp->hdr.id;
-
     /* Duplicate the values */
     if ((stat = nc4_get_typelen_mem(grp->nc4_info, typeid, &typesize))) goto done;
     clonesize = len*typesize;
     if((clone = malloc(clonesize))==NULL) {stat = NC_ENOMEM; goto done;}
-    if((stat = nc_copy_data(ncid, typeid, values, len, clone))) goto done;
-
+    if((stat = NC_copy_data(grp->nc4_info->controller, typeid, values, len, clone))) goto done;
     if((stat=nc4_att_list_add(attlist,name,&att)))
 	goto done;
     if((zatt = calloc(1,sizeof(NCZ_ATT_INFO_T))) == NULL)
