@@ -43,11 +43,15 @@ extern "C" {
 #include "ncbytes.h"
 #include "ncauth.h"
 #include "nclog.h"
-#include "ncrc.h"
+#include "ncutil.h"
+#include "ncs3sdk.h"
 #include "ncindex.h"
 #include "ncjson.h"
+#include "ncproplist.h"
+#include "ncutil.h"
 
 #include "zmap.h"
+#include "zmetadata.h"
 #include "zinternal.h"
 #include "zdispatch.h"
 #include "zprovenance.h"
