@@ -16,12 +16,7 @@
  */
 
 #include "zincludes.h"
-
-/* These are the default chunk cache sizes for ZARR files created or
- * opened with netCDF-4. */
-extern size_t ncz_chunk_cache_size;
-extern size_t ncz_chunk_cache_nelems;
-extern float ncz_chunk_cache_preemption;
+#include "zfilter.h"
 
 /* Forward */
 
@@ -60,23 +55,21 @@ NCZ_initialize_internal(void)
 {
     int stat = NC_NOERR;
     char* dimsep = NULL;
-    NCRCglobalstate* ngs = NULL;
+    NCglobalstate* ngs = NULL;
 
     ncz_initialized = 1;
-    /* Load the .rc file */
-    if((stat=NC_rcload())) goto done;
-    ngs = ncrc_getglobalstate();
+    ngs = NC_getglobalstate();
     if(ngs != NULL) {
         /* Defaults */
 	ngs->zarr.dimension_separator = DFALT_DIM_SEPARATOR;
-        dimsep = NC_rclookup("ZARR.DIMENSION_SEPARATOR",NULL);
+        dimsep = NC_rclookup("ZARR.DIMENSION_SEPARATOR",NULL,NULL);
         if(dimsep != NULL) {
             /* Verify its value */
 	    if(dimsep != NULL && strlen(dimsep) == 1 && islegaldimsep(dimsep[0]))
 		ngs->zarr.dimension_separator = dimsep[0];
         }    
     }
-done:
+
     return stat;
 }
 
@@ -89,6 +82,12 @@ NCZ_finalize_internal(void)
 {
     /* Reclaim global resources */
     ncz_initialized = 0;
+#ifdef NETCDF_ENABLE_NCZARR_FILTERS
+    NCZ_filter_finalize();
+#endif
+#ifdef NETCDF_ENABLE_S3
+    NCZ_s3finalize();
+#endif
     return NC_NOERR;
 }
 
@@ -114,7 +113,7 @@ find_var_dim_max_length(NC_GRP_INFO_T *grp, int varid, int dimid,
     *maxlen = 0;
 
     /* Find this var. */
-    var = (NC_VAR_INFO_T*)ncindexith(grp->vars,varid);
+    var = (NC_VAR_INFO_T*)ncindexith(grp->vars,(size_t)varid);
     if (!var) return NC_ENOTVAR;
     assert(var->hdr.id == varid);
 
@@ -231,7 +230,7 @@ ncz_find_dim_len(NC_GRP_INFO_T *grp, int dimid, size_t **len)
 {
     NC_VAR_INFO_T *var;
     int retval;
-    int i;
+    size_t i;
 
     assert(grp && len);
     LOG((3, "%s: grp->name %s dimid %d", __func__, grp->hdr.name, dimid));
@@ -330,10 +329,10 @@ close_vars(NC_GRP_INFO_T *grp)
             {
                 if (var->type_info)
                 {
-                    if (var->type_info->nc_type_class == NC_VLEN)
-                        nc_free_vlen((nc_vlen_t *)var->fill_value);
-                    else if (var->type_info->nc_type_class == NC_STRING && *(char **)var->fill_value)
-                        free(*(char **)var->fill_value);
+		    int stat = NC_NOERR;
+		    if((stat = NC_reclaim_data(grp->nc4_info,var->type_info->hdr.id,var->fill_value,1)))
+		        return stat;
+		    nullfree(var->fill_value);
                 }
             }
         }
@@ -359,7 +358,7 @@ close_vars(NC_GRP_INFO_T *grp)
 
 	/* Reclaim filters */
 	if(var->filters != NULL) {
-	    (void)NCZ_filter_freelist(var);
+	    (void)NCZ_filter_freelists(var);
 	}
 	var->filters = NULL;
 
@@ -383,7 +382,7 @@ static int
 close_dims(NC_GRP_INFO_T *grp)
 {
     NC_DIM_INFO_T *dim;
-    int i;
+    size_t i;
 
     for (i = 0; i < ncindexsize(grp->dim); i++)
     {
@@ -418,7 +417,7 @@ close_dims(NC_GRP_INFO_T *grp)
 static int
 close_types(NC_GRP_INFO_T *grp)
 {
-    int i;
+    size_t i;
 
     for (i = 0; i < ncindexsize(grp->type); i++)
     {
@@ -457,7 +456,7 @@ close_types(NC_GRP_INFO_T *grp)
 static int
 ncz_rec_grp_NCZ_del(NC_GRP_INFO_T *grp)
 {
-    int i;
+    size_t i;
     int retval;
 
     assert(grp && grp->format_grp_info);
@@ -609,7 +608,7 @@ ncz_find_grp_var_att(int ncid, int varid, const char *name, int attnum,
     if (att)
     {
         my_att = use_name ? (NC_ATT_INFO_T *)ncindexlookup(attlist, my_norm_name) :
-            (NC_ATT_INFO_T *)ncindexith(attlist, attnum);
+            (NC_ATT_INFO_T *)ncindexith(attlist, (size_t)attnum);
         if (!my_att)
             return NC_ENOTATT;
     }
@@ -630,46 +629,61 @@ ncz_find_grp_var_att(int ncid, int varid, const char *name, int attnum,
 }
 
 /**
- * @internal What fill value should be used for a variable?
+ * @internal Ensure that either var->no_fill || var->fill_value != NULL.
+ * Side effects: set as default if necessary and build _FillValue attribute.
  *
  * @param h5 Pointer to file info struct.
  * @param var Pointer to variable info struct.
- * @param fillp Pointer that gets pointer to fill value.
  *
  * @returns NC_NOERR No error.
  * @returns NC_ENOMEM Out of memory.
  * @author Ed Hartnett, Dennis Heimbigner
  */
 int
-ncz_get_fill_value(NC_FILE_INFO_T *h5, NC_VAR_INFO_T *var, void **fillp)
+NCZ_ensure_fill_value(NC_VAR_INFO_T *var)
 {
     size_t size;
-    int retval;
+    int retval = NC_NOERR;
+    NC_FILE_INFO_T *h5 = var->container->nc4_info;
 
+    if(var->no_fill)
+        return NC_NOERR;
+
+#if 0 /*LOOK*/
     /* Find out how much space we need for this type's fill value. */
     if (var->type_info->nc_type_class == NC_VLEN)
         size = sizeof(nc_vlen_t);
     else if (var->type_info->nc_type_class == NC_STRING)
         size = sizeof(char *);
     else
-    {
-        if ((retval = nc4_get_typelen_mem(h5, var->type_info->hdr.id, &size)))
-            return retval;
-    }
+#endif
+
+    if ((retval = nc4_get_typelen_mem(h5, var->type_info->hdr.id, &size))) goto done;
     assert(size);
 
-    /* Allocate the space. */
-    if (!((*fillp) = calloc(1, size)))
-        return NC_ENOMEM;
+    /* If the user has set a fill_value for this var, use, otherwise find the default fill value. */
 
-    /* If the user has set a fill_value for this var, use, otherwise
-     * find the default fill value. */
-    if (var->fill_value)
-    {
-        LOG((4, "Found a fill value for var %s", var->hdr.name));
+    if (var->fill_value == NULL) {
+	/* initialize the fill_value to the default */
+	/* Allocate the fill_value space. */
+        if((var->fill_value = calloc(1, size))==NULL)
+	    {retval = NC_ENOMEM; goto done;}
+        if((retval = nc4_get_default_fill_value(var->type_info, var->fill_value))) {
+            /* Note: release memory, but don't return error on failure */
+	    (void)NCZ_reclaim_fill_value(var);
+	    retval = NC_NOERR;
+	    goto done;
+        }
+    }
+    assert(var->fill_value != NULL);
+
+    LOG((4, "Found a fill value for var %s", var->hdr.name));
+#if 0 /*LOOK*/
+	/* Need to copy both vlen and a single basetype */
         if (var->type_info->nc_type_class == NC_VLEN)
         {
-            nc_vlen_t *in_vlen = (nc_vlen_t *)(var->fill_value), *fv_vlen = (nc_vlen_t *)(*fillp);
+            nc_vlen_t *in_vlen = (nc_vlen_t *)(var->fill_value);
+	    nc_vlen-t *fv_vlen = (nc_vlen_t *)fill;
             size_t basetypesize = 0;
 
             if((retval=nc4_get_typelen_mem(h5, var->type_info->u.v.base_nc_typeid, &basetypesize)))
@@ -694,20 +708,9 @@ ncz_get_fill_value(NC_FILE_INFO_T *h5, NC_VAR_INFO_T *var, void **fillp)
                     return NC_ENOMEM;
                 }
         }
-        else
-            memcpy((*fillp), var->fill_value, size);
-    }
-    else
-    {
-        if (nc4_get_default_fill_value(var->type_info->hdr.id, *fillp))
-        {
-            /* Note: release memory, but don't return error on failure */
-            free(*fillp);
-            *fillp = NULL;
-        }
-    }
-
-    return NC_NOERR;
+#endif /*0*/
+done:
+    return retval;
 }
 
 #ifdef LOGGING
@@ -727,7 +730,7 @@ int
 NCZ_set_log_level()
 {
     /* If the user wants to completely turn off logging, turn off NCZ
-       logging too. Now I truely can't think of what to do if this
+       logging too. Now I truly can't think of what to do if this
        fails, so just ignore the return code. */
     if (nc_log_level == NC_TURN_OFF_LOGGING)
     {
@@ -749,6 +752,27 @@ NCZ_set_log_level()
 }
 #endif /* LOGGING */
 
+/**
+ * @internal Get the format (i.e. NC_FORMAT_NETCDF4 pr
+ * NC_FORMAT_NETCDF4_CLASSIC) of an open netCDF-4 file.
+ *
+ * @param ncid File ID (ignored).
+ * @param formatp Pointer that gets the constant indicating format.
+
+ * @return ::NC_NOERR No error.
+ * @return ::NC_EBADID Bad ncid.
+ * @author Ed Hartnett
+ */
+int
+NCZ_inq_format(int ncid, int *formatp)
+{
+    int stat = NC_NOERR;
+
+    ZTRACE(0,"ncid=%d formatp=%p",ncid,formatp);
+    stat = NC4_inq_format(ncid,formatp);
+    return ZUNTRACEX(stat,"formatp=%d",(formatp?-1:*formatp));
+}
+
 /**
  * @internal Return the extended format (i.e. the dispatch model),
  * plus the mode associated with an open file.
@@ -756,7 +780,7 @@ NCZ_set_log_level()
  * @param ncid File ID (ignored).
  * @param formatp a pointer that gets the extended format. Note that
  * this is not the same as the format provided by nc_inq_format(). The
- * extended foramt indicates the dispatch layer model. NetCDF-4 files
+ * extended format indicates the dispatch layer model. NetCDF-4 files
  * will always get NC_FORMATX_NC4.
  * @param modep a pointer that gets the open/create mode associated with
  * this file. Ignored if NULL.
