@@ -11,6 +11,7 @@
  */
 
 #include "zincludes.h"
+#include <stddef.h>
 
 #undef DEBUG
 
@@ -18,7 +19,7 @@
 /* Static zarr type name table */
 
 /* Table of nc_type X {Zarr,NCZarr} X endianness
-Issue: Need to distinquish NC_STRING && MAXSTRLEN==1 from NC_CHAR
+Issue: Need to distinguish NC_STRING && MAXSTRLEN==1 from NC_CHAR
 in a way that allows other Zarr implementations to read the data.
 
 Available info:
@@ -31,7 +32,7 @@ so if we have "var:strattr = \"abcdef\"", then we need
 to decide how to infer the type: NC_STRING vs NC_CHAR.
 
 Solution:
-For variables and for NCZarr type attributes, distinquish by using:
+For variables and for NCZarr type attributes, distinguish by using:
 * ">S1" for NC_CHAR.
 * "|S1" for NC_STRING && MAXSTRLEN==1
 * "|Sn" for NC_STRING && MAXSTRLEN==n
@@ -125,7 +126,7 @@ NCZ_grpkey(const NC_GRP_INFO_T* grp, char** pathp)
     NClist* segments = nclistnew();
     NCbytes* path = NULL;
     NC_GRP_INFO_T* parent = NULL;
-    int i;
+    size_t i;
 
     nclistinsert(segments,0,(void*)grp);
     parent = grp->parent;
@@ -225,8 +226,9 @@ ncz_splitkey(const char* key, NClist* segments)
 @internal Down load a .z... structure into memory
 @param zmap - [in] controlling zarr map
 @param key - [in] .z... object to load
-@param jsonp - [out] root of the loaded json
+@param jsonp - [out] root of the loaded json (NULL if key does not exist)
 @return NC_NOERR
+@return NC_EXXX
 @author Dennis Heimbigner
 */
 int
@@ -237,17 +239,22 @@ NCZ_downloadjson(NCZMAP* zmap, const char* key, NCjson** jsonp)
     char* content = NULL;
     NCjson* json = NULL;
 
-    if((stat = nczmap_len(zmap, key, &len)))
-	goto done;
+    switch(stat = nczmap_len(zmap, key, &len)) {
+    case NC_NOERR: break;
+    case NC_ENOOBJECT: case NC_EEMPTY:
+        stat = NC_NOERR;
+        goto exit;
+    default: goto done;
+    }
     if((content = malloc(len+1)) == NULL)
 	{stat = NC_ENOMEM; goto done;}
     if((stat = nczmap_read(zmap, key, 0, len, (void*)content)))
 	goto done;
     content[len] = '\0';
-
     if((stat = NCJparse(content,0,&json)) < 0)
 	{stat = NC_ENCZARR; goto done;}
 
+exit:
     if(jsonp) {*jsonp = json; json = NULL;}
 
 done:
@@ -265,7 +272,7 @@ done:
 @author Dennis Heimbigner
 */
 int
-NCZ_uploadjson(NCZMAP* zmap, const char* key, NCjson* json)
+NCZ_uploadjson(NCZMAP* zmap, const char* key, const NCjson* json)
 {
     int stat = NC_NOERR;
     char* content = NULL;
@@ -280,8 +287,11 @@ fprintf(stderr,"uploadjson: %s\n",key); fflush(stderr);
 	goto done;
     ZTRACEMORE(4,"\tjson=%s",content);
     
+if(getenv("NCS3JSON") != NULL)
+fprintf(stderr,">>>> uploadjson: %s: %s\n",key,content);
+
     /* Write the metadata */
-    if((stat = nczmap_write(zmap, key, 0, strlen(content), content)))
+    if((stat = nczmap_write(zmap, key, strlen(content), content)))
 	goto done;
 
 done:
@@ -306,13 +316,9 @@ NCZ_createdict(NCZMAP* zmap, const char* key, NCjson** jsonp)
     NCjson* json = NULL;
 
     /* See if it already exists */
-    stat = NCZ_downloadjson(zmap,key,&json);
-    if(stat != NC_NOERR) {
-	if(stat == NC_EEMPTY) {/* create it */
-	    if((stat = nczmap_def(zmap,key,NCZ_ISMETA)))
-		goto done;	    
-        } else
-	    goto done;
+    if((stat = NCZ_downloadjson(zmap,key,&json))) goto done;
+    ifjson == NULL) {
+	if((stat = nczmap_def(zmap,key,NCZ_ISMETA))) goto done;	    
     } else {
 	/* Already exists, fail */
 	stat = NC_EINVAL;
@@ -342,18 +348,14 @@ NCZ_createarray(NCZMAP* zmap, const char* key, NCjson** jsonp)
     int stat = NC_NOERR;
     NCjson* json = NULL;
 
-    stat = NCZ_downloadjson(zmap,key,&json);
-    if(stat != NC_NOERR) {
-	if(stat == NC_EEMPTY) {/* create it */
-	    if((stat = nczmap_def(zmap,key,NCZ_ISMETA)))
-		goto done;	    
-	    /* Create the initial array */
-	    if((stat = NCJnew(NCJ_ARRAY,&json)))
-		goto done;
-        } else {
-	    stat = NC_EINVAL;
-	    goto done;
-	}
+    if((stat = NCZ_downloadjson(zmap,key,&json))) goto done;
+    if(json == NULL) { /* create it */
+	if((stat = nczmap_def(zmap,key,NCZ_ISMETA))) goto done;	    
+        /* Create the initial array */
+	if((stat = NCJnew(NCJ_ARRAY,&json))) goto done;
+    } else {
+	 stat = NC_EINVAL;
+	goto done;
     }
     if(json->sort != NCJ_ARRAY) {stat = NC_ENCZARR; goto done;}
     if(jsonp) {*jsonp = json; json = NULL;}
@@ -363,54 +365,6 @@ done:
 }
 #endif /*0*/
 
-/**
-@internal Get contents of a meta object; fail it it does not exist
-@param zmap - [in] map
-@param key - [in] key of the object
-@param jsonp - [out] return parsed json
-@return NC_NOERR
-@return NC_EEMPTY [object did not exist]
-@author Dennis Heimbigner
-*/
-int
-NCZ_readdict(NCZMAP* zmap, const char* key, NCjson** jsonp)
-{
-    int stat = NC_NOERR;
-    NCjson* json = NULL;
-
-    if((stat = NCZ_downloadjson(zmap,key,&json)))
-	goto done;
-    if(NCJsort(json) != NCJ_DICT) {stat = NC_ENCZARR; goto done;}
-    if(jsonp) {*jsonp = json; json = NULL;}
-done:
-    NCJreclaim(json);
-    return stat;
-}
-
-/**
-@internal Get contents of a meta object; fail it it does not exist
-@param zmap - [in] map
-@param key - [in] key of the object
-@param jsonp - [out] return parsed json
-@return NC_NOERR
-@return NC_EEMPTY [object did not exist]
-@author Dennis Heimbigner
-*/
-int
-NCZ_readarray(NCZMAP* zmap, const char* key, NCjson** jsonp)
-{
-    int stat = NC_NOERR;
-    NCjson* json = NULL;
-
-    if((stat = NCZ_downloadjson(zmap,key,&json)))
-	goto done;
-    if(NCJsort(json) != NCJ_ARRAY) {stat = NC_ENCZARR; goto done;}
-    if(jsonp) {*jsonp = json; json = NULL;}
-done:
-    NCJreclaim(json);
-    return stat;
-}
-
 #if 0
 /**
 @internal Given an nc_type, produce the corresponding
@@ -447,19 +401,6 @@ ncz_fill_value_sort(nc_type nctype, int* sortp)
     return NC_NOERR;	        
 }
 
-/* Return 1 if this machine is little endian */
-int
-NCZ_isLittleEndian(void)
-{
-    union {
-        unsigned char bytes[SIZEOF_INT];
-	int i;
-    } u;
-    u.i = 1;
-    return (u.bytes[0] == 1 ? 1 : 0);
-}
-
-
 /*
 Given a path to a group, return the list of objects
 that contain another object with the name of the tag.
@@ -472,7 +413,8 @@ Note: need to test with "/", "", and with and without trailing "/".
 int
 NCZ_subobjects(NCZMAP* map, const char* prefix, const char* tag, char dimsep, NClist* objlist)
 {
-    int i,stat=NC_NOERR;
+    size_t i;
+    int stat = NC_NOERR;
     NClist* matches = nclistnew();
     NCbytes* path = ncbytesnew();
 
@@ -659,7 +601,7 @@ primarily on the first atomic value encountered
 recursively.
 */
 int
-NCZ_inferattrtype(NCjson* value, nc_type typehint, nc_type* typeidp)
+NCZ_inferattrtype(const NCjson* value, nc_type typehint, nc_type* typeidp)
 {
     int i,stat = NC_NOERR;
     nc_type typeid;
@@ -668,7 +610,7 @@ NCZ_inferattrtype(NCjson* value, nc_type typehint, nc_type* typeidp)
     long long i64;
     int negative = 0;
 
-    if(NCJsort(value) == NCJ_ARRAY && NCJlength(value) == 0)
+    if(NCJsort(value) == NCJ_ARRAY && NCJarraylength(value) == 0)
         {typeid = NC_NAT; goto done;} /* Empty array is illegal */
 
     if(NCJsort(value) == NCJ_NULL)
@@ -679,7 +621,7 @@ NCZ_inferattrtype(NCjson* value, nc_type typehint, nc_type* typeidp)
 
     /* If an array, make sure all the elements are simple */
     if(value->sort == NCJ_ARRAY) {
-	for(i=0;i<NCJlength(value);i++) {
+	for(i=0;i<NCJarraylength(value);i++) {
 	    j=NCJith(value,i);
 	    if(!NCJisatomic(j))
 	        {typeid = NC_NAT; goto done;}
@@ -798,8 +740,8 @@ NCZ_comma_parse(const char* s, NClist* list)
 	endp = strchr(p,',');
 	if(endp == NULL) endp = p + strlen(p);
 	slen = (endp - p);
-	if((s = malloc(slen+1)) == NULL) {stat = NC_ENOMEM; goto done;}
-	memcpy(s,p,slen);
+	if((s = malloc((size_t)slen+1)) == NULL) {stat = NC_ENOMEM; goto done;}
+	memcpy(s,p,(size_t)slen);
 	s[slen] = '\0';
 	if(nclistmatch(list,s,0)) {
 	    nullfree(s); /* duplicate */
@@ -815,6 +757,7 @@ done:
 }
 
 /**************************************************/
+#if 0
 /* Endianness support */
 /* signature: void swapinline16(void* ip) */
 #define swapinline16(ip) \
@@ -853,6 +796,7 @@ done:
     u.b[7] = src[0]; \
     *((unsigned long long*)ip) = u.i; \
 }
+#endif /*0*/
 
 int
 NCZ_swapatomicdata(size_t datalen, void* data, int typesize)
@@ -948,9 +892,8 @@ NCZ_reclaim_fill_value(NC_VAR_INFO_T* var)
 {
     int stat = NC_NOERR;
     if(var->fill_value) {
-	int ncid = var->container->nc4_info->controller->ext_ncid;
 	int tid = var->type_info->hdr.id;
-	stat = nc_reclaim_data_all(ncid,tid,var->fill_value,1);
+	stat = NC_reclaim_data_all(var->container->nc4_info->controller,tid,var->fill_value,1);
 	var->fill_value = NULL;
     }
     /* Reclaim any existing fill_chunk */
@@ -962,16 +905,15 @@ int
 NCZ_copy_fill_value(NC_VAR_INFO_T* var, void**  dstp)
 {
     int stat = NC_NOERR;
-    int ncid = var->container->nc4_info->controller->ext_ncid;
     int tid = var->type_info->hdr.id;
     void* dst = NULL;
 
     if(var->fill_value) {
-	if((stat = nc_copy_data_all(ncid,tid,var->fill_value,1,&dst))) goto done;
+	if((stat = NC_copy_data_all(var->container->nc4_info->controller,tid,var->fill_value,1,&dst))) goto done;
     }
     if(dstp) {*dstp = dst; dst = NULL;}
 done:
-    if(dst) (void)nc_reclaim_data_all(ncid,tid,dst,1);
+    if(dst) (void)NC_reclaim_data_all(var->container->nc4_info->controller,tid,dst,1);
     return stat;
 }
 
@@ -1012,12 +954,12 @@ NCZ_fixed2char(const void* fixed, char** charp, size_t count, int maxstrlen)
 	if(p[0] == '\0') {
 	    sp = NULL;
 	} else {
-	    if((sp = (unsigned char*)malloc(maxstrlen+1))==NULL) /* ensure null terminated */
+	    if((sp = (unsigned char*)malloc((size_t)maxstrlen+1))==NULL) /* ensure null terminated */
 	        return NC_ENOMEM; 
-	    memcpy(sp,p,maxstrlen);
+	    memcpy(sp,p,(size_t)maxstrlen);
 	    sp[maxstrlen] = '\0';
 	}
-	charp[i] = sp;
+	charp[i] = (char*)sp;
 	sp = NULL;
     }
     return NC_NOERR;
@@ -1046,9 +988,11 @@ NCZ_char2fixed(const char** charp, void* fixed, size_t count, int maxstrlen)
 Wrap NC_copy_data, but take string value into account when overwriting
 */
 int
-NCZ_copy_data(NC_FILE_INFO_T* file, NC_TYPE_INFO_T* xtype, const void* memory, size_t count, int noclear, void* copy)
+NCZ_copy_data(NC_FILE_INFO_T* file, NC_VAR_INFO_T* var, const void* memory, size_t count, int reading, void* copy)
 {
-    if(xtype->hdr.id == NC_STRING && !noclear) {
+    int stat = NC_NOERR;    
+    NC_TYPE_INFO_T* xtype = var->type_info;
+    if(xtype->hdr.id == NC_STRING && !reading) {
 	size_t i;
 	char** scopy = (char**)copy;
 	/* Reclaim any string fill values in copy */
@@ -1057,7 +1001,8 @@ NCZ_copy_data(NC_FILE_INFO_T* file, NC_TYPE_INFO_T* xtype, const void* memory, s
 	    scopy[i] = NULL;
 	}
     }
-    return nc_copy_data(file->controller->ext_ncid,xtype->hdr.id,memory,count,copy);
+    stat = NC_copy_data(file->controller,xtype->hdr.id,memory,count,copy);
+    return stat;
 }
 
 #if 0
@@ -1070,7 +1015,7 @@ checksimplejson(NCjson* json, int depth)
     switch (NCJsort(json)) {
     case NCJ_ARRAY:
 	if(depth > 0) return 0;  /* e.g. [...,[...],...]  or [...,{...},...] */
-	for(i=0;i < NCJlength(json);i++) {
+	for(i=0;i < NCJarraylength(json);i++) {
 	    NCjson* j = NCJith(json,i);
 	    if(!checksimplejson(j,depth+1)) return 0;
         }
@@ -1087,7 +1032,7 @@ checksimplejson(NCjson* json, int depth)
 
 /* Return 1 if the attribute will be stored as a complex JSON valued attribute; return 0 otherwise */
 int
-NCZ_iscomplexjson(NCjson* json, nc_type typehint)
+NCZ_iscomplexjson(const NCjson* json, nc_type typehint)
 {
     int i, stat = 0;
 
@@ -1096,7 +1041,7 @@ NCZ_iscomplexjson(NCjson* json, nc_type typehint)
 	/* If the typehint is NC_CHAR, then always treat it as complex */
 	if(typehint == NC_CHAR) {stat = 1; goto done;}
 	/* Otherwise see if it is a simple vector of atomic values */
-	for(i=0;i < NCJlength(json);i++) {
+	for(i=0;i < NCJarraylength(json);i++) {
 	    NCjson* j = NCJith(json,i);
 	    if(!NCJisatomic(j)) {stat = 1; goto done;}
         }
