@@ -282,7 +282,7 @@ write_meta(int ncid, int *data_varid, int s, int f, int nsd, int deflate, int u,
     {
         char data_var_name[NC_MAX_NAME + 1];
 
-        sprintf(data_var_name, "var_%d", dv);
+        snprintf(data_var_name, sizeof(data_var_name), "var_%d", dv);
         if (nc_redef(ncid)) ERR;
         if (nc_def_var(ncid, data_var_name, NC_FLOAT, NDIM4, dimid_data, &data_varid[dv])) ERR;
 
@@ -511,7 +511,7 @@ main(int argc, char **argv)
     /* Determine 2D data decomposition to write lat/lon coordinate vars. */
     if (decomp_latlon(dim_len, latlon_start, latlon_count, &lat, &lon)) ERR;
 
-    /* Decompose grid_xt and grid_yt coordiate vars. */
+    /* Decompose grid_xt and grid_yt coordinate vars. */
     if (decomp_grid(dim_len, &grid_xt_start, &grid_xt_size,
                     &grid_yt_start, &grid_yt_size, &grid_xt, &grid_yt)) ERR;
 
@@ -567,7 +567,7 @@ main(int argc, char **argv)
 
                         /* Use the same filename every time, so we don't
                          * create many large files, just one. ;-) */
-                        sprintf(file_name, "%s.nc", TEST_NAME);
+                        snprintf(file_name, sizeof(file_name), "%s.nc", TEST_NAME);
 
                         /* Remove the last file. Ignore errors. */
                         remove(file_name);
