@@ -1,6 +1,5 @@
 #!/bin/sh
 
-
 if test "x$srcdir" = x ; then srcdir=`pwd`; fi
 . ../test_common.sh
 
@@ -22,20 +21,19 @@ OMEMFILE=f03tst_open_mem.nc
 echo ""
 echo "*** Testing in-memory operations"
 
-HASHDF5=`${top_builddir}/nc-config --has-hdf5`
-
 # Execute the core of the inmemory tests
 ${execdir}/tst_inmemory
-if test "x$HASNC4" = xyes ; then
+if test "x$FEATURE_HDF5" = "xyes" ; then
 ${execdir}/tst_open_mem ${srcdir}/${OMEMFILE}
 fi
 
-echo "**** Test ncdump of the resulting inmemory data"
+echo "**** Test ncdump of the resulting inmemory data: netcdf-3"
 ${NCDUMP} -n "${FILE3}" ${FILE3}.nc > ${FILE3}.cdl
 ${NCDUMP} -n "${FILE3}" ${CREATE3}.nc > ${CREATE3}.cdl
 diff -wb ${FILE3}.cdl ${CREATE3}.cdl
 
-if test "x$HASHDF5" = "xyes" ; then
+if test "x$FEATURE_HDF5" = "xyes" ; then
+echo "**** Test ncdump of the resulting inmemory data: netcdf-4"
 ${NCDUMP} ${FILE4}.nc > ${FILE4}.cdl
 ${NCDUMP} -n ${FILE4} ${CREATE4}.nc > ${CREATE4}.cdl
 diff -wb ${FILE4}.cdl ${CREATE4}.cdl
