@@ -83,7 +83,7 @@ tst_att_ordering(int cmode)
    int ncid;
    char name[NUM_ATTS][ATT_MAX_NAME + 1] = {"Gc", "Gb", "Gs", "Gi", "Gf",
 					    "Gd", "Gatt-name-dashes", "Gatt.name.dots"};
-   int len[NUM_ATTS] = {0, 2, 3, 3, 3, 3, 1, 1};
+   size_t len[NUM_ATTS] = {0, 2, 3, 3, 3, 3, 1, 1};
    signed char b[2] = {-128, 127};
    short s[3] = {-32768, 0, 32767};
    int i[3] = {42, 0, -42};
@@ -816,6 +816,7 @@ main(int argc, char **argv)
       if (nc_get_att_text(ncid, NC_GLOBAL, ATT_TEXT_NAME2, speech_in)) ERR;
       if (strcmp(speech, speech_in)) ERR;
       free(speech_in);
+      speech_in = NULL;
       if (nc_get_att_text(ncid, NC_GLOBAL, ATT_TEXT_NAME, speech_in) != NC_ENOTATT) ERR;
       if (nc_close(ncid)) ERR;
 
@@ -838,7 +839,7 @@ main(int argc, char **argv)
       if (nc_del_att(ncid, 0, ATT_INT_NAME)) ERR;
       if (nc_close(ncid)) ERR;
 
-      /* Reopen the file and readd the attribute. Enddef and redef,
+      /* Reopen the file and re-add the attribute. Enddef and redef,
        * and delete it, then check to make sure it's gone. */
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
       if (nc_put_att_int(ncid, varid, ATT_INT_NAME, NC_INT, 3, int_out)) ERR;
