@@ -15,6 +15,13 @@ MULTI=1
 REP=1
 ORDER=1
 
+if test "x$TESTNCZARR" = x1; then
+. $srcdir/test_nczarr.sh
+s3isolate "testdir_specific_filters"
+THISDIR=`pwd`
+cd $ISOPATH
+fi
+
 # Load the findplugins function
 . ${builddir}/findplugin.sh
 echo "findplugin.sh loaded"
@@ -43,24 +50,27 @@ sed -e 's/[ 	]*\([^ 	].*\)/\1/' <$1 >$2
 # Locate the plugin dir and the library names; argument order is critical
 # Find bzip2 and capture
 findplugin h5bzip2
-BZIP2LIB="${HDF5_PLUGIN_LIB}"
-BZIP2DIR="${HDF5_PLUGIN_DIR}/${BZIP2LIB}"
+BZIP2PATH="${HDF5_PLUGIN_DIR}/${HDF5_PLUGIN_LIB}"
+
 # Find misc and capture
 findplugin h5misc
-MISCDIR="${HDF5_PLUGIN_DIR}/${HDF5_PLUGIN_LIB}"
+MISCPATH="${HDF5_PLUGIN_DIR}/${HDF5_PLUGIN_LIB}"
+
 # Find noop and capture
 findplugin h5noop
-NOOPLIB="${HDF5_PLUGIN_LIB}"
-NOOPDIR="${HDF5_PLUGIN_DIR}/${HDF5_PLUGIN_LIB}"
+NOOPPATH="${HDF5_PLUGIN_DIR}/${HDF5_PLUGIN_LIB}"
 
 echo "final HDF5_PLUGIN_DIR=${HDF5_PLUGIN_DIR}"
 export HDF5_PLUGIN_DIR
 export HDF5_PLUGIN_PATH="$HDF5_PLUGIN_DIR"
 
 # Verify
-if ! test -f ${BZIP2DIR} ; then echo "Unable to locate ${BZIP2DIR}"; exit 1; fi
-if ! test -f ${MISCDIR} ; then echo "Unable to locate ${MISCDIR}"; exit 1; fi
-if ! test -f ${NOOPDIR} ; then echo "Unable to locate ${NOOPDIR}"; exit 1; fi
+echo "Verify BZIP2PATH"
+if ! test -f ${BZIP2PATH} ; then echo "Unable to locate ${BZIP2PATH}"; exit 1; fi
+echo "Verify MISCPATH"
+if ! test -f ${MISCPATH} ; then echo "Unable to locate ${MISCPATH}"; exit 1; fi
+echo "Verify NOOPPATH"
+if ! test -f ${NOOPPATH} ; then echo "Unable to locate ${NOOPPATH}"; exit 1; fi
 
 # See if we have szip
 if avail szip; then HAVE_SZIP=1; else HAVE_SZIP=0; fi
