@@ -83,7 +83,7 @@ int main(int argc, char **argv)
    facc_type_open = 0;
 
    /* Create file name. */
-   sprintf(file_name, "%s/%s", TEMP_LARGE, FILE_NAME);
+   snprintf(file_name, sizeof(file_name), "%s/%s", TEMP_LARGE, FILE_NAME);
 
    /* Test NetCDF4 with MPI-IO driver */
    if (mpi_rank == 0)
@@ -743,7 +743,7 @@ int test_pio_extend(int flag){
  * 		further calls to getenv_all take place.
  *
  * Return:	pointer to a string containing the value of the environment variable
- * 		NULL if the varialbe doesn't exist in task 'root's environment.
+ * 		NULL if the variable doesn't exist in task 'root's environment.
  *
  * Programmer:	Leon Arber
  *              4/4/05
