@@ -8,6 +8,7 @@
 
 #include <nc_tests.h>
 #include "err_macros.h"
+#include "hdf5internal.h"
 #include <mpi.h>
 
 #define FILE "tst_parallel5.nc"
@@ -19,9 +20,16 @@
 #define NUM_PROC 4
 #define NUM_SLABS 10
 #define NUM_ACCESS_TESTS 2
-#define HDF5_DEFAULT_CACHE_SIZE 1048576
-#define HDF5_DEFAULT_NELEMS 521
-#define HDF5_DEFAULT_PREEMPTION 0.75
+
+#if H5_VERSION_GE(2,0,0)
+    #define HDF5_DEFAULT_CACHE_SIZE 8388608
+    #define HDF5_DEFAULT_NELEMS 8191
+    #define HDF5_DEFAULT_PREEMPTION 0.75
+#else
+    #define HDF5_DEFAULT_CACHE_SIZE 1048576
+    #define HDF5_DEFAULT_NELEMS 521
+    #define HDF5_DEFAULT_PREEMPTION 0.75
+#endif
 
 int
 nc4_hdf5_get_chunk_cache(int ncid, size_t *sizep, size_t *nelemsp,
@@ -56,7 +64,7 @@ main(int argc, char **argv)
         for (acc = 0; acc < NUM_ACCESS_TESTS; acc++)
         {
             /* Create a parallel netcdf-4 file. */
-            /*nc_set_log_level(3);*/
+            /* nc_set_log_level(3); */
             if (nc_create_par(FILE, NC_NETCDF4, comm, info, &ncid)) ERR;
 
             /* Create a dimension. */
@@ -88,6 +96,7 @@ main(int argc, char **argv)
 
             /* Close the netcdf file. */
             if (nc_close(ncid)) ERR;
+            /* nc_set_log_level(-1); */
         }
     }
     if (!mpi_rank)
@@ -112,7 +121,6 @@ main(int argc, char **argv)
             unsigned long long data = 1000000, data_in = TEST_VAL_42;
 
             /* Create a parallel netcdf-4 file. */
-            /*nc_set_log_level(3);*/
             if (nc_create_par(FILE, NC_NETCDF4, comm, info, &ncid)) ERR;
 
             /* Create a dimension. */
@@ -176,7 +184,6 @@ main(int argc, char **argv)
             int dim_size = NC_MAX_NAME + 1;
 
             /* Create a parallel netcdf-4 file. */
-            /*nc_set_log_level(3);*/
             if (nc_create_par(FILE, NC_NETCDF4, comm, info, &ncid)) ERR;
 
             /* Create a dimension. */
@@ -331,8 +338,18 @@ main(int argc, char **argv)
                           &ncid)) ERR;
         if (nc4_hdf5_get_chunk_cache(ncid, &size, &nelems, &preemption)) ERR;
         if (size != HDF5_DEFAULT_CACHE_SIZE || nelems != HDF5_DEFAULT_NELEMS ||
-            preemption != HDF5_DEFAULT_PREEMPTION) ERR;
-        /* printf("%ld %ld %g\n", size, nelems, preemption); */
+            preemption != HDF5_DEFAULT_PREEMPTION) {
+                printf("hdf5_default: size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                    size, HDF5_DEFAULT_CACHE_SIZE,
+                    nelems, HDF5_DEFAULT_NELEMS,
+                    preemption, HDF5_DEFAULT_PREEMPTION); 
+                ERR;
+            } else {
+                /*printf("hdf5_default: size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                    size, HDF5_DEFAULT_CACHE_SIZE,
+                    nelems, HDF5_DEFAULT_NELEMS,
+                    preemption, HDF5_DEFAULT_PREEMPTION); */
+            }
         if (nc_close(ncid)) ERR;
 
         /* Create a file with sequential I/O and check cache settings
@@ -345,7 +362,18 @@ main(int argc, char **argv)
             if (nc4_hdf5_get_chunk_cache(ncid, &size, &nelems, &preemption)) ERR;
             /* printf("%ld %ld %g\n", size, nelems, preemption); */
             if (size != CHUNK_CACHE_SIZE || nelems != CHUNK_CACHE_NELEMS ||
-                preemption != CHUNK_CACHE_PREEMPTION) ERR;
+                preemption != CHUNK_CACHE_PREEMPTION) {
+                    printf("netcdf_default: size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                        size, CHUNK_CACHE_SIZE,
+                        nelems, CHUNK_CACHE_NELEMS,
+                        preemption, CHUNK_CACHE_PREEMPTION); 
+                    ERR;    
+                } else {
+                    /*printf("netcdf_default: size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                        size, CHUNK_CACHE_SIZE,
+                        nelems, CHUNK_CACHE_NELEMS,
+                        preemption, CHUNK_CACHE_PREEMPTION); */
+                }
             if (nc_close(ncid)) ERR;
         }
 
@@ -353,7 +381,18 @@ main(int argc, char **argv)
         if (nc_open_par(FILE, 0, comm, info, &ncid)) ERR;
         if (nc4_hdf5_get_chunk_cache(ncid, &size, &nelems, &preemption)) ERR;
         if (size != HDF5_DEFAULT_CACHE_SIZE || nelems != HDF5_DEFAULT_NELEMS ||
-            preemption != HDF5_DEFAULT_PREEMPTION) ERR;
+            preemption != HDF5_DEFAULT_PREEMPTION) {
+                printf("netcdf_default (reopen): size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                    size, HDF5_DEFAULT_CACHE_SIZE,
+                    nelems, HDF5_DEFAULT_NELEMS,
+                    preemption, HDF5_DEFAULT_PREEMPTION); 
+                ERR;    
+            } else {
+               /* printf("hdf5_default (reopen): size: %ld (%ld) nelems: %ld (%ld) preemption: %g (%g)\n", 
+                    size, HDF5_DEFAULT_CACHE_SIZE,
+                    nelems, HDF5_DEFAULT_NELEMS,
+                    preemption, HDF5_DEFAULT_PREEMPTION); */
+            }
         if (nc_close(ncid)) ERR;
 
         /* Open the file with sequential I/O and check cache settings
@@ -368,11 +407,10 @@ main(int argc, char **argv)
                 preemption != CHUNK_CACHE_PREEMPTION) ERR;
             if (nc_close(ncid)) ERR;
         }
-        nc_set_log_level(0);
     }
     if (!mpi_rank)
         SUMMARIZE_ERR;
-#ifdef USE_SZIP
+#ifdef HAVE_H5Z_SZIP
 #ifdef HDF5_SUPPORTS_PAR_FILTERS
 #define SZIP_DIM_LEN 256
 #define SZIP_DIM_NAME "Barrels"
@@ -421,9 +459,9 @@ main(int argc, char **argv)
     if (!mpi_rank)
         SUMMARIZE_ERR;
 #endif /* HDF5_SUPPORTS_PAR_FILTERS */
-#endif /* USE_SZIP */
+#endif /* HAVE_H5Z_SZIP */
 
-#ifdef USE_SZIP
+#ifdef HAVE_H5Z_SZIP
 #ifdef HDF5_SUPPORTS_PAR_FILTERS
 #define SZIP_DIM_LEN 256
 #define SZIP_DIM_NAME "Barrels"
@@ -471,7 +509,61 @@ main(int argc, char **argv)
     if (!mpi_rank)
         SUMMARIZE_ERR;
 #endif /* HDF5_SUPPORTS_PAR_FILTERS */
-#endif /* USE_SZIP */
+#endif /* HAVE_H5Z_SZIP */
+
+#ifdef HAVE_H5Z_SZIP
+#ifdef HDF5_SUPPORTS_PAR_FILTERS
+#define SZIP_DIM_LEN 256
+#define SZIP_DIM_NAME "Barrels"
+#define SZIP_VAR_NAME "Best_Sligo_Rags"
+#define SZIP_PIXELS_PER_BLOCK 32
+    if (!mpi_rank)
+        printf("*** testing zstd compression with parallel I/O...");
+    {
+        int ncid, dimid, varid;
+        float *data;
+        float *data_in;
+        int elements_per_pe = SZIP_DIM_LEN/mpi_size;
+        size_t start[NDIMS1], count[NDIMS1];
+        int i, ret;
+
+        /* Create test data. */
+        if (!(data = malloc(elements_per_pe * sizeof(float)))) ERR;
+        for (i = 0; i < elements_per_pe; i++)
+            data[i] = mpi_rank + i * 0.1;
+
+        /* Crate a file with a scalar NC_BYTE value. */
+        if (nc_create_par(FILE, NC_NETCDF4, MPI_COMM_WORLD, MPI_INFO_NULL,
+                          &ncid)) ERR;
+        if (nc_def_dim(ncid, SZIP_DIM_NAME, SZIP_DIM_LEN, &dimid)) ERR;
+        if (nc_def_var(ncid, SZIP_VAR_NAME, NC_FLOAT, NDIMS1, &dimid, &varid)) ERR;
+        if ((ret = nc_def_var_zstandard(ncid, varid, 4)))
+	{
+	    printf("%s\n", nc_strerror(ret));
+	    ERR;
+	}
+        if (nc_enddef(ncid)) ERR;
+        start[0] = mpi_rank * elements_per_pe;
+        count[0] = elements_per_pe;
+        if (nc_put_vara_float(ncid, varid, start, count, data));
+        if (nc_close(ncid)) ERR;
+
+        /* Reopen the file and check. */
+        if (nc_open_par(FILE, 0, comm, info, &ncid)) ERR;
+        if (!(data_in = malloc(elements_per_pe * sizeof(float)))) ERR;
+        if (nc_get_vara_float(ncid, varid, start, count, data_in));
+        if (nc_close(ncid)) ERR;
+        for (i = 0; i < elements_per_pe; i++)
+            if (data_in[i] != data[i]) ERR;
+
+        /* Release resources. */
+        free(data_in);
+        free(data);
+    }
+    if (!mpi_rank)
+        SUMMARIZE_ERR;
+#endif /* HDF5_SUPPORTS_PAR_FILTERS */
+#endif /* HAVE_H5Z_SZIP */
 
     /* Shut down MPI. */
     MPI_Finalize();
