@@ -25,11 +25,6 @@ BLOSCCODEC='[{\"id\": \"blosc\",\"clevel\": 5,\"blocksize\": 256,\"cname\": \"lz
 . ${builddir}/findplugin.sh
 echo "findplugin.sh loaded"
 
-# Locate the plugin path and the library names; argument order is critical
-# Find bzip2 and capture
-# Assume all test filters are in same plugin dir
-findplugin h5bzip2
-
 # Function to remove selected -s attributes from file;
 # These attributes might be platform dependent
 sclean() {
@@ -63,22 +58,33 @@ sed -e 's/[ 	]*\([^ 	].*\)/\1/' <$1 >$2
 
 
 setfilter() {
-    FF="$1"
-    FSRC="$2"
-    FDST="$3"
-    FIH5="$4"
-    FICX="$5"
+    FF="$1"   # Filter
+    FSRC="$2" # Src file
+    FDST="$3" # Dest file
+    FIH5="$4" # Filter params
+    FICX="$5" # Filter codec
     FFH5="$6"
     FFCX="$7"
     if test "x$FFH5" = x ; then FFH5="$FIH5" ; fi
     if test "x$FFCX" = x ; then FFCX="$FICX" ; fi
     rm -f $FDST
+if test 1 = 1 ; then
     cat ${srcdir}/$FSRC \
 	| sed -e "s/ref_any/${FF}/" \
 	| sed -e "s/IH5/${FIH5}/" -e "s/FH5/${FFH5}/" \
 	| sed -e "s/ICX/${FICX}/" -e "s/FCX/${FFCX}/" \
 	| sed -e 's/"/\\"/g' -e 's/@/"/g' \
 	| cat > $FDST
+else
+    cp ${srcdir}/$FSRC ./tmp_$FF
+    sed -i.bak -e "s/ref_any/${FF}/" < ${srcdir}/${FSRC} ./tmp_$FF
+    sed -i.bak -e "s/IH5/${FIH5}/" -e "s/FH5/${FFH5}/" ./tmp_$FF
+    sed -i.bak -e "s/ICX/${FICX}/" -e "s/FCX/${FFCX}/" ./tmp_$FF
+    sed -i.bak -e 's/"/\\"/g' -e 's/@/"/g' ./tmp_$FF
+    if ls -l $FDST ; then x=0; else x=1; fi
+    mv ./tmp_$FF $FDST
+    rm -f tmp_$FF.bak
+fi
 }
 
 # Execute the specified tests
@@ -128,7 +134,7 @@ testshuffle() {
 testdeflate() {
   zext=$1
   if ! avail deflate; then return 0; fi
-  runfilter $zext deflate '1,9' '[{\"id\": \"zlib\",\"level\": \"9\"}]'
+  runfilter $zext deflate '1,9' '[{\"id\": \"zlib\",\"level\": 9}]'
   # need to replace _DeflateLevel
 #  sed -e 's/_DeflateLevel = 9/_Filter = "1,9"/' < tmp_filt_deflate.dump > tmp_filt_deflatex.dump
   diff -b -w "tmp_filt_deflate.cdl" "tmp_filt_deflate.dump"
@@ -137,7 +143,7 @@ testdeflate() {
 testbzip2() {
   zext=$1
   if ! avail bzip2; then return 0; fi
-  runfilter $zext bzip2 '307,9' '[{\"id\": \"bz2\",\"level\": \"9\"}]'
+  runfilter $zext bzip2 '307,9' '[{\"id\": \"bz2\",\"level\": 9}]'
   diff -b -w "tmp_filt_bzip2.cdl" "tmp_filt_bzip2.dump"
 }
 
@@ -163,7 +169,7 @@ testblosc() {
 testzstd() {
   zext=$1
   if ! avail zstd; then return 0; fi
-  runfilter $zext zstd '32015,1' '[{\"id\": \"zstd\",\"level\": \"1\"}]'
+  runfilter $zext zstd '32015,1' '[{\"id\": \"zstd\",\"level\": 1}]'
   diff -b -w "tmp_filt_zstd.cdl" "tmp_filt_zstd.dump"
 }
 
