@@ -11,11 +11,11 @@ extern "C" {
 #endif
 
 /* Handle nested group indentation */
-extern void indent_init();	/* initialize indent to zero */
-extern void indent_out();	/* output current indent */
-extern void indent_more();	/* increment current indent */
-extern void indent_less();	/* decrement current indent */
-extern int  indent_get();	/* return current indent */
+extern void indent_init(void);	/* initialize indent to zero */
+extern void indent_out(void);	/* output current indent */
+extern void indent_more(void);	/* increment current indent */
+extern void indent_less(void);	/* decrement current indent */
+extern int  indent_get(void);	/* return current indent */
 
 #ifdef __cplusplus
 }
