@@ -51,9 +51,18 @@ typedef int OCflags;
 /*!\def OCONDISK
 Cause oc_fetch to store the retrieved data on disk.
 */
-
 #define OCONDISK 1
 
+/*!\def OCENCODEPATH
+Cause oc_fetch to encode path part of a URL
+*/
+#define OCENCODEPATH 2
+
+/*!\def OCENCODEQUERY
+Cause oc_fetch to encode query part of a URL
+*/
+#define OCENCODEQUERY 4
+
 /**************************************************/
 /* OCtype */
 
@@ -132,6 +141,7 @@ OC_EBADTYPE=-27,
 OC_ESCALAR=-28,
 OC_EOVERRUN=-29,
 OC_EAUTH=-30,
+OC_EACCESS=-31,
 } OCerror;
 
 /*!\def OCLOGNOTE
@@ -408,7 +418,8 @@ EXTERNL OCerror oc_data_ithelement(OClink, OCdatanode data, size_t* indices, OCd
 EXTERNL OCerror oc_data_ithrecord(OClink, OCdatanode data, size_t index, OCdatanode* recordp);
 
 /* Free up an data that is no longer being used */
-EXTERNL OCerror oc_data_free(OClink, OCdatanode data);
+/* Currently does nothing */
+#define oc_data_free(link,datanode)
 
 /* Count the records associated with a sequence */
 EXTERNL OCerror oc_data_recordcount(OClink, OCdatanode, size_t*);
@@ -438,7 +449,7 @@ EXTERNL OCerror oc_data_readscalar(OClink, OCdatanode, size_t, void*);
 */
 EXTERNL OCerror oc_data_readn(OClink, OCdatanode, const size_t*, size_t, size_t, void*);
 
-/* Return the indices for this datas; Assumes the data
+/* Return the indices for this data; Assumes the data
    was obtained using oc_data_ithelement or oc_data_ithrecord;
    if not, then an error is returned.
 */
@@ -582,7 +593,7 @@ EXTERNL OCerror oc_set_curlopt(OClink link, const char* option, void* value);
 EXTERNL OCerror oc_get_connection(OCobject ocnode, OCobject* linkp);
 
 /* Resend a url as a head request to check the Last-Modified time */
-EXTERNL OCerror oc_update_lastmodified_data(OClink);
+EXTERNL OCerror oc_update_lastmodified_data(OClink,OCflags);
 
 /* Get last known modification time; -1 => data unknown */
 EXTERNL long oc_get_lastmodified_data(OClink);
@@ -596,6 +607,8 @@ EXTERNL OCerror oc_ping(const char* url);
 */
 EXTERNL OCerror oc_raw_xdrsize(OClink,OCddsnode,off_t*);
 
+EXTERNL int ocdebug;
+
 #ifdef __cplusplus
 }
 #endif
