@@ -21,12 +21,14 @@
 #endif
 #endif
 #include "ncrc.h"
+#include "ncutil.h"
 #include "ocinternal.h"
 #include "ocdebug.h"
 #include "ochttp.h"
 #include "ocread.h"
 #include "occurlfunctions.h"
 #include "ncpathmgr.h"
+#include "ncutil.h"
 
 /*Forward*/
 static int readpacket(OCstate* state, NCURI*, NCbytes*, OCdxd, OCflags, long*);
@@ -139,7 +141,7 @@ fprintf(stderr,"readDATADDS:\n");
         stat = readpacket(state,state->uri,state->packet,OCDATADDS,ocflags,&lastmod);
         if(stat == OC_NOERR)
             state->datalastmodified = lastmod;
-        tree->data.datasize = ncbyteslength(state->packet);
+        tree->data.datasize = (off_t)ncbyteslength(state->packet);
     } else { /*((flags & OCONDISK) != 0) */
         NCURI* url = state->uri;
         int fileprotocol = 0;
@@ -201,7 +203,7 @@ fprintf(stderr,"readfiletofile: written!=length: %lu :: %lu\n",
 	    stat = OC_EIO;
 	}
     }
-    if(sizep != NULL) *sizep = len;
+    if(sizep != NULL) *sizep = (off_t)len;
 unwind:
     ncbytesfree(packet);
     return OCTHROW(stat);
