@@ -1,4 +1,4 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
 /*
@@ -66,8 +66,11 @@
 #include <arpa/inet.h>
 #endif
 
+#include "ncexternl.h"
+#include "ncutil.h"
 #include "xxdr.h"
 
+
 int xxdr_network_order; /* network order is big endian */
 static int xxdr_big_endian; /* what is this machine? */
 
@@ -184,7 +187,7 @@ xxdr_string(XXDR* xdrs, char** sp, off_t* lenp)
     s[len] = '\0'; /* make sure it is null terminated */
     if(sp) *sp = s;
     if(lenp) *lenp = len;
-    /* xxdr_opaque will have skippped any trailing bytes */
+    /* xxdr_opaque will have skipped any trailing bytes */
     return 1;    
 }
 
@@ -218,7 +221,7 @@ xxdr_free(XXDR* xdr)
 
 /***********************************/
 
-/* Skip exacly "len" bytes in the input; any rounding must be done by the caller*/
+/* Skip exactly "len" bytes in the input; any rounding must be done by the caller*/
 int
 xxdr_skip(XXDR* xdrs, off_t len)
 {
@@ -248,9 +251,7 @@ xxdr_skip_strings(XXDR* xdrs, off_t n)
 unsigned int
 xxdr_roundup(off_t n)
 {
-    unsigned int rounded;
-    rounded = RNDUP(n);
-    return rounded;
+    return (unsigned int)RNDUP(n);
 }
 
 unsigned int
@@ -280,7 +281,6 @@ static int
 xxdr_filegetbytes(XXDR* xdrs, char* addr, off_t len)
 {
     int ok = 1;
-    int count;
 
 xxdrtrace(xdrs,"getbytes",len);
     if(len < 0) len = 0;
@@ -295,8 +295,8 @@ xxdrtrace(xdrs,"getbytes",len);
     if(xdrs->pos + len > xdrs->length)
         return 0;
     if(len > 0) {
-        count = fread(addr, (size_t)len, (size_t)1, (FILE*)xdrs->data);
-        if(count <= 0) {
+        size_t count = fread(addr, (size_t)len, (size_t)1, (FILE*)xdrs->data);
+        if(count == 0) {
 	    ok=0;
 	    goto done;
 	}
@@ -502,7 +502,8 @@ xxdrntohdouble(char* c8, double* dp)
 	ii[0] = ii[1];
 	ii[1] = tmp;
     }
-    if(dp) *dp = *(double*)ii;
+    /* use memcpy avoid type punning */
+    if(dp) memcpy(dp, ii, sizeof(double));
 }
 
 void
